@@ -1,3 +1,27 @@
+/**
+ * Copyright (c) 2001-2004 artofcode LLC.
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use, copy,
+ * modify, merge, publish, distribute, sublicense, and/or sell copies
+ * of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
+**/
+
 #include <stdlib.h>
 #include <stdio.h>
 #include "unistd_.h"
@@ -529,7 +553,7 @@ ijs_server_set_param (IjsServerCtx *ctx, IjsJobId job_id, const char *key,
     {
       int size = value_size;
 
-      if (size < (int)sizeof(ctx->ph->cs) - 1)
+      if (size > (int)sizeof(ctx->ph->cs) - 1)
 	size = sizeof(ctx->ph->cs) - 1;
       memcpy (ctx->ph->cs, value, size);
       ctx->ph->cs[size] = 0;
