@@ -1,23 +1,17 @@
-/* Copyright (C) 1999, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-// $RCSfile: dwinst.cpp,v $ $Revision: 1.2.2.1 $
+// $Id: dwinst.cpp 6651 2006-03-13 16:18:19Z stefan $
 
 #define STRICT
 #include <windows.h>
@@ -247,6 +241,16 @@ BOOL CInstall::InstallFiles(BOOL bNoCopy, BOOL *pbQuit)
 }
 
 
+void CInstall::AppendFileNew(const char *filename)
+{
+    FILE *f;
+    /* mark backup file for uninstall */
+    if ((f = fopen(m_szFileNew, "a")) != (FILE *)NULL) {
+	fputs(filename, f);
+	fputs("\n", f);
+	fclose(f);
+    }
+}
 
 // recursive mkdir
 // requires a full path to be specified, so ignores root \ 
@@ -308,6 +312,13 @@ BOOL CInstall::MakeDir(const char *dirname)
     return SetCurrentDirectory(dirname);
 }
 
+void CInstall::ResetReadonly(const char *filename)
+{
+    DWORD dwAttr = GetFileAttributes(filename);
+    if (dwAttr & FILE_ATTRIBUTE_READONLY)
+        SetFileAttributes(filename, dwAttr & (~FILE_ATTRIBUTE_READONLY));
+}
+
 BOOL CInstall::InstallFile(char *filename, BOOL bNoCopy)
 {
 	char existing_name[MAXSTR];
@@ -355,6 +366,7 @@ BOOL CInstall::InstallFile(char *filename, BOOL bNoCopy)
 			AddMessage(message);
 			return FALSE;
 		}
+		ResetReadonly(new_name);
 		fputs(new_name, m_fLogNew);
 		fputs("\n", m_fLogNew);
 	}
@@ -729,6 +741,7 @@ BOOL CInstall::WriteUninstall(const char *szProg, BOOL bNoCopy)
 		AddMessage(message);
 		return FALSE;
 	}
+	ResetReadonly(ungsprog);
 	
 	/* write registry entries for uninstall */
 	if ((rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, UNINSTALLKEY, 0, 
