@@ -1,20 +1,16 @@
-/* Copyright (C) 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$Id: gdevcljc.c,v 1.4.2.1 2002/02/22 19:45:56 ray Exp $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevcljc.c 6651 2006-03-13 16:18:19Z stefan $ */
 /*
  * H-P Color LaserJet 5/5M contone device; based on the gdevclj.c.
  */
@@ -96,6 +92,6 @@ const gx_device_printer gs_cljet5c_device =
 		    85, 110, X_DPI, Y_DPI,
 		    0.167, 0.167,
 		    0.167, 0.167,
-		    3, 24, 255, 255, 5, 5,
+		    3, 24, 255, 255, 256, 256,
 		    cljc_print_page)
 };
