@@ -1,20 +1,16 @@
-/* Copyright (C) 1989-1994, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$Id: gdevlxm.c,v 1.2.6.1 2002/02/22 19:45:56 ray Exp $*/
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevlxm.c 6651 2006-03-13 16:18:19Z stefan $*/
 /*
  * Lexmark 5700 ink-jet printer driver for Ghostscript
  *
@@ -154,18 +150,18 @@ lxm5700m_print_page(gx_device_printer *pdev, FILE *prn_stream)
     /* Note that in_size is a multiple of 8. */
     int in_size = line_size * (swipeHeight);
     int swipeBuf_size = in_size;
-    byte *buf1 = (byte *)gs_malloc(in_size, 1, "lxm_print_page(buf1)");
+    byte *buf1 = (byte *)gs_malloc(pdev->memory, in_size, 1, "lxm_print_page(buf1)");
     byte *swipeBuf =
-	(byte *)gs_malloc(swipeBuf_size, 1, "lxm_print_page(swipeBuf)");
+	(byte *)gs_malloc(pdev->memory, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");
     byte *in = buf1;
 
     /* Check allocations */
     if ( buf1 == 0 || swipeBuf == 0 ) {
 	if ( buf1 ) 
 quit_ignomiously: /* and a goto into an if statement is pretty ignomious! */
-	gs_free((char *)buf1, in_size, 1, "lxm_print_page(buf1)");
+	gs_free(pdev->memory, (char *)buf1, in_size, 1, "lxm_print_page(buf1)");
 	if ( swipeBuf ) 
-	    gs_free((char *)swipeBuf, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");
+	    gs_free(pdev->memory, (char *)swipeBuf, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");
 	return_error(gs_error_VMerror);
     }
 
@@ -250,9 +246,9 @@ quit_ignomiously: /* and a goto into an if statement is pretty ignomious! */
 
 	/* macro, not fcn call.  Space penalty is modest, speed helps */
 #define buffer_store(x) if(outp-swipeBuf>=swipeBuf_size) {\
-	    gs_free((char *)swipeBuf, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");\
+	    gs_free(pdev->memory, (char *)swipeBuf, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");\
 	    swipeBuf_size*=2;\
-	    swipeBuf = (byte *)gs_malloc(swipeBuf_size, 1, "lxm_print_page(swipeBuf)");\
+	    swipeBuf = (byte *)gs_malloc(pdev->memory, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");\
 	    if (swipeBuf == 0) goto quit_ignomiously;\
 	    break;}\
 	else *outp++ = (x)
@@ -359,8 +355,8 @@ quit_ignomiously: /* and a goto into an if statement is pretty ignomious! */
     }
     fflush(prn_stream);
 
-    gs_free((char *)swipeBuf, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");
-    gs_free((char *)buf1, in_size, 1, "lxm_print_page(buf1)");
+    gs_free(pdev->memory, (char *)swipeBuf, swipeBuf_size, 1, "lxm_print_page(swipeBuf)");
+    gs_free(pdev->memory, (char *)buf1, in_size, 1, "lxm_print_page(buf1)");
     return 0;
 }
 
