@@ -1,23 +1,16 @@
-/* Copyright (C) 1993, 1994, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$RCSfile: gdevmpla.c,v $ $Revision: 1.2.2.1 $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevmpla.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Any-depth planar "memory" (stored bitmap) device */
 #include "memory_.h"
 #include "gx.h"
@@ -261,7 +254,7 @@ mem_planar_copy_color(gx_device * dev, const byte * base, int sourcex,
 		    for (ix = 0; ix < cw; ++ix) {
 			gx_color_index value;
 
-			sample_load_next32(value, sptr, sbit, source_depth);
+			sample_load_next_any(value, sptr, sbit, source_depth);
 			value = (value >> shift) & mask;
 			sample_store_next16(value, dptr, dbit, plane_depth,
 					    dbbyte);
@@ -423,7 +416,7 @@ planar_to_chunky(gx_device_memory *mdev, int x, int y, int w, int h,
 		sample_load_next16(value, sptr[pi], sbit[pi], plane_depth);
 		color |= (gx_color_index)value << mdev->planes[pi].shift;
 	    }
-	    sample_store_next32(color, dptr, dbit, ddepth, dbbyte);
+	    sample_store_next_any(color, dptr, dbit, ddepth, dbbyte);
 	}
 	sample_store_flush(dptr, dbit, ddepth, dbbyte);
     }
