@@ -1,20 +1,16 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$Id: gdevmrop.h,v 1.3.6.1 2002/02/22 19:45:56 ray Exp $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevmrop.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Definitions for device RasterOp implementations. */
 /* Requires gxdevmem.h, gsropt.h */
 
@@ -25,17 +21,17 @@
  * Compute the effective RasterOp for the 1-bit case,
  * taking transparency into account.
  */
-gs_rop3_t gs_transparent_rop(P1(gs_logical_operation_t lop));
+gs_rop3_t gs_transparent_rop(gs_logical_operation_t lop);
 
 #ifdef DEBUG
 /* Trace a [strip_]copy_rop call. */
-void trace_copy_rop(P16(const char *cname, gx_device * dev,
-			const byte * sdata, int sourcex, uint sraster,
-			gx_bitmap_id id, const gx_color_index * scolors,
-			const gx_strip_bitmap * textures,
-			const gx_color_index * tcolors,
-			int x, int y, int width, int height,
-			int phase_x, int phase_y, gs_logical_operation_t lop));
+void trace_copy_rop(const char *cname, gx_device * dev,
+		    const byte * sdata, int sourcex, uint sraster,
+		    gx_bitmap_id id, const gx_color_index * scolors,
+		    const gx_strip_bitmap * textures,
+		    const gx_color_index * tcolors,
+		    int x, int y, int width, int height,
+		    int phase_x, int phase_y, gs_logical_operation_t lop);
 #endif
 
 /*
@@ -70,14 +66,14 @@ struct gx_device_rop_texture_s {
     gx_device_finalize)
 
 /* Create a RasterOp source device. */
-int gx_alloc_rop_texture_device(P3(gx_device_rop_texture ** prsdev,
-				   gs_memory_t * mem,
-				   client_name_t cname));
+int gx_alloc_rop_texture_device(gx_device_rop_texture ** prsdev,
+				gs_memory_t * mem,
+				client_name_t cname);
 
 /* Initialize a RasterOp source device. */
-void gx_make_rop_texture_device(P4(gx_device_rop_texture * rsdev,
-				   gx_device * target,
-				   gs_logical_operation_t lop,
-				   const gx_device_color * texture));
+void gx_make_rop_texture_device(gx_device_rop_texture * rsdev,
+				gx_device * target,
+				gs_logical_operation_t lop,
+				const gx_device_color * texture);
 
 #endif /* gdevmrop_INCLUDED */
