@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfj.c,v 1.49 2005/09/06 13:47:10 leonardo Exp $ */
+/* $Id: gdevpdfj.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Image-writing utilities for pdfwrite driver */
 #include "memory_.h"
 #include "string_.h"
@@ -134,8 +131,12 @@ pdf_put_pixel_image_values(cos_dict_t *pcd, gx_device_pdf *pdev,
 					    COS_OBJECT(pca)));
 	}
     }
-    if (pim->Interpolate)
-	CHECK(cos_dict_put_c_strings(pcd, pin->Interpolate, "true"));
+    if (pim->Interpolate) {
+	if (pdev->PDFA)
+	    eprintf("PDFA doesn't allow images with Interpolate true.\n");
+	else
+	    CHECK(cos_dict_put_c_strings(pcd, pin->Interpolate, "true"));
+    }
     return 0;
 }
 int
@@ -261,7 +262,7 @@ pdf_do_image_by_id(gx_device_pdf * pdev, double scale,
     if (pimat)
 	pdf_put_image_matrix(pdev, pimat, scale);
     pprintld1(pdev->strm, "/R%ld Do\nQ\n", id);
-    return pdf_register_charproc_resource(pdev, id, resourceXObject);
+    return 0;
 }
 int
 pdf_do_image(gx_device_pdf * pdev, const pdf_resource_t * pres,
