@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfv.c,v 1.38 2005/02/16 09:13:06 igor Exp $ */
+/* $Id: gdevpdfv.c 6723 2006-04-27 17:36:41Z leonardo $ */
 /* Color value writing for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -370,7 +367,7 @@ pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	 */
 	m_tile = pdc->mask.m_tile;
 	if (m_tile) {
-	    if (p_tile && !(p_tile->depth & 7) && p_tile->depth <= sizeof(gx_color_index) * 8) {
+	    if (p_tile && !(p_tile->depth & 7) && p_tile->depth <= arch_sizeof_color_index * 8) {
 		int depth_bytes = p_tile->depth >> 3;
 		int width = p_tile->tbits.rep_width;
 		int skip = p_tile->tbits.raster -
@@ -444,10 +441,11 @@ pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 		return code;
 	}
 	pdf_image_writer_init(&writer);
+	pdev->ParamCompatibilityLevel = pdev->CompatibilityLevel;
 	if ((code = pdf_begin_write_image(pdev, &writer, gs_no_id, w, h, NULL, false)) < 0 ||
 	    (code = psdf_setup_lossless_filters((gx_device_psdf *)pdev,
 						&writer.binary[0],
-						(gs_pixel_image_t *)&image)) < 0 ||
+						(gs_pixel_image_t *)&image, false)) < 0 ||
 	    (code = pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image, &cs_value, 0)) < 0
 	    )
 	    return code;
@@ -482,7 +480,7 @@ pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 /* Write parameters common to all Shadings. */
 private int
 pdf_put_shading_common(cos_dict_t *pscd, const gs_shading_t *psh,
-		       const gs_range_t **ppranges)
+		       bool shfill, const gs_range_t **ppranges)
 {
     gs_shading_type_t type = ShadingType(psh);
     const gs_color_space *pcs = psh->params.ColorSpace;
@@ -497,7 +495,7 @@ pdf_put_shading_common(cos_dict_t *pscd, const gs_shading_t *psh,
 	(code = cos_dict_put_c_key(pscd, "/ColorSpace", &cs_value)) < 0
 	)
 	return code;
-    if (psh->params.Background) {
+    if (psh->params.Background && !shfill) {
 	/****** SCALE Background ******/
 	code = cos_dict_put_c_key_floats(pscd, "/Background",
 				   psh->params.Background->paint.values,
@@ -911,12 +909,12 @@ pdf_put_pattern2(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	/* Shading has an associated data stream. */
 	cos_become(psco, cos_type_stream);
 	code = pdf_put_shading_common(cos_stream_dict((cos_stream_t *)psco),
-				      psh, &pranges);
+				      psh, pinst->shfill, &pranges);
 	if (code >= 0)
 	    code1 = pdf_put_mesh_shading((cos_stream_t *)psco, psh, pranges);
     } else {
 	cos_become(psco, cos_type_dict);
-	code = pdf_put_shading_common((cos_dict_t *)psco, psh, &pranges);
+	code = pdf_put_shading_common((cos_dict_t *)psco, psh, pinst->shfill, &pranges);
 	if (code >= 0)
 	    code = pdf_put_scalar_shading((cos_dict_t *)psco, psh, pranges);
     }
