@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtb.c,v 1.35 2005/04/04 20:28:27 igor Exp $ */
+/* $Id: gdevpdtb.c 6669 2006-03-21 12:09:23Z leonardo $ */
 /* BaseFont implementation for pdfwrite */
 #include "memory_.h"
 #include "ctype_.h"
@@ -30,7 +27,9 @@
 #include "gdevpdfx.h"
 #include "gdevpdfo.h"
 #include "gdevpdtb.h"
+#include "gdevpdfg.h"
 #include "gdevpdtf.h"
+#include "smd5.h"
 
 /*
  * Adobe's Distiller Parameters documentation for Acrobat Distiller 5
@@ -41,52 +40,10 @@
  */
 #define MAX_NO_SUBSET_GLYPHS 500	/* arbitrary */
 
-/* ---------------- Definitions ---------------- */
-
-struct pdf_base_font_s {
-    /*
-     * For the standard 14 fonts, copied == complete is a complete copy
-     * of the font, and DO_SUBSET = NO.
-     *
-     * For fonts that MAY be subsetted, copied is a partial copy,
-     * complete is a complete copy, and DO_SUBSET = UNKNOWN until
-     * pdf_font_do_subset is called.
-     *
-     * For fonts that MUST be subsetted, copied == complete is a partial
-     * copy, and DO_SUBSET = YES.
-     */
-    gs_font_base *copied;
-    gs_font_base *complete;
-    enum {
-	DO_SUBSET_UNKNOWN = 0,
-	DO_SUBSET_NO,
-	DO_SUBSET_YES
-    } do_subset;
-    bool is_standard;
-    /*
-     * For CIDFonts, which are always subsetted, num_glyphs is CIDCount.
-     * For optionally subsetted fonts, num_glyphs is the count of glyphs
-     * in the font when originally copied.  Note that if the font is
-     * downloaded incrementally, num_glyphs may be 0.
-     */
-    int num_glyphs;
-    byte *CIDSet;		/* for CIDFonts */
-    gs_string font_name;
-    bool written;
-    cos_dict_t *FontFile;
-};
-BASIC_PTRS(pdf_base_font_ptrs) {
-    GC_OBJ_ELT(pdf_base_font_t, copied),
-    GC_OBJ_ELT(pdf_base_font_t, complete),
-    GC_OBJ_ELT(pdf_base_font_t, CIDSet),
-    GC_OBJ_ELT(pdf_base_font_t, FontFile),
-    GC_STRING_ELT(pdf_base_font_t, font_name)
-};
-gs_private_st_basic(st_pdf_base_font, pdf_base_font_t, "pdf_base_font_t",
-		    pdf_base_font_ptrs, pdf_base_font_data);
-
 /* ---------------- Private ---------------- */
 
+private_st_pdf_base_font();
+
 #define SUBSET_PREFIX_SIZE 7	/* XXXXXX+ */
 
 /*
@@ -145,32 +102,6 @@ pdf_add_subset_prefix(const gx_device_pdf *pdev, gs_string *pstr, byte *used, in
     return 0;
 }
 
-/* Begin writing FontFile* data. */
-private int
-pdf_begin_fontfile(gx_device_pdf *pdev, long FontFile_id,
-		   const char *entries, long len1, pdf_data_writer_t *pdw)
-{
-    int code;
-    code = pdf_begin_data_stream(pdev, pdw, DATA_STREAM_BINARY | DATA_STREAM_ENCRYPT |
-				 (pdev->CompressFonts ?
-				  DATA_STREAM_COMPRESS : 0), FontFile_id);
-    if (len1 >= 0) {
-	code = cos_dict_put_c_key_int((cos_dict_t *)pdw->pres->object, "/Length1", len1);
-	if (code < 0)
-	    return code;
-    }
-    if (entries != NULL) {
-	const char *p = strchr(entries + 1, '/');
-
-	code = cos_dict_put_string((cos_dict_t *)pdw->pres->object, 
-		    (const byte *)entries, p - entries,
-		    (const byte *)p, strlen(p));
-	if (code < 0)
-	    return code;
-    }
-    return code;
-}
-
 /* Finish writing FontFile* data. */
 private int
 pdf_end_fontfile(gx_device_pdf *pdev, pdf_data_writer_t *pdw)
@@ -262,10 +193,14 @@ pdf_base_font_alloc(gx_device_pdf *pdev, pdf_base_font_t **ppbfont,
 	/* The only possibly non-subsetted fonts are Type 1/2 and Type 42. */
 	if (is_standard)
 	    complete = copied, code = 0;
-	else
+	else {
 	    code = gs_copy_font((gs_font *)font, &font->FontMatrix, mem, &complete);
-	if (code >= 0)
-	    code = gs_copy_font_complete((gs_font *)font, complete);
+	    if (code < 0)
+		goto fail;
+	}
+        code = gs_copy_font_complete((gs_font *)font, complete);
+	if (code < 0)
+	    goto fail;
 	if (pbfont->num_glyphs < 0) { /* Type 1 */
 	    int index, count;
 	    gs_glyph glyph;
@@ -501,18 +436,30 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
     bool do_subset = pdf_do_subset_font(pdev, pbfont, rid);
     gs_font_base *out_font =
 	(do_subset || pbfont->complete == NULL ? pbfont->copied : pbfont->complete);
-    long FontFile_id;
     gs_const_string fnstr;
     pdf_data_writer_t writer;
+    byte digest[6] = {0,0,0,0,0,0};
     int code;
 
     if (pbfont->written)
 	return 0;		/* already written */
-    FontFile_id = -1; /* A stub for old code compatibility. */
-    /* We write the font data to a delayed stream
-       to provide it to be written after the font descriptor 
-       for ps2write needs.
-     */
+    code = copied_order_font((gs_font *)out_font);
+    if (code < 0)
+	return code;
+    code = pdf_begin_data_stream(pdev, &writer, DATA_STREAM_BINARY | 
+			    /* Don't set DATA_STREAM_ENCRYPT since we write to a temporary file.
+			       See comment in pdf_begin_encrypt. */
+				 (pdev->CompressFonts ?
+				  DATA_STREAM_COMPRESS : 0), 0);
+    if (code < 0)
+	return code;
+    if (pdev->PDFA) {
+	stream *s = s_MD5C_make_stream(pdev->pdf_memory, writer.binary.strm);
+
+	if (s == NULL)
+	    return_error(gs_error_VMerror);
+	writer.binary.strm = s;
+    }
     if (pdev->CompatibilityLevel == 1.2 &&
 	!do_subset && !pbfont->is_standard ) {
 	/*
@@ -523,7 +470,7 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
 	 * Note that subsetted fonts already have an unique name
 	 * due to subset prefix.
 	 */
-	 int code = pdf_adjust_font_name(pdev, FontFile_id, pbfont);
+	 int code = pdf_adjust_font_name(pdev, writer.pres->object->id, pbfont);
 	 if (code < 0)
 	    return code;
     }
@@ -552,10 +499,6 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
 	    /* Write the type 1 font with no converting to CFF. */
 	    int lengths[3];
 
-	    code = pdf_begin_fontfile(pdev, FontFile_id, NULL, -1L,
-					&writer);
-	    if (code < 0)
-		return code;
 	    code = psf_write_type1_font(writer.binary.strm,
 				(gs_font_type1 *)out_font,
 				WRITE_TYPE1_WITH_LENIV |
@@ -585,8 +528,7 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
 	     * them.  Also remove lenIV, so Type 2 fonts will compress better.
 	     */
 #define TYPE2_OPTIONS (WRITE_TYPE2_NO_LENIV | WRITE_TYPE2_CHARSTRINGS)
-	    code = pdf_begin_fontfile(pdev, FontFile_id, "/Subtype/Type1C", -1L,
-				      &writer);
+	    code = cos_dict_put_string_copy((cos_dict_t *)writer.pres->object, "/Subtype", "/Type1C");
 	    if (code < 0)
 		return code;
 	    code = psf_write_type2_font(writer.binary.strm,
@@ -621,8 +563,9 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
 	code = psf_write_truetype_font(&poss, pfont, options, NULL, 0, &fnstr);
 	if (code < 0)
 	    return code;
-	code = pdf_begin_fontfile(pdev, FontFile_id, NULL, stell(&poss),
-				  &writer);
+	code = cos_dict_put_c_key_int((cos_dict_t *)writer.pres->object, "/Length1", stell(&poss));
+	if (code < 0)
+	    return code;
 	if (code < 0)
 	    return code;
 	code = psf_write_truetype_font(writer.binary.strm, pfont,
@@ -631,8 +574,7 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
     }
 
     case ft_CID_encrypted:
-	code = pdf_begin_fontfile(pdev, FontFile_id, "/Subtype/CIDFontType0C",
-				  -1L, &writer);
+	code = cos_dict_put_string_copy((cos_dict_t *)writer.pres->object, "/Subtype", "/CIDFontType0C");
 	if (code < 0)
 	    return code;
 	code = psf_write_cid0_font(writer.binary.strm,
@@ -643,20 +585,31 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
     case ft_CID_TrueType:
 	/* CIDFontType 2 fonts don't use cmap, name, OS/2, or post. */
 #define CID2_OPTIONS WRITE_TRUETYPE_HVMTX
-	code = pdf_begin_fontfile(pdev, FontFile_id, NULL, -1L, &writer);
-	if (code < 0)
-	    return code;
 	code = psf_write_cid2_font(writer.binary.strm,
 				   (gs_font_cid2 *)out_font,
 				   CID2_OPTIONS, NULL, 0, &fnstr);
     finish:
-	pdf_reserve_object_id(pdev, writer.pres, gs_no_id);
+	if (pdev->PDFA) {
+	    sflush(writer.binary.strm);
+	    s_MD5C_get_digest(writer.binary.strm, digest, sizeof(digest));
+	}
 	*ppcd = (cos_dict_t *)writer.pres->object;
 	if (code < 0) {
 	    pdf_end_fontfile(pdev, &writer);
 	    return code;
 	}
 	code = pdf_end_fontfile(pdev, &writer);
+	if (pdev->PDFA && code >= 0) {
+	    gs_id metadata_object_id;
+
+	    code = pdf_font_metadata(pdev, pbfont, digest, sizeof(digest), &metadata_object_id);
+	    if (metadata_object_id && code >= 0) {
+		char buf[20];
+
+		sprintf(buf, "%ld 0 R", metadata_object_id);
+		code = cos_dict_put_string_copy(*ppcd, "/Metadata", buf);
+	    }
+	}
 	break;
 
     default:
