@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtf.c,v 1.44 2005/04/05 09:48:46 igor Exp $ */
+/* $Id: gdevpdtf.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Font and CMap resource implementation for pdfwrite text */
 #include "memory_.h"
 #include "string_.h"
@@ -27,11 +24,13 @@
 #include "gxfcopy.h"
 #include "gxfont.h"
 #include "gxfont1.h"
+#include "gdevpsf.h"
 #include "gdevpdfx.h"
 #include "gdevpdtb.h"
 #include "gdevpdtd.h"
 #include "gdevpdtf.h"
 #include "gdevpdtw.h"
+#include "gdevpdti.h"
 
 /* GC descriptors */
 public_st_pdf_font_resource();
@@ -58,6 +57,7 @@ case 7: switch (pdfont->FontType) {
  case ft_CID_TrueType:
      ENUM_RETURN(pdfont->u.cidfont.Widths2);
  default:
+     pdf_mark_glyph_names(pdfont, mem);
      ENUM_RETURN(pdfont->u.simple.Encoding);
 }
 case 8: switch (pdfont->FontType) {
@@ -95,7 +95,7 @@ case 10: switch (pdfont->FontType) {
 }
 case 11: switch (pdfont->FontType) {
  case ft_user_defined:
-     ENUM_RETURN(pdfont->u.simple.s.type3.used_resources);
+     ENUM_RETURN(pdfont->u.simple.s.type3.Resources);
  case ft_CID_encrypted:
  case ft_CID_TrueType:
      ENUM_RETURN(pdfont->u.cidfont.used2);
@@ -125,7 +125,7 @@ RELOC_PTRS_WITH(pdf_font_resource_reloc_ptrs, pdf_font_resource_t *pdfont)
 	RELOC_VAR(pdfont->u.simple.v);
 	RELOC_VAR(pdfont->u.simple.s.type3.char_procs);
 	RELOC_VAR(pdfont->u.simple.s.type3.cached);
-	RELOC_VAR(pdfont->u.simple.s.type3.used_resources);
+	RELOC_VAR(pdfont->u.simple.s.type3.Resources);
 	break;
     case ft_CID_encrypted:
     case ft_CID_TrueType:
@@ -328,7 +328,8 @@ pdf_clean_standard_fonts(const gx_device_pdf *pdev)
 /* ------ Private ------ */
 
 
-private int pdf_resize_array(gs_memory_t *mem, void **p, int elem_size, int old_size, int new_size)
+private int 
+pdf_resize_array(gs_memory_t *mem, void **p, int elem_size, int old_size, int new_size)
 {
     void *q = gs_alloc_byte_array(mem, new_size, elem_size, "pdf_resize_array");
 
@@ -375,7 +376,7 @@ font_resource_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 	    memset(widths, 0, chars_count * sizeof(*widths));
 	memset(used, 0, size);
     }
-    code = pdf_alloc_resource(pdev, rtype, rid, (pdf_resource_t **)&pfres, 0L);
+    code = pdf_alloc_resource(pdev, rtype, rid, (pdf_resource_t **)&pfres, -1L);
     if (code < 0)
 	goto fail;
     memset((byte *)pfres + sizeof(pdf_resource_t), 0,
@@ -387,6 +388,8 @@ font_resource_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
     pfres->write_contents = write_contents;
     pfres->res_ToUnicode = NULL;
     pfres->cmap_ToUnicode = NULL;
+    pfres->mark_glyph = 0;
+    pfres->mark_glyph_data = 0;
     *ppfres = pfres;
     return 0;
  fail:
@@ -394,6 +397,29 @@ font_resource_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
     gs_free_object(mem, widths, "font_resource_alloc(Widths)");
     return code;
 }
+
+int
+pdf_assign_font_object_id(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
+{
+    if (pdf_resource_id((pdf_resource_t *)pdfont) == -1) {
+	int code;
+
+	pdf_reserve_object_id(pdev, (pdf_resource_t *)pdfont, 0);
+	code = pdf_mark_font_descriptor_used(pdev, pdfont->FontDescriptor);
+	if (code < 0)
+	    return code;
+	if (pdfont->FontType == 0) {
+	    pdf_font_resource_t *pdfont1 = pdfont->u.type0.DescendantFont;
+
+	    pdf_reserve_object_id(pdev, (pdf_resource_t *)pdfont1, 0);
+	    code = pdf_mark_font_descriptor_used(pdev, pdfont1->FontDescriptor);
+	    if (code < 0)
+		return code;
+	}
+    }
+    return 0;
+}
+
 private int
 font_resource_simple_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 			   gs_id rid, font_type ftype, int chars_count,
@@ -599,6 +625,29 @@ embed_as_standard(gx_device_pdf *pdev, gs_font *font, int index,
     return (find_std_appearance(pdev, (gs_font_base *)font, -1,
 				pairs, num_glyphs) == index);
 }
+private bool
+has_extension_glyphs(gs_font *pfont)
+{
+    psf_glyph_enum_t genum;
+    gs_glyph glyph;
+    gs_const_string str;
+    int code, j, l;
+    const int sl = strlen(gx_extendeg_glyph_name_separator);
+
+    psf_enumerate_glyphs_begin(&genum, (gs_font *)pfont, NULL, 0, GLYPH_SPACE_NAME);
+    for (glyph = gs_no_glyph; (code = psf_enumerate_glyphs_next(&genum, &glyph)) != 1; ) {
+	code = pfont->procs.glyph_name(pfont, glyph, &str);
+	if (code < 0)
+	    return code;
+	l = str.size - sl, j;
+	for (j = 0; j < l; j ++)
+	    if (!memcmp(gx_extendeg_glyph_name_separator, str.data + j, sl))
+		return true;
+    }
+    psf_enumerate_glyphs_reset(&genum);
+    return false;
+}
+
 /*
  * Choose a name for embedded font.
  */
@@ -627,11 +676,16 @@ pdf_font_embed_status(gx_device_pdf *pdev, gs_font *font, int *pindex,
      */
     if (pindex)
 	*pindex = index;
+    if (pdev->PDFX)
+	return FONT_EMBED_YES;
     if (pdev->CompatibilityLevel < 1.3) {
 	if (index >= 0 && 
 	    (embed_as_standard_called = true,
-	     do_embed_as_standard = embed_as_standard(pdev, font, index, pairs, num_glyphs)))
+		do_embed_as_standard = embed_as_standard(pdev, font, index, pairs, num_glyphs))) {
+	    if (pdev->ForOPDFRead && has_extension_glyphs(font))
+		return FONT_EMBED_YES;
 	    return FONT_EMBED_STANDARD;
+	}
     }
     /* Check the Embed lists. */
     if (!embed_list_includes(&pdev->params.NeverEmbed, chars, size) ||
@@ -794,6 +848,7 @@ pdf_font_std_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 	return code;
     pdfont->BaseFont.data = (byte *)psfi->fname; /* break const */
     pdfont->BaseFont.size = strlen(psfi->fname);
+    pdfont->mark_glyph = pfont->dir->ccache.mark_glyph;
     set_is_MM_instance(pdfont, pfont);
     if (is_original) {
 	psf->pdfont = pdfont;
