@@ -1,23 +1,21 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevps.c,v 1.19.2.2 2002/03/13 17:09:11 igor Exp $ */
+/* $Id: gdevps.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* PostScript-writing driver */
 #include "math_.h"
 #include "memory_.h"
+#include "string_.h"
 #include "time_.h"
 #include "gx.h"
 #include "gserrors.h"
@@ -40,6 +38,12 @@
 /* Current ProcSet version */
 #define PSWRITE_PROCSET_VERSION 1
 
+/* Guaranteed size of operand stack accoeding to PLRM */
+#define MAXOPSTACK 500
+
+private int psw_open_printer(gx_device * dev);
+private int psw_close_printer(gx_device * dev);
+
 /* ---------------- Device definition ---------------- */
 
 /* Device procedures */
@@ -112,7 +116,7 @@ gs_private_st_suffix_add1_final(st_device_pswrite, gx_device_pswrite,
 		psw_copy_mono,\
 		psw_copy_color,\
 		NULL,			/* draw_line */\
-		NULL,			/* get_bits */\
+		psdf_get_bits,\
 		psw_get_params,\
 		psw_put_params,\
 		NULL,			/* map_cmyk_color */\
@@ -135,7 +139,12 @@ gs_private_st_suffix_add1_final(st_device_pswrite, gx_device_pswrite,
 		NULL,			/* image_data */\
 		NULL,			/* end_image */\
 		NULL,			/* strip_tile_rectangle */\
-		NULL/******psw_strip_copy_rop******/\
+		NULL,/******psw_strip_copy_rop******/\
+		NULL,			/* get_clipping_box */\
+		NULL,			/* begin_typed_image */\
+		psdf_get_bits_rectangle,\
+		NULL,			/* map_color_rgb_alpha */\
+		psdf_create_compositor\
 	}
 
 const gx_device_pswrite gs_pswrite_device = {
@@ -166,21 +175,24 @@ const gx_device_pswrite gs_epswrite_device = {
 
 /* Vector device implementation */
 private int
-    psw_beginpage(P1(gx_device_vector * vdev)),
-    psw_setcolors(P2(gx_device_vector * vdev, const gx_drawing_color * pdc)),
-    psw_dorect(P6(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1,
-		  fixed y1, gx_path_type_t type)),
-    psw_beginpath(P2(gx_device_vector * vdev, gx_path_type_t type)),
-    psw_moveto(P6(gx_device_vector * vdev, floatp x0, floatp y0,
-		  floatp x, floatp y, gx_path_type_t type)),
-    psw_lineto(P6(gx_device_vector * vdev, floatp x0, floatp y0,
-		  floatp x, floatp y, gx_path_type_t type)),
-    psw_curveto(P10(gx_device_vector * vdev, floatp x0, floatp y0,
-		    floatp x1, floatp y1, floatp x2, floatp y2,
-		    floatp x3, floatp y3, gx_path_type_t type)),
-    psw_closepath(P6(gx_device_vector * vdev, floatp x0, floatp y0,
-		     floatp x_start, floatp y_start, gx_path_type_t type)),
-    psw_endpath(P2(gx_device_vector * vdev, gx_path_type_t type));
+    psw_beginpage(gx_device_vector * vdev),
+    psw_can_handle_hl_color(gx_device_vector * vdev, const gs_imager_state * pis, 
+                  const gx_drawing_color * pdc),
+    psw_setcolors(gx_device_vector * vdev, const gs_imager_state * pis, 
+                  const gx_drawing_color * pdc),
+    psw_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1,
+	       fixed y1, gx_path_type_t type),
+    psw_beginpath(gx_device_vector * vdev, gx_path_type_t type),
+    psw_moveto(gx_device_vector * vdev, floatp x0, floatp y0,
+	       floatp x, floatp y, gx_path_type_t type),
+    psw_lineto(gx_device_vector * vdev, floatp x0, floatp y0,
+	       floatp x, floatp y, gx_path_type_t type),
+    psw_curveto(gx_device_vector * vdev, floatp x0, floatp y0,
+		floatp x1, floatp y1, floatp x2, floatp y2,
+		floatp x3, floatp y3, gx_path_type_t type),
+    psw_closepath(gx_device_vector * vdev, floatp x0, floatp y0,
+		  floatp x_start, floatp y_start, gx_path_type_t type),
+    psw_endpath(gx_device_vector * vdev, gx_path_type_t type);
 private const gx_device_vector_procs psw_vector_procs = {
 	/* Page management */
     psw_beginpage,
@@ -193,6 +205,7 @@ private const gx_device_vector_procs psw_vector_procs = {
     psdf_setflat,
     psdf_setlogop,
 	/* Other state */
+    psw_can_handle_hl_color,
     psw_setcolors,		/* fill & stroke colors are the same */
     psw_setcolors,
 	/* Paths */
@@ -243,7 +256,7 @@ private const char *const psw_procset[] = {
     "/q/gsave #/Q/grestore #/rf{re fill}!",
     "/Y{P clip newpath}!/Y*{P eoclip newpath}!/rY{re Y}!",
 	/* <w> <h> <name> <data> <?> |= <w> <h> <data> */
-    "/|={pop exch 4 1 roll 3 array astore cvx exch 1 index def exec}!",
+    "/|={pop exch 4 1 roll 1 array astore cvx 3 array astore cvx exch 1 index def exec}!",
 	/* <w> <h> <name> <length> <src> | <w> <h> <data> */
     "/|{exch string readstring |=}!",
 	/* <w> <?> <name> (<length>|) + <w> <?> <name> <length> */
@@ -302,7 +315,7 @@ private const char *const psw_2_procset[] = {
     "/IC{3 1 roll 10 dict begin 1{/ImageType/Interpolate/Decode/DataSource",
     "/ImageMatrix/BitsPerComponent/Height/Width}{exch def}forall",
     "currentdict end image}!",
-    /* A hack for compatibility with interpreters, which don't consume
+    /* A hack for compatibility with interpreters, which don't consume 
        ASCII85Decode EOD when reader stops immediately before it : */
     "/~{@ read {pop} if}!",
     0
@@ -317,23 +330,32 @@ private const char *const psw_2_procset[] = {
 private int
 psw_begin_file(gx_device_pswrite *pdev, const gs_rect *pbbox)
 {
+    int code;
     FILE *f = pdev->file;
+    char const * const *p1;
+    char const * const *p2;
 
-    psw_begin_file_header(f, (gx_device *)pdev, pbbox,
-			  &pdev->pswrite_common,
-			  pdev->params.ASCII85EncodePages);
-    psw_print_lines(f, psw_procset);
     if (pdev->pswrite_common.LanguageLevel < 1.5) {
-	psw_print_lines(f, psw_1_x_procset);
-	psw_print_lines(f, psw_1_procset);
+	p1 = psw_1_x_procset;
+	p2 = psw_1_procset;
     } else if (pdev->pswrite_common.LanguageLevel > 1.5) {
-	psw_print_lines(f, psw_1_5_procset);
-	psw_print_lines(f, psw_2_procset);
+	p1 = psw_1_5_procset;
+	p2 = psw_2_procset;
     } else {
-	psw_print_lines(f, psw_1_x_procset);
-	psw_print_lines(f, psw_1_5_procset);
+	p1 = psw_1_x_procset;
+	p2 = psw_1_5_procset;
     }
-    psw_end_file_header(f);
+
+    if ((code = psw_begin_file_header(f, (gx_device *)pdev, pbbox,
+			  &pdev->pswrite_common,
+			  pdev->params.ASCII85EncodePages)) < 0 ||
+        (code = psw_print_lines(f, psw_procset)) < 0 ||
+        (code = psw_print_lines(f, p1)) < 0 ||
+        (code = psw_print_lines(f, p2)) < 0 ||
+        (code = psw_end_file_header(f)) < 0)
+        return code;
+    if(fflush(f) == EOF)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -418,17 +440,20 @@ psw_image_stream_setup(gx_device_pswrite * pdev, bool binary_ok)
 }
 
 /* Clean up after writing an image. */
-private void
+private int
 psw_image_cleanup(gx_device_pswrite * pdev)
 {
+    int code = 0;
+    
     if (pdev->image_stream != 0) {
-	psdf_end_binary(pdev->image_writer);
+	code = psdf_end_binary(pdev->image_writer);
 	memset(pdev->image_writer, 0, sizeof(*pdev->image_writer));
     }
+    return code;
 }
 
 /* Write data for an image.  Assumes width > 0, height > 0. */
-private void
+private int
 psw_put_bits(stream * s, const byte * data, int data_x_bit, uint raster,
 	     uint width_bits, int height)
 {
@@ -436,7 +461,7 @@ psw_put_bits(stream * s, const byte * data, int data_x_bit, uint raster,
     int shift = data_x_bit & 7;
     int y;
 
-    for (y = 0; y < height; ++y, row += raster)
+    for (y = 0; y < height; ++y, row += raster) {
 	if (shift == 0)
 	    stream_write(s, row, (width_bits + 7) >> 3);
 	else {
@@ -445,19 +470,27 @@ psw_put_bits(stream * s, const byte * data, int data_x_bit, uint raster,
 	    int cshift = 8 - shift;
 
 	    for (; wleft + shift > 8; ++src, wleft -= 8)
-		stream_putc(s, (*src << shift) + (src[1] >> cshift));
+		stream_putc(s, (byte)((*src << shift) + (src[1] >> cshift)));
 	    if (wleft > 0)
-		stream_putc(s, (*src << shift) & (byte)(0xff00 >> wleft));
+		stream_putc(s, (byte)((*src << shift) & (byte)(0xff00 >> wleft)));
 	}
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
+    }
+    return 0;
 }
 private int
 psw_put_image_bits(gx_device_pswrite *pdev, const char *op,
 		   const byte * data, int data_x, uint raster,
 		   int width, int height, int depth)
 {
+    int code;
+    
     pprints1(pdev->strm, "%s\n", op);
-    psw_put_bits(pdev->image_stream, data, data_x * depth, raster,
+    code = psw_put_bits(pdev->image_stream, data, data_x * depth, raster,
 		 width * depth, height);
+    if (code < 0)
+        return code;
     psw_image_cleanup(pdev);
     return 0;
 }
@@ -495,6 +528,8 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	sprintf(str, "%d%c", index / 26, index % 26 + 'A');
 	pprintd2(s, "%d %d ", x, y);
 	pprints2(s, "%s %s\n", str, imagestr);
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
 	return 0;
     }
     pprintd4(s, "%d %d %d %d ", x, y, width, height);
@@ -527,6 +562,8 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	op = cached[encode];
 	sprintf(endstr, "\n%s\n", imagestr);
     }
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     /*
      * In principle, we should put %%BeginData: / %%EndData around all data
      * sections.  However, as long as the data are ASCII (not binary), they
@@ -550,6 +587,7 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	 */
 	stream poss;
 
+	s_init(s, pdev->memory);
 	swrite_position_only(&poss);
 	pdev->strm = &poss;
 	code = psw_put_image(pdev, op, encode, data, data_x, raster,
@@ -565,6 +603,8 @@ psw_image_write(gx_device_pswrite * pdev, const char *imagestr,
 	stream_puts(s, "\n%%EndData");
     }
     stream_puts(s, endstr);
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -583,14 +623,18 @@ psw_check_erasepage(gx_device_pswrite *pdev)
     int code = 0;
 
     if (pdev->page_fill.color != gx_no_color_index) {
-	code = gdev_vector_fill_rectangle((gx_device *)pdev,
-					  pdev->page_fill.rect.p.x,
-					  pdev->page_fill.rect.p.y,
-					  pdev->page_fill.rect.q.x -
-					    pdev->page_fill.rect.p.x,
-					  pdev->page_fill.rect.q.y -
-					    pdev->page_fill.rect.p.y,
-					  pdev->page_fill.color);
+	if (pdev->page_fill.rect.p.x != pdev->page_fill.rect.q.x
+	    && pdev->page_fill.rect.p.y != pdev->page_fill.rect.q.y)
+	{
+	    code = gdev_vector_fill_rectangle((gx_device *)pdev,
+					      pdev->page_fill.rect.p.x,
+					      pdev->page_fill.rect.p.y,
+					      pdev->page_fill.rect.q.x -
+						pdev->page_fill.rect.p.x,
+					      pdev->page_fill.rect.q.y -
+						pdev->page_fill.rect.p.y,
+					      pdev->page_fill.color);
+	}
 	pdev->page_fill.color = gx_no_color_index;
     }
     return code;
@@ -610,6 +654,7 @@ psw_is_separate_pages(gx_device_vector *const vdev)
     const char *fmt;
     gs_parsed_file_name_t parsed;
     int code = gx_parse_output_file_name(&parsed, &fmt, vdev->fname, strlen(vdev->fname));
+    
     return (code >= 0 && fmt != 0);
 }
 
@@ -618,32 +663,48 @@ psw_is_separate_pages(gx_device_vector *const vdev)
 private int
 psw_beginpage(gx_device_vector * vdev)
 {
-    stream *s;
     gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
-    if (!vdev->is_open) {
-	int code = psw_open((gx_device *)vdev);
-	if (code < 0)
+    int code = psw_open_printer((gx_device *)vdev);
+
+    stream *s = vdev->strm;
+
+    if (code < 0)
+	 return code;
+
+    if (pdev->first_page) {
+	code = psw_begin_file(pdev, NULL);
+        if (code < 0)
 	     return code;
-	vdev->is_open = true;
     }
-    s = vdev->strm;
-    if (pdev->first_page)
-	psw_begin_file(pdev, NULL);
+
+    code = psw_write_page_header(s, (gx_device *)vdev, &pdev->pswrite_common, true, 
+                          (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount + 1),
+                          image_cache_size);
+    if (code < 0)
+	 return code;
 
-    psw_write_page_header(s, (gx_device *)vdev, &pdev->pswrite_common, true, 
-                          (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount + 1));
     pdev->page_fill.color = gx_no_color_index;
     return 0;
 }
 
+
 private int
-psw_setcolors(gx_device_vector * vdev, const gx_drawing_color * pdc)
+psw_can_handle_hl_color(gx_device_vector * vdev, const gs_imager_state * pis1, 
+              const gx_drawing_color * pdc)
 {
+    return false; /* High level color is not implemented yet. */
+}
+
+private int
+psw_setcolors(gx_device_vector * vdev, const gs_imager_state * pis1, 
+              const gx_drawing_color * pdc)
+{
+    const gs_imager_state * pis = NULL; /* High level color is not implemented yet. */
     if (!gx_dc_is_pure(pdc))
 	return_error(gs_error_rangecheck);
     /* PostScript only keeps track of a single color. */
-    vdev->fill_color = *pdc;
-    vdev->stroke_color = *pdc;
+    gx_hld_save_color(pis, pdc, &vdev->saved_fill_color);
+    gx_hld_save_color(pis, pdc, &vdev->saved_stroke_color);
     {
 	stream *s = gdev_vector_stream(vdev);
 	gx_color_index color = gx_dc_pure_color(pdc);
@@ -664,6 +725,8 @@ psw_setcolors(gx_device_vector * vdev, const gx_drawing_color * pdc)
 	    pprintd2(s, "%d %d r5\n", g, b);
 	else
 	    pprintd3(s, "%d %d %d rG\n", r, g, b);
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -678,6 +741,8 @@ psw_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1, fixed y1,
     pprintg4(gdev_vector_stream(vdev), "%g %g %g %g rf\n",
 	     fixed2float(x0), fixed2float(y0),
 	     fixed2float(x1 - x0), fixed2float(y1 - y0));
+    if ((gdev_vector_stream(vdev))->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -723,6 +788,8 @@ psw_beginpath(gx_device_vector * vdev, gx_path_type_t type)
 
 	stream_puts(s, "Q q\n");
 	gdev_vector_reset(vdev);
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -746,6 +813,8 @@ psw_moveto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
     print_coord2(s, x, y, NULL);
     pdev->path_state.num_points = 1;
     pdev->path_state.move = 1;
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -763,7 +832,12 @@ psw_lineto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 	stream *s = gdev_vector_stream(vdev);
 	gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
 
-	if (pdev->path_state.num_points > 0 &&
+	if (pdev->path_state.num_points > MAXOPSTACK / 2 - 10) {
+ 	    stream_puts(s, (pdev->path_state.move ? "P\n" : "p\n"));
+            pdev->path_state.num_points = 0;
+            pdev->path_state.move = 0;
+        }
+        else if (pdev->path_state.num_points > 0 &&
 	    !(pdev->path_state.num_points & 7)
 	    )
 	    stream_putc(s, '\n');	/* limit line length for DSC compliance */
@@ -778,6 +852,8 @@ psw_lineto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 	pdev->path_state.dprev[1] = pdev->path_state.dprev[0];
 	pdev->path_state.dprev[0].x = dx;
 	pdev->path_state.dprev[0].y = dy;
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return 0;
 }
@@ -810,6 +886,8 @@ psw_curveto(gx_device_vector * vdev, floatp x0, floatp y0,
     }
     pdev->path_state.num_points = 0;
     pdev->path_state.move = 0;
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -824,6 +902,8 @@ psw_closepath(gx_device_vector * vdev, floatp x0, floatp y0,
 	   "H\n" : "h\n"));
     pdev->path_state.num_points = 0;
     pdev->path_state.move = 0;
+    if ((gdev_vector_stream(vdev))->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -850,6 +930,8 @@ psw_endpath(gx_device_vector * vdev, gx_path_type_t type)
     }
     if (type & gx_path_type_clip)
 	pprints1(s, "Y%s\n", star);
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
     return 0;
 }
 
@@ -867,23 +949,26 @@ psw_open(gx_device * dev)
 
     vdev->v_memory = mem;
     vdev->vec_procs = &psw_vector_procs;
-    {
-	int code = gdev_vector_open_file_options(vdev, 512,
-					VECTOR_OPEN_FILE_SEQUENTIAL_OK |
-					VECTOR_OPEN_FILE_BBOX);
 
-	if (code < 0)
-	    return code;
-    }
     gdev_vector_init(vdev);
     vdev->fill_options = vdev->stroke_options = gx_path_type_optimize;
-    pdev->first_page = true;
     pdev->binary_ok = !pdev->params.ASCII85EncodePages;
     pdev->image_writer = gs_alloc_struct(mem, psdf_binary_writer,
 					 &st_psdf_binary_writer,
 					 "psw_open(image_writer)");
     memset(pdev->image_writer, 0, sizeof(*pdev->image_writer));  /* for GC */
     image_cache_reset(pdev);
+
+    pdev->strm = NULL;
+
+    /* if (ppdev->OpenOutputFile) */
+    {
+	int code = psw_open_printer(dev);
+
+	if (code < 0)
+	    return code;
+    }
+
     return 0;
 }
 
@@ -891,6 +976,7 @@ psw_open(gx_device * dev)
 private int
 psw_output_page(gx_device * dev, int num_copies, int flush)
 {
+    int code;
     gx_device_vector *const vdev = (gx_device_vector *)dev;
     gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
     stream *s = gdev_vector_stream(vdev);
@@ -898,7 +984,9 @@ psw_output_page(gx_device * dev, int num_copies, int flush)
     /* Check for a legitimate empty page. */
     CHECK_BEGIN_PAGE(pdev);
     sflush(s);			/* sync stream and file */
-    psw_write_page_trailer(vdev->file, num_copies, flush);
+    code = psw_write_page_trailer(vdev->file, num_copies, flush);
+    if(code < 0)
+        return code;
     vdev->in_page = false;
     pdev->first_page = false;
     gdev_vector_reset(vdev);
@@ -908,8 +996,10 @@ psw_output_page(gx_device * dev, int num_copies, int flush)
 
     dev->PageCount ++;
     if (psw_is_separate_pages(vdev)) {
-	psw_close(dev);
-	dev->is_open = false;
+	code = psw_close_printer(dev);
+
+	if (code < 0) 
+	    return code;
     }
     return 0;
 }
@@ -922,34 +1012,19 @@ psw_close(gx_device * dev)
 {
     gx_device_vector *const vdev = (gx_device_vector *)dev;
     gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
-    FILE *f = vdev->file;
-    gs_rect bbox;
 
-    gx_device_bbox_bbox(vdev->bbox_device, &bbox);
-    if (pdev->first_page & !vdev->in_page) {
-	/* Nothing has been written.  Write the file header now. */
-	psw_begin_file(pdev, &bbox);
-    } else {
-	/* If there is an incomplete page, complete it now. */
-	if (vdev->in_page) {
-	    /*
-	     * Flush the stream if it hasn't been flushed (and finalized)
-	     * already.
-	     */
-	    stream *s = vdev->strm;
-
-	    if (s->swptr != s->cbuf - 1)
-		sflush(s);
-	    psw_write_page_trailer(vdev->file, 1, 1);
-	    dev->PageCount++;
-	}
-    }
-    psw_end_file(f, dev, &pdev->pswrite_common, &bbox, 
-                 (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount));
     gs_free_object(pdev->v_memory, pdev->image_writer,
 		   "psw_close(image_writer)");
     pdev->image_writer = 0;
-    return gdev_vector_close_file(vdev);
+
+    if (vdev->strm != NULL) {
+	int code = psw_close_printer(dev);
+
+	if (code < 0)
+	    return code;
+    }
+
+    return 0;
 }
 
 /* ---------------- Get/put parameters ---------------- */
@@ -982,7 +1057,7 @@ psw_put_params(gx_device * dev, gs_param_list * plist)
 
     switch (code = param_read_float(plist, (param_name = "LanguageLevel"), &ll)) {
 	case 0:
-	    if (ll == 1.0 || ll == 1.5 || ll == 2.0)
+	    if (ll == 1.0 || ll == 1.5 || ll == 2.0 || ll == 3.0)
 		break;
 	    code = gs_error_rangecheck;
 	default:
@@ -1046,6 +1121,75 @@ psw_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     return gdev_vector_fill_rectangle(dev, x, y, w, h, color);
 }
 
+/*
+ * Open the printer's output file if necessary.
+ */
+
+private int 
+psw_open_printer(gx_device * dev)
+{
+    gx_device_vector *const vdev = (gx_device_vector *)dev;
+    gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
+
+    if (vdev->strm != 0) {
+	return 0;
+    }
+    {
+	int code = gdev_vector_open_file_options(vdev, 512,
+					VECTOR_OPEN_FILE_SEQUENTIAL_OK |
+					VECTOR_OPEN_FILE_BBOX);
+
+	if (code < 0)
+	    return code;
+    }
+
+    pdev->first_page = true;
+
+    return 0;
+}
+
+/*
+ * Close the printer's output file.
+ */
+
+private int 
+psw_close_printer(gx_device * dev)
+{
+    int code;
+    gx_device_vector *const vdev = (gx_device_vector *)dev;
+    gx_device_pswrite *const pdev = (gx_device_pswrite *)vdev;
+    FILE *f = vdev->file;
+    gs_rect bbox;
+
+    gx_device_bbox_bbox(vdev->bbox_device, &bbox);
+    if (pdev->first_page & !vdev->in_page) {
+	/* Nothing has been written.  Write the file header now. */
+	code = psw_begin_file(pdev, &bbox);
+        if (code < 0)
+            return code;
+    } else {
+	/* If there is an incomplete page, complete it now. */
+	if (vdev->in_page) { 
+	    /*
+	     * Flush the stream after writing page trailer.
+	     */
+
+	    stream *s = vdev->strm;
+	    code = psw_write_page_trailer(vdev->file, 1, 1);
+            if(code < 0)
+                return code;
+	    sflush(s);
+
+	    dev->PageCount++;
+	}
+    }
+    code = psw_end_file(f, dev, &pdev->pswrite_common, &bbox, 
+                 (psw_is_separate_pages(vdev) ? 1 : vdev->PageCount));
+    if(code < 0)
+        return code;
+
+    return gdev_vector_close_file(vdev);
+}
 
 /* ---------------- Images ---------------- */
 
@@ -1068,9 +1212,9 @@ psw_copy_mono(gx_device * dev, const byte * data,
 	((gx_device *) vdev->bbox_device, data, data_x, raster, id,
 	 x, y, w, h, zero, one);
     if (one == gx_no_color_index) {
-	color_set_pure(&color, zero);
+	set_nonclient_dev_color(&color, zero);
 	code = gdev_vector_update_fill_color((gx_device_vector *) pdev,
-					     &color);
+					     NULL, &color);
 	op = "If";
     } else if (zero == vdev->black && one == vdev->white)
 	op = "1 I";
@@ -1080,9 +1224,9 @@ psw_copy_mono(gx_device * dev, const byte * data,
 	    if (code < 0)
 		return code;
 	}
-	color_set_pure(&color, one);
+	set_nonclient_dev_color(&color, one);
 	code = gdev_vector_update_fill_color((gx_device_vector *) pdev,
-					     &color);
+					     NULL, &color);
 	op = ",";
     }
     if (code < 0)
@@ -1183,7 +1327,9 @@ psw_stroke_path(gx_device * dev, const gs_imager_state * pis,
 		psw_put_matrix(s, &mat);
 		stream_puts(s, "concat\n");
 	    }
-	}
+            if (s->end_status == ERRC)
+                return_error(gs_error_ioerror);
+        }
 	code = gdev_vector_dopath(vdev, ppath, gx_path_type_stroke,
 				  (set_ctm ? &mat : (const gs_matrix *)0));
 	if (code < 0)
@@ -1212,9 +1358,14 @@ psw_fill_mask(gx_device * dev,
     CHECK_BEGIN_PAGE(pdev);
     if (w <= 0 || h <= 0)
 	return 0;
+
+    /* gdev_vector_update_clip_path may output a grestore and gsave,
+     * causing the setting of the color to be lost.  Therefore, we
+     * must update the clip path first.
+     */
     if (depth > 1 ||
-	gdev_vector_update_fill_color(vdev, pdcolor) < 0 ||
 	gdev_vector_update_clip_path(vdev, pcpath) < 0 ||
+	gdev_vector_update_fill_color(vdev, NULL, pdcolor) < 0 ||
 	gdev_vector_update_log_op(vdev, lop) < 0
 	)
 	return gx_default_fill_mask(dev, data, data_x, raster, id,
@@ -1249,7 +1400,7 @@ psw_begin_image(gx_device * dev,
     gdev_vector_image_enum_t *pie;
     const gs_color_space *pcs = pim->ColorSpace;
     const gs_color_space *pbcs = pcs;
-    const char *base_name;
+    const char *base_name = NULL;
     gs_color_space_index index;
     int num_components;
     bool binary = pdev->binary_ok;
@@ -1349,6 +1500,7 @@ psw_begin_image(gx_device * dev,
 	buffer = gs_alloc_bytes(mem, bsize, "psw_begin_image(buffer)");
 	bs = s_alloc(mem, "psw_begin_image(buffer stream)");
 	if (buffer && bs) {
+            s_init(bs, mem);
 	    swrite_string(bs, buffer, bsize);
 	} else {
 	    /* An allocation failed. */
@@ -1434,6 +1586,12 @@ psw_begin_image(gx_device * dev,
 	}
 	stream_putc(s, '\n');
 	pprints1((bs ? bs : s), "%s\n", op);
+        if (s->end_status == ERRC) {
+            gs_free_object(mem, bs, "psw_begin_image(buffer stream)");
+            gs_free_object(mem, buffer, "psw_begin_image(buffer)");
+            gs_free_object(mem, pie, "psw_begin_image");
+            return_error(gs_error_ioerror);
+        }
     }
     *pinfo = (gx_image_enum_common_t *) pie;
     return 0;
@@ -1465,6 +1623,8 @@ psw_image_plane_data(gx_image_enum_common_t * info,
 		     planes[pi].data_x * info->plane_depths[pi],
 		     planes[pi].raster, pie->bits_per_row,
 		     *rows_used);
+        if (pdev->image_stream->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     pie->y += *rows_used;
     return code;
@@ -1507,6 +1667,8 @@ psw_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 	    gs_free_object(mem, buffer, "psw_image_end_image(buffer)");
 	}
 	stream_puts(s, "\nQ\n");
+        if (s->end_status == ERRC)
+            return_error(gs_error_ioerror);
     }
     return code;
 }
