@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevpsd.c,v 1.18 2004/10/01 03:40:35 dan Exp $ */
+/*$Id: gdevpsd.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* PhotoShop (PSD) export device, supporting DeviceN color models. */
 
 #include "math_.h"
@@ -30,6 +27,9 @@
 #include "gdevdevn.h"
 #include "gsequivc.h"
 
+/* Enable logic for a local ICC output profile. */
+#define ENABLE_ICC_PROFILE 0
+
 /* Define the device parameters. */
 #ifndef X_DPI
 #  define X_DPI 72
@@ -39,6 +39,7 @@
 #endif
 
 /* The device descriptor */
+private dev_proc_open_device(psd_prn_open);
 private dev_proc_get_params(psd_get_params);
 private dev_proc_put_params(psd_put_params);
 private dev_proc_print_page(psd_print_page);
@@ -48,6 +49,8 @@ private dev_proc_get_color_mapping_procs(get_psd_color_mapping_procs);
 private dev_proc_get_color_comp_index(psd_get_color_comp_index);
 private dev_proc_encode_color(psd_encode_color);
 private dev_proc_decode_color(psd_decode_color);
+private dev_proc_encode_color(psd_encode_compressed_color);
+private dev_proc_decode_color(psd_decode_compressed_color);
 private dev_proc_update_spot_equivalent_colors(psd_update_spot_equivalent_colors);
 
 /* This is redundant with color_info.cm_name. We may eliminate this
@@ -91,13 +94,17 @@ typedef struct psd_device_s {
 private 
 ENUM_PTRS_WITH(psd_device_enum_ptrs, psd_device *pdev)
 {
+    if (index == 0)
+	ENUM_RETURN(pdev->devn_params.compressed_color_list);
+    index--;
     if (index < pdev->devn_params.separations.num_separations)
-	ENUM_RETURN(pdev->devn_params.separations.names[index]);
+	ENUM_RETURN(pdev->devn_params.separations.names[index].data);
     ENUM_PREFIX(st_device_printer,
 		    pdev->devn_params.separations.num_separations);
+    return 0;
 }
-
 ENUM_PTRS_END
+
 private RELOC_PTRS_WITH(psd_device_reloc_ptrs, psd_device *pdev)
 {
     RELOC_PREFIX(st_device_printer);
@@ -105,9 +112,10 @@ private RELOC_PTRS_WITH(psd_device_reloc_ptrs, psd_device *pdev)
 	int i;
 
 	for (i = 0; i < pdev->devn_params.separations.num_separations; ++i) {
-	    RELOC_PTR(psd_device, devn_params.separations.names[i]);
+	    RELOC_PTR(psd_device, devn_params.separations.names[i].data);
 	}
     }
+    RELOC_PTR(psd_device, devn_params.compressed_color_list);
 }
 RELOC_PTRS_END
 
@@ -127,8 +135,8 @@ gs_private_st_composite_final(st_psd_device, psd_device,
 /*
  * Macro definition for psd device procedures
  */
-#define device_procs(get_color_mapping_procs)\
-{	gdev_prn_open,\
+#define device_procs(get_color_mapping_procs, encode_color, decode_color)\
+{	psd_prn_open,\
 	gx_default_get_initial_matrix,\
 	NULL,				/* sync_output */\
 	gdev_prn_output_page,		/* output_page */\
@@ -179,8 +187,8 @@ gs_private_st_composite_final(st_psd_device, psd_device,
 	NULL,				/* discard_transparency_layer */\
 	get_color_mapping_procs,	/* get_color_mapping_procs */\
 	psd_get_color_comp_index,	/* get_color_comp_index */\
-	psd_encode_color,		/* encode_color */\
-	psd_decode_color,		/* decode_color */\
+	encode_color,			/* encode_color */\
+	decode_color,			/* decode_color */\
 	NULL,				/* pattern_manage */\
 	NULL,				/* fill_rectangle_hl_color */\
 	NULL,				/* include_color_space */\
@@ -203,7 +211,7 @@ private fixed_colorant_name DeviceRGBComponents[] = {
 	0		/* List terminator */
 };
 
-#define psd_device_body(procs, dname, ncomp, pol, depth, mg, mc, cn)\
+#define psd_device_body(procs, dname, ncomp, pol, depth, mg, mc, sl, cn)\
     std_device_full_body_type_extended(psd_device, &procs, dname,\
 	  &st_psd_device,\
 	  (int)((long)(DEFAULT_WIDTH_10THS) * (X_DPI) / 10),\
@@ -215,7 +223,7 @@ private fixed_colorant_name DeviceRGBComponents[] = {
 	  depth, 0,		/* Depth, GrayIndex */\
 	  mg, mc,		/* MaxGray, MaxColor */\
 	  mg + 1, mc + 1,	/* DitherGray, DitherColor */\
-	  GX_CINFO_SEP_LIN,	/* Linear & Separable */\
+	  sl,			/* Linear & Separable? */\
 	  cn,			/* Process color model name */\
 	  0, 0,			/* offsets */\
 	  0, 0, 0, 0		/* margins */\
@@ -225,18 +233,19 @@ private fixed_colorant_name DeviceRGBComponents[] = {
 /*
  * PSD device with RGB process color model.
  */
-private const gx_device_procs spot_rgb_procs = device_procs(get_psdrgb_color_mapping_procs);
+private const gx_device_procs spot_rgb_procs =
+    device_procs(get_psdrgb_color_mapping_procs, psd_encode_color, psd_decode_color);
 
 const psd_device gs_psdrgb_device =
 {   
-    psd_device_body(spot_rgb_procs, "psdrgb", 3, GX_CINFO_POLARITY_ADDITIVE, 24, 255, 255, "DeviceRGB"),
+    psd_device_body(spot_rgb_procs, "psdrgb", 3, GX_CINFO_POLARITY_ADDITIVE, 24, 255, 255, GX_CINFO_SEP_LIN, "DeviceRGB"),
     /* devn_params specific parameters */
     { 8,	/* Bits per color - must match ncomp, depth, etc. above */
       DeviceRGBComponents,	/* Names of color model colorants */
       3,			/* Number colorants for RGB */
       0,			/* MaxSeparations has not been specified */
       {0},			/* SeparationNames */
-      {0},			/* SeparationOrder names */
+      0,			/* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 }	/* Initial component SeparationOrder */
     },
     { true },			/* equivalent CMYK colors for spot colors */
@@ -246,26 +255,40 @@ const psd_device gs_psdrgb_device =
 
 /*
  * Select the default number of components based upon the number of bits
- * that we have in a gx_color_index
+ * that we have in a gx_color_index.  If we have 64 bits then we can compress
+ * the colorant data.  This allows us to handle more colorants.  However the
+ * compressed encoding is not separable.  If we do not have 64 bits then we
+ * use a simple non-compressable encoding.
  */
-#define NC ((sizeof(gx_color_index) <= 8) ? sizeof(gx_color_index) : 8)
+#if USE_COMPRESSED_ENCODING
+#define NC GX_DEVICE_COLOR_MAX_COMPONENTS 
+#define SL GX_CINFO_SEP_LIN_NONE
+#define ENCODE_COLOR psd_encode_compressed_color
+#define DECODE_COLOR psd_decode_compressed_color
+#else
+#define NC ARCH_SIZEOF_GX_COLOR_INDEX
+#define SL GX_CINFO_SEP_LIN
+#define ENCODE_COLOR psd_encode_color
+#define DECODE_COLOR psd_decode_color
+#endif
+#define GCIB (ARCH_SIZEOF_GX_COLOR_INDEX * 8)
 
 /*
  * PSD device with CMYK process color model and spot color support.
  */
 private const gx_device_procs spot_cmyk_procs
-		= device_procs(get_psd_color_mapping_procs);
+	= device_procs(get_psd_color_mapping_procs, ENCODE_COLOR, DECODE_COLOR);
 
 const psd_device gs_psdcmyk_device =
 {   
-    psd_device_body(spot_cmyk_procs, "psdcmyk", NC, GX_CINFO_POLARITY_SUBTRACTIVE, NC * 8, 255, 255, "DeviceCMYK"),
+    psd_device_body(spot_cmyk_procs, "psdcmyk", NC, GX_CINFO_POLARITY_SUBTRACTIVE, GCIB, 255, 255, SL, "DeviceCMYK"),
     /* devn_params specific parameters */
     { 8,	/* Bits per color - must match ncomp, depth, etc. above */
       DeviceCMYKComponents,	/* Names of color model colorants */
       4,			/* Number colorants for CMYK */
       0,			/* MaxSeparations has not been specified */
       {0},			/* SeparationNames */
-      {0},			/* SeparationOrder names */
+      0,			/* SeparationOrder names */
       {0, 1, 2, 3, 4, 5, 6, 7 }	/* Initial component SeparationOrder */
     },
     { true },			/* equivalent CMYK colors for spot colors */
@@ -274,6 +297,26 @@ const psd_device gs_psdcmyk_device =
 };
 
 #undef NC
+#undef SL
+#undef ENCODE_COLOR
+#undef DECODE_COLOR
+
+/* Open the psd devices */
+int
+psd_prn_open(gx_device * pdev)
+{
+    int code = gdev_prn_open(pdev);
+
+#if !USE_COMPRESSED_ENCODING
+    /*
+     * If we are using the compressed encoding scheme, then set the separable
+     * and linear info.
+     */
+    set_linear_color_bits_mask_shift(pdev);
+    pdev->color_info.separable_and_linear = GX_CINFO_SEP_LIN;
+#endif
+    return code;
+}
 
 /*
  * The following procedures are used to map the standard color spaces into
@@ -486,6 +529,29 @@ psd_decode_color(gx_device * dev, gx_color_index color, gx_color_value * out)
     return 0;
 }
 
+/*
+ * Encode a list of colorant values into a gx_color_index_value.
+ * With 64 bit gx_color_index values, we compress the colorant values.  This
+ * allows us to handle more than 8 colorants.
+ */
+private gx_color_index
+psd_encode_compressed_color(gx_device *dev, const gx_color_value colors[])
+{
+    return devn_encode_compressed_color(dev, colors, &(((psd_device *)dev)->devn_params));
+}
+
+/*
+ * Decode a gx_color_index value back to a list of colorant values.
+ * With 64 bit gx_color_index values, we compress the colorant values.  This
+ * allows us to handle more than 8 colorants.
+ */
+private int
+psd_decode_compressed_color(gx_device * dev, gx_color_index color, gx_color_value * out)
+{
+    return devn_decode_compressed_color(dev, color, out,
+		    &(((psd_device *)dev)->devn_params));
+}
+
 /*
  * Convert a gx_color_index to RGB.
  */
@@ -516,6 +582,7 @@ psd_update_spot_equivalent_colors(gx_device *pdev, const gs_state * pgs)
     return 0;
 }
 
+#if ENABLE_ICC_PROFILE
 private int
 psd_open_profile(psd_device *xdev, char *profile_fn, icmLuBase **pluo,
 		 int *poutn)
@@ -559,6 +626,7 @@ psd_open_profiles(psd_device *xdev)
     }
     return code;
 }
+#endif
 
 /* Get parameters.  We provide a default CRD. */
 private int
@@ -566,9 +634,11 @@ psd_get_params(gx_device * pdev, gs_param_list * plist)
 {
     psd_device *xdev = (psd_device *)pdev;
     int code;
+#if ENABLE_ICC_PROFILE
     gs_param_string pos;
     gs_param_string prgbs;
     gs_param_string pcmyks;
+#endif
 
     code = gdev_prn_get_params(pdev, plist);
     if (code < 0)
@@ -578,6 +648,7 @@ psd_get_params(gx_device * pdev, gs_param_list * plist)
     if (code < 0)
 	return code;
 
+#if ENABLE_ICC_PROFILE
     pos.data = (const byte *)xdev->profile_out_fn,
 	pos.size = strlen(xdev->profile_out_fn),
 	pos.persistent = false;
@@ -589,15 +660,19 @@ psd_get_params(gx_device * pdev, gs_param_list * plist)
 	prgbs.size = strlen(xdev->profile_rgb_fn),
 	prgbs.persistent = false;
     code = param_write_string(plist, "ProfileRgb", &prgbs);
+    if (code < 0)
+	return code;
 
     pcmyks.data = (const byte *)xdev->profile_cmyk_fn,
 	pcmyks.size = strlen(xdev->profile_cmyk_fn),
 	pcmyks.persistent = false;
     code = param_write_string(plist, "ProfileCmyk", &prgbs);
+#endif
 
     return code;
 }
 
+#if ENABLE_ICC_PROFILE
 private int
 psd_param_read_fn(gs_param_list *plist, const char *name,
 		  gs_param_string *pstr, uint max_len)
@@ -612,6 +687,7 @@ psd_param_read_fn(gs_param_list *plist, const char *name,
     }
     return code;
 }
+#endif
 
 /* Compare a C string and a gs_param_string. */
 static bool
@@ -657,15 +733,17 @@ private int
 psd_put_params(gx_device * pdev, gs_param_list * plist)
 {
     psd_device * const pdevn = (psd_device *) pdev;
-    int ecode = 0;
-    int code;
+    int code = 0;
+#if ENABLE_ICC_PROFILE
     gs_param_string po;
     gs_param_string prgb;
     gs_param_string pcmyk;
+#endif
     gs_param_string pcm;
     psd_color_model color_model = pdevn->color_model;
     gx_device_color_info save_info = pdevn->color_info;
 
+#if ENABLE_ICC_PROFILE
     code = psd_param_read_fn(plist, "ProfileOut", &po,
 				 sizeof(pdevn->profile_out_fn));
     if (code >= 0)
@@ -674,6 +752,7 @@ psd_put_params(gx_device * pdev, gs_param_list * plist)
     if (code >= 0)
 	code = psd_param_read_fn(plist, "ProfileCmyk", &pcmyk,
 				 sizeof(pdevn->profile_cmyk_fn));
+#endif
 
     if (code >= 0)
 	code = param_read_name(plist, "ProcessColorModel", &pcm);
@@ -691,22 +770,21 @@ psd_put_params(gx_device * pdev, gs_param_list * plist)
 			       code = gs_error_rangecheck);
 	}
     }
-    if (code < 0)
-	ecode = code;
 
-    if (ecode >= 0)
-        ecode = psd_set_color_model(pdevn, color_model);
+    if (code >= 0)
+        code = psd_set_color_model(pdevn, color_model);
 
     /* handle the standard DeviceN related parameters */
-    if (ecode == 0)
-        ecode = devn_printer_put_params(pdev, plist,
+    if (code == 0)
+        code = devn_printer_put_params(pdev, plist,
 		&(pdevn->devn_params), &(pdevn->equiv_cmyk_colors));
 
-    if (ecode < 0) {
+    if (code < 0) {
 	pdev->color_info = save_info;
-	return ecode;
+	return code;
     }
 
+#if ENABLE_ICC_PROFILE
     /* Open any ICC profiles that have been specified. */
     if (po.data != 0) {
 	memcpy(pdevn->profile_out_fn, po.data, po.size);
@@ -723,6 +801,7 @@ psd_put_params(gx_device * pdev, gs_param_list * plist)
     if (memcmp(&pdevn->color_info, &save_info,
 			    size_of(gx_device_color_info)) != 0)
         code = psd_open_profiles(pdevn);
+#endif
 
     return code;
 }
@@ -860,7 +939,7 @@ psd_write_header(psd_write_ctx *xc, psd_device *pdev)
     int chan_idx;
     int chan_names_len = 0;
     int sep_num;
-    const gs_param_string *separation_name;
+    const devn_separation_name *separation_name;
 
     psd_write(xc, (const byte *)"8BPS", 4); /* Signature */
     psd_write_16(xc, 1); /* Version - Always equal to 1*/
@@ -881,7 +960,7 @@ psd_write_header(psd_write_ctx *xc, psd_device *pdev)
     /* Channel Names */
     for (chan_idx = NUM_CMYK_COMPONENTS; chan_idx < xc->num_channels; chan_idx++) {
 	sep_num = xc->chnl_to_orig_sep[chan_idx] - NUM_CMYK_COMPONENTS;
-	separation_name = pdev->devn_params.separations.names[sep_num];
+	separation_name = &(pdev->devn_params.separations.names[sep_num]);
 	chan_names_len += (separation_name->size + 1);
     }    
     psd_write_32(xc, 12 + (chan_names_len + (chan_names_len % 2))
@@ -893,7 +972,7 @@ psd_write_header(psd_write_ctx *xc, psd_device *pdev)
     psd_write_32(xc, chan_names_len + (chan_names_len % 2));
     for (chan_idx = NUM_CMYK_COMPONENTS; chan_idx < xc->num_channels; chan_idx++) {
 	sep_num = xc->chnl_to_orig_sep[chan_idx] - NUM_CMYK_COMPONENTS;
-	separation_name = pdev->devn_params.separations.names[sep_num];
+	separation_name = &(pdev->devn_params.separations.names[sep_num]);
 	psd_write_8(xc, (byte) separation_name->size);
 	psd_write(xc, separation_name->data, separation_name->size);
     }    
@@ -1003,37 +1082,46 @@ psd_write_image_data(psd_write_ctx *xc, gx_device_printer *pdev)
     int i, j;
     byte *line, *sep_line;
     int base_bytes_pp = xc->base_bytes_pp;
-    int bytes_pp =pdev->color_info.num_components;
     int chan_idx;
     psd_device *xdev = (psd_device *)pdev;
     icmLuBase *luo = xdev->lu_out;
-    byte *row;
+    byte * row, * unpacked;
+    int width = pdev->width;
+    int non_encodable_count = 0;
+    int num_comp = xc->num_channels;
 
     psd_write_16(xc, 0); /* Compression */
 
     line = gs_alloc_bytes(pdev->memory, raster, "psd_write_image_data");
     sep_line = gs_alloc_bytes(pdev->memory, xc->width, "psd_write_sep_line");
+    unpacked = gs_alloc_bytes(pdev->memory, width *num_comp,
+							"psd_write_image");
+    if (line == NULL || sep_line == NULL || unpacked == NULL)
+	return_error(gs_error_VMerror);
 
     /* Print the output planes */
-    for (chan_idx = 0; chan_idx < xc->num_channels; chan_idx++) {
+    for (chan_idx = 0; chan_idx < num_comp; chan_idx++) {
 	for (j = 0; j < xc->height; ++j) {
 	    int data_pos = xc->chnl_to_position[chan_idx];
 
 	    /* Check if the separation is present in the SeparationOrder */
 	    if (data_pos >= 0) {
 	        code = gdev_prn_get_bits(pdev, j, line, &row);
+	        /* Unpack the encoded color info */
+	        non_encodable_count += devn_unpack_row((gx_device *)pdev,
+			num_comp, &(xdev->devn_params), width, row, unpacked);
 	        if (luo == NULL) {
 		    for (i = 0; i < xc->width; ++i) {
 		        if (base_bytes_pp == 3) {
 			    /* RGB */
-			    sep_line[i] = row[i*bytes_pp + data_pos];
+			    sep_line[i] = unpacked[i * num_comp + data_pos];
 		        } else {
 			    /* CMYK */
-			    sep_line[i] = 255 - row[i*bytes_pp + data_pos];
+			    sep_line[i] = 255 - unpacked[i * num_comp + data_pos];
 		        }
 		    }
 	        } else {
-		    psd_calib_row(xc, &sep_line, row, data_pos, luo);
+		    psd_calib_row(xc, &sep_line, unpacked, data_pos, luo);
 	        }	
 	        psd_write(xc, sep_line, xc->width);
 	    } else {
