@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpsfx.c,v 1.24 2003/09/04 14:28:46 igor Exp $ */
+/* $Id: gdevpsfx.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Convert Type 1 Charstrings to Type 2 */
 #include "math_.h"
 #include "memory_.h"
@@ -779,7 +776,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 		    c = c2_vvcurveto;
 		    csp[-1] = csp[0];
 		    if (csp[-5] == 0) {
-			memcpy(csp - 5, csp - 4, sizeof(*csp) * 4);
+			memmove(csp - 5, csp - 4, sizeof(*csp) * 4);
 			POP(2);
 		    } else
 			POP(1);
@@ -787,7 +784,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 		    /* A B|0 C D E 0 rrcurveto => [B] A C D E hhcurveto */
 		    c = c2_hhcurveto;
 		    if (csp[-4] == 0) {
-			memcpy(csp - 4, csp - 3, sizeof(*csp) * 3);
+			memmove(csp - 4, csp - 3, sizeof(*csp) * 3);
 			POP(2);
 		    } else {
 			*csp = csp[-5], csp[-5] = csp[-4], csp[-4] = *csp;
@@ -809,7 +806,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 	    case c2_hhcurveto:	/* hrcurveto (x1 0 x2 y2 x3 0 rrcurveto)* => */
 				/* hhcurveto */
 		if (csp[-4] == 0 && *csp == 0) {
-		    memcpy(csp - 4, csp - 3, sizeof(*csp) * 3);
+		    memmove(csp - 4, csp - 3, sizeof(*csp) * 3);
 		    c = prev_op;
 		    POP(2);
 		    goto put;
@@ -818,7 +815,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 	    case c2_vvcurveto:	/* rvcurveto (0 y1 x2 y2 0 y3 rrcurveto)* => */
 				/* vvcurveto */
 		if (csp[-5] == 0 && csp[-1] == 0) {
-		    memcpy(csp - 5, csp - 4, sizeof(*csp) * 3);
+		    memmove(csp - 5, csp - 4, sizeof(*csp) * 3);
 		    csp[-2] = *csp;
 		    c = prev_op;
 		    POP(2);
@@ -834,7 +831,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 		/* hvcurveto (vhcurveto hvcurveto)* vrcurveto => hvcurveto */
 		if (csp[-5] != 0)
 		    goto copy;
-		memcpy(csp - 5, csp - 4, sizeof(*csp) * 5);
+		memmove(csp - 5, csp - 4, sizeof(*csp) * 5);
 		c = prev_op;
 		POP(1);
 		goto put;
@@ -848,7 +845,7 @@ psf_convert_type1_to_type2(stream *s, const gs_glyph_data_t *pgd,
 		if (csp[-4] != 0)
 		    goto copy;
 		/* A 0 C D E F => A C D F E */
-		memcpy(csp - 4, csp - 3, sizeof(*csp) * 2);
+		memmove(csp - 4, csp - 3, sizeof(*csp) * 2);
 		csp[-2] = *csp;
 		c = prev_op;
 		POP(1);
