@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevvec.c,v 1.25 2004/09/27 04:55:00 ray Exp $ */
+/* $Id: gdevvec.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Utilities for "vector" devices */
 #include "math_.h"
 #include "memory_.h"
@@ -74,7 +71,7 @@ gdev_vector_dopath(gx_device_vector *vdev, const gx_path * ppath,
      * which requires (untransformed) device coordinates.
      */
     if (rtype != prt_none &&
-	!((type & gx_path_type_stroke) && rtype == prt_open) &&
+	(!(type & gx_path_type_stroke) || rtype == prt_closed) &&
 	(pmat == 0 || is_xxyy(pmat) || is_xyyx(pmat)) &&
 	(state.scale_mat.xx == 1.0 && state.scale_mat.yy == 1.0 &&
 	 is_xxyy(&state.scale_mat) &&
@@ -622,8 +619,10 @@ gdev_vector_dopath_segment(gdev_vector_dopath_state_t *state, int pe_op,
 
     switch (pe_op) {
 	case gs_pe_moveto:
-	    gs_point_transform_inverse(fixed2float(vs[0].x),
+	    code = gs_point_transform_inverse(fixed2float(vs[0].x),
 				       fixed2float(vs[0].y), pmat, &vp[0]);
+	    if (code < 0)
+		return code;
 	    if (state->first)
 		state->start = vp[0], state->first = false;
 	    code = vdev_proc(vdev, moveto)
@@ -632,18 +631,24 @@ gdev_vector_dopath_segment(gdev_vector_dopath_state_t *state, int pe_op,
 	    state->prev = vp[0];
 	    break;
 	case gs_pe_lineto:
-	    gs_point_transform_inverse(fixed2float(vs[0].x),
+	    code = gs_point_transform_inverse(fixed2float(vs[0].x),
 				       fixed2float(vs[0].y), pmat, &vp[0]);
+	    if (code < 0)
+		return code;
 	    code = vdev_proc(vdev, lineto)
 		(vdev, state->prev.x, state->prev.y, vp[0].x, vp[0].y,
 		 state->type);
 	    state->prev = vp[0];
 	    break;
 	case gs_pe_curveto:
-	    gs_point_transform_inverse(fixed2float(vs[0].x),
+	    code = gs_point_transform_inverse(fixed2float(vs[0].x),
 				       fixed2float(vs[0].y), pmat, &vp[0]);
-	    gs_point_transform_inverse(fixed2float(vs[1].x),
+	    if (code < 0)
+		return code;
+	    code = gs_point_transform_inverse(fixed2float(vs[1].x),
 				       fixed2float(vs[1].y), pmat, &vp[1]);
+	    if (code < 0)
+		return code;
 	    gs_point_transform_inverse(fixed2float(vs[2].x),
 				       fixed2float(vs[2].y), pmat, &vp[2]);
 	    code = vdev_proc(vdev, curveto)
@@ -798,17 +803,21 @@ gdev_vector_close_file(gx_device_vector * vdev)
     gs_free_object(vdev->v_memory, vdev->bbox_device,
 		   "vector_close(bbox_device)");
     vdev->bbox_device = 0;
-    sclose(vdev->strm);
-    gs_free_object(vdev->v_memory, vdev->strm, "vector_close(strm)");
-    vdev->strm = 0;
-    gs_free_object(vdev->v_memory, vdev->strmbuf, "vector_close(strmbuf)");
-    vdev->strmbuf = 0;
+    if (vdev->strm) {
+	sclose(vdev->strm);
+	gs_free_object(vdev->v_memory, vdev->strm, "vector_close(strm)");
+	vdev->strm = 0;
+	gs_free_object(vdev->v_memory, vdev->strmbuf, "vector_close(strmbuf)");
+	vdev->strmbuf = 0;
+    }
     vdev->file = 0;
-    err = ferror(f);
-    /* We prevented sclose from closing the file. */
-    if (gx_device_close_output_file((gx_device *)vdev, vdev->fname, f) != 0 
-	|| err != 0)
-	return_error(gs_error_ioerror);
+    if (f) {
+	err = ferror(f);
+	/* We prevented sclose from closing the file. */
+	if (gx_device_close_output_file((gx_device *)vdev, vdev->fname, f) != 0 
+		|| err != 0)
+	    return_error(gs_error_ioerror);
+    }
     return 0;
 }
 
