@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevxcf.c,v 1.9 2004/07/01 04:41:38 dan Exp $ */
+/*$Id: gdevxcf.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Gimp (XCF) export device, supporting DeviceN color models. */
 
 #include "math_.h"
@@ -346,7 +343,7 @@ cmyk_cs_to_spotn_cm(gx_device * dev, frac c, frac m, frac y, frac k, frac out[])
     int i;
 
     if (luo != NULL) {
-	double in[3];
+	double in[4];
 	double tmp[MAX_CHAN];
 	int outn = xdev->lu_cmyk_outn;
 
@@ -1343,7 +1340,7 @@ xcf_write_footer(xcf_write_ctx *xc, xcf_device *pdev)
 	int offset;
 	int tile_idx;
 
-	dlprintf2("actual tile offset: %d %d\n", xc->offset, sizeof(gx_color_index));
+	dlprintf2("actual tile offset: %d %d\n", xc->offset, (int)arch_sizeof_color_index);
 	xcf_write_32(xc, xc->width);
 	xcf_write_32(xc, xc->height);
 	xcf_write_32(xc, separation_name->size + 1);
