@@ -1,23 +1,17 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gdevxini.c,v $ $Revision: 1.9.2.2 $ */
+/* $Id: gdevxini.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* X Windows driver initialization/finalization */
 #include "memory_.h"
 #include "x_.h"
@@ -29,7 +23,7 @@
 #include "gdevbbox.h"
 #include "gdevx.h"
 
-extern char *getenv(P1(const char *));
+extern char *getenv(const char *);
 
 extern const gx_device_bbox gs_bbox_device;
 extern const gx_device_X gs_x11_device;
@@ -51,7 +45,7 @@ private_st_x11fontmap();
 /* ---------------- Opening/initialization ---------------- */
 
 /* Forward references */
-private void gdev_x_setup_fontmap(P1(gx_device_X *));
+private void gdev_x_setup_fontmap(gx_device_X *);
 
 /* Catch the alloc error when there is not enough resources for the
  * backing pixmap.  Automatically shut off backing pixmap and let the
@@ -99,7 +93,7 @@ gdev_x_open(gx_device_X * xdev)
     Display *dpy;
     XColor xc;
     int zero = 0;
-    int xid_height, xid_width;
+    int xid_height = 0, xid_width = 0;
     int code;
 
 #ifdef DEBUG
@@ -307,6 +301,9 @@ gdev_x_open(gx_device_X * xdev)
 	XCloseDisplay(xdev->dpy);
 	return code;
     }
+    /* Now that the color map is setup check if the device is separable. */
+    check_device_separable((gx_device *)xdev);
+
     gdev_x_setup_fontmap(xdev);
 
     if (!xdev->ghostview) {
@@ -609,11 +606,14 @@ x_set_buffer(gx_device_X * xdev)
 	COPY_PROC(text_begin);
 #undef COPY_PROC
 	if (xdev->is_buffered) {
+    	    check_device_separable((gx_device *)xdev);
 	    gx_device_forward_fill_in_procs((gx_device_forward *)xdev);
 	    xdev->box_procs = gdev_x_box_procs;
 	    xdev->box_proc_data = xdev;
-	} else
+	} else {
+    	    check_device_separable((gx_device *)xdev);
 	    gx_device_fill_in_procs((gx_device *)xdev);
+	}
     }
     return 0;
 }
