@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: genconf.c,v 1.4.4.1 2002/02/22 19:45:57 ray Exp $ */
+/* $Id: genconf.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Generate configuration files */
 #include "stdpre.h"
 #include <assert.h>
@@ -41,6 +38,12 @@
  * See the next section on command line switches for the meaning of
  * <name_prefix>.
  *
+ *    -comp <name>
+ *
+ *	Adds compositor_(<name_prefix>_composite_<name>_type) to <gconfig.h>.
+ *	Used for gs_composite_type_t structures, whose identity may need to
+ *	passed through the command list.
+ *
  *    -dev <device>
  *
  *	Adds device_(<name_prefix><device>_device) to <gconfig.h>.
@@ -131,6 +134,13 @@
  *	<opername> is (usually) the name of the source file in which the
  *	table appears.
  *
+ *    -plugin <plugin_name>
+ *
+ *	Adds plugin_(<name_prefix><plugin_name>_instantiate) to <gconfig.h>.
+ *	Used for plugins to be instantiated when Ghostscript
+ *	is started.  By convention, <plugin_name> is (almost always) the name
+ *	of the source file in which the plugin is defined.
+ *
  *    -ps <psname>
  *
  *	Adds psfile_("<psname>.ps",<len+3>), where <len> is the number of
@@ -196,7 +206,8 @@
  *
  *	Writes the list of library paths, links, and library file names
  *	on <lib.tr>.  -lo also writes the list of object file names,
- *	as for -o.
+ *	as for -o. This feature is obsolete and is not longer in use.
+ *      Use -ol instead.
  *
  *    -o[l] <obj.tr>
  *
@@ -269,13 +280,14 @@ typedef struct config_s {
 	    string_list_t sorted_resources;
 	    string_list_t resources;
 	    string_list_t devs;	/* also includes devs2 */
+	    string_list_t compositors;
 	    string_list_t fonts;
 	    string_list_t libs;
 	    string_list_t libpaths;
 	    string_list_t links;
 	    string_list_t objs;
 	} named;
-#define NUM_RESOURCE_LISTS 8
+#define NUM_RESOURCE_LISTS 9
 	string_list_t indexed[NUM_RESOURCE_LISTS];
     } lists;
     string_pattern_t lib_p;
@@ -298,6 +310,7 @@ static const config_t init_config = {
 static const string_list_t init_config_lists[] = {
     {"resource", 100, uniq_first},
     {"sorted_resource", 20, uniq_first},
+    {"-comp", 10, uniq_first},
     {"-dev", 100, uniq_first},
     {"-font", 50, uniq_first},
     {"-lib", 20, uniq_last},
@@ -307,20 +320,20 @@ static const string_list_t init_config_lists[] = {
 };
 
 /* Forward definitions */
-private void *mrealloc(P3(void *, size_t, size_t));
-int alloc_list(P1(string_list_t *));
-void dev_file_name(P1(char *));
-int process_replaces(P1(config_t *));
-int read_dev(P2(config_t *, const char *));
-int read_token(P3(char *, int, const char **));
-int add_entry(P4(config_t *, char *, const char *, int));
-string_item_t *add_item(P3(string_list_t *, const char *, int));
-void sort_uniq(P2(string_list_t *, bool));
-void write_list(P3(FILE *, const string_list_t *, const char *));
-void write_list_pattern(P3(FILE *, const string_list_t *, const string_pattern_t *));
-bool var_expand(P3(char *, char [MAX_STR], const config_t *));
-void add_definition(P4(const char *, const char *, string_list_t *, bool));
-string_item_t *lookup(P2(const char *, const string_list_t *));
+private void *mrealloc(void *, size_t, size_t);
+int alloc_list(string_list_t *);
+void dev_file_name(char *);
+int process_replaces(config_t *);
+int read_dev(config_t *, const char *);
+int read_token(char *, int, const char **);
+int add_entry(config_t *, char *, const char *, int);
+string_item_t *add_item(string_list_t *, const char *, int);
+void sort_uniq(string_list_t *, bool);
+void write_list(FILE *, const string_list_t *, const char *);
+void write_list_pattern(FILE *, const string_list_t *, const string_pattern_t *);
+bool var_expand(char *, char [MAX_STR], const config_t *);
+void add_definition(const char *, const char *, string_list_t *, bool);
+string_item_t *lookup(const char *, const string_list_t *);
 
 int
 main(int argc, char *argv[])
@@ -470,6 +483,7 @@ main(int argc, char *argv[])
 	    case 'h':
 		process_replaces(&conf);
 		fputs("/* This file was generated automatically by genconf.c. */\n", out);
+		write_list(out, &conf.lists.named.compositors, "%s\n");
 		write_list(out, &conf.lists.named.devs, "%s\n");
 		sort_uniq(&conf.lists.named.resources, true);
 		write_list(out, &conf.lists.named.resources, "%s\n");
@@ -742,6 +756,12 @@ add_entry(config_t * pconf, char *category, const char *item, int file_index)
 	/* Handle a few resources specially; just queue the rest. */
 	switch (category[0]) {
 #define IS_CAT(str) !strcmp(category, str)
+	    case 'c':
+		if (!IS_CAT("comp"))
+		    goto err;
+		pat = "compositor_(%scomposite_%%s_type)";
+		list = &pconf->lists.named.compositors;
+		goto pre;
 	    case 'd':
 		if (IS_CAT("dev"))
 		    pat = "device_(%s%%s_device)";
@@ -824,7 +844,10 @@ pre:		sprintf(template, pat, pconf->name_prefix);
 			    item, (uint)(strlen(item) + 3));
 		    item = str;
 		    break;
-		}
+		} else if (IS_CAT("plugin")) {
+		    pat = "plugin_(%s%%s_instantiate)";
+                    goto pre;
+                }
 		goto err;
 	    case 'r':
 		if (IS_CAT("replace")) {
