@@ -1,34 +1,31 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gp_vms.c,v $ $Revision: 1.4.2.3 $ */
+/* $Id: gp_vms.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* VAX/VMS specific routines for Ghostscript */
+
 #include "string_.h"
+#include "memory_.h"
 #include "gx.h"
 #include "gp.h"
+#include "gpmisc.h"
 #include "gsstruct.h"
 #include <stat.h>
 #include <stdlib.h>		/* for exit() with some compiler versions */
 #include <errno.h>		/* for exit() with other compiler versions */
 #include <unixio.h>
 
-extern char *getenv(P1(const char *));
+extern char *getenv(const char *);
 
 /* Apparently gcc doesn't allow extra arguments for fopen: */
 #ifdef VMS			/* DEC C */
@@ -116,7 +113,7 @@ gp_get_realtime(long *pdt)
     struct {
 	uint _l0, _l1;
     } binary_date, now, difference;
-    long lib$ediv(), lib$subx(), sys$bintim(), sys$gettim();
+    long LIB$EDIV(), LIB$SUBX(), SYS$BINTIM(), SYS$GETTIM();
     long units_per_second = 10000000;
     char *jan_1_1980 = "1-JAN-1980 00:00:00.00";
     descrip str_desc;
@@ -133,14 +130,14 @@ gp_get_realtime(long *pdt)
     /* Convert January 1, 1980 into a binary absolute time */
     str_desc.dsc$w_length = strlen(jan_1_1980);
     str_desc.dsc$a_pointer = jan_1_1980;
-    (void)sys$bintim(&str_desc, &binary_date);
+    (void)SYS$BINTIM(&str_desc, &binary_date);
 
     /* Compute number of 100 nanosecond units since January 1, 1980.  */
-    (void)sys$gettim(&now);
-    (void)lib$subx(&now, &binary_date, &difference);
+    (void)SYS$GETTIM(&now);
+    (void)LIB$SUBX(&now, &binary_date, &difference);
 
     /* Convert to seconds and nanoseconds.  */
-    (void)lib$ediv(&units_per_second, &difference, &pdt[0], &pdt[1]);
+    (void)LIB$EDIV(&units_per_second, &difference, &pdt[0], &pdt[1]);
     pdt[1] *= 100;
 }
 
@@ -152,6 +149,24 @@ gp_get_usertime(long *pdt)
     gp_get_realtime(pdt);	/* Use an approximation for now.  */
 }
 
+
+/* ------ Persistent data cache ------*/
+
+/* insert a buffer under a (type, key) pair */
+int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
+{
+    /* not yet implemented */
+    return 0;
+}
+
+/* look up a (type, key) in the cache */
+int gp_cache_query(int type, byte* key, int keylen, void **buffer,
+    gp_cache_alloc alloc, void *userdata)
+{
+    /* not yet implemented */
+    return -1;
+}
+
 /* ------ Screen management ------ */
 
 /* Get the environment variable that specifies the display to use. */
@@ -220,12 +235,31 @@ FILE *
 gp_open_scratch_file(const char *prefix, char fname[gp_file_name_sizeof],
 		     const char *mode)
 {
-    if (strlen(prefix) + 6 >= gp_file_name_sizeof)
+    FILE *f;
+    char tmpdir[gp_file_name_sizeof];
+    int tdlen = gp_file_name_sizeof;
+    int flen[1];
+
+    if (!gp_file_name_is_absolute(prefix, strlen(prefix)) &&
+	gp_gettmpdir(tmpdir, &tdlen) == 0) {
+      flen[0] = gp_file_name_sizeof;
+	if (gp_file_name_combine(tmpdir, tdlen, prefix, strlen(prefix),
+			     false, fname, flen ) != gp_combine_success ) {
+	    return NULL;
+	}
+       fname[ *flen ] = 0;
+    } else {
+	strcpy(fname, prefix);
+    }
+    if (strlen(fname) + 6 >= gp_file_name_sizeof)
 	return 0;		/* file name too long */
-    strcpy(fname, prefix);
     strcat(fname, "XXXXXX");
-    mktemp(fname);
-    return fopen(fname, mode);
+   mktemp(fname);
+    f = fopen(fname, mode);
+   
+    if (f == NULL)
+	eprintf1("**** Could not open temporary file %s\n", fname);
+   return f;
 }
 
 /* Open a file with the given name, as a stream of uninterpreted bytes. */
@@ -252,106 +286,6 @@ gp_setmode_binary(FILE * pfile, bool binary)
     return 0;			/* Noop under VMS */
 }
 
-/*  Answer whether a file name contains a directory/device specification, i.e.,
- *  is absolute (not directory- or device-relative).  Since for VMS, the concept
- *  of an "absolute" file reference has no meaning.  As Ghostscript is here
- *  merely checking to see if it will make sense to paste a path to the front
- *  of the file name, we use the VMS system service SYS$FILESCAN to check that
- *  the file name has no node, device, root, or directory specification: if all
- *  four of these items are missing from the file name then it is considered to
- *  a relative file name to which a path may be prefixed. (Roots are associated
- *  with rooted logical names.)
- */
-
-bool
-gp_file_name_is_absolute(const char *fname, uint len)
-{
-    descrip str_desc;
-
-    /* SYS$FILESCAN takes a uint *, but we want to extract bits. */
-    union {
-	uint i;
-	struct {
-	    unsigned fscn$v_node:1;
-	    unsigned fscn$v_device:1;
-	    unsigned fscn$v_root:1;
-	    unsigned fscn$v_directory:1;
-	    unsigned fscn$v_name:1;
-	    unsigned fscn$v_type:1;
-	    unsigned fscn$v_version:1;
-	    unsigned fscn$v_fill_23:1;
-	} s;
-    } flags;
-    uint zero = 0;
-
-    str_desc.dsc$w_length = len;
-    str_desc.dsc$a_pointer = (char *)fname;
-    SYS$FILESCAN(&str_desc, &zero, &flags.i);
-    if (flags.s.fscn$v_directory || flags.s.fscn$v_root ||
-	flags.s.fscn$v_device || flags.s.fscn$v_node)
-	return true;
-    else
-	return false;
-}
-
-/* Answer whether the file_name references the directory	*/
-/* containing the specified path (parent). 			*/
-bool
-gp_file_name_references_parent(const char *fname, unsigned len)
-{
-    int i = 0, last_sep_pos = -gp_file_name_sizeof;
-
-    /* A file name references its parent directory if it contains -. */
-    /* inside the [ ] part of the file specification */
-    while (i < len && fname[i] != ']') {
-	if (fname[i] == '.' || fname[i] == '[') {
-	    last_sep_pos = i++;
-	    continue;
-	}
-	if (fname[i++] != '-')
-	    continue;
-        if (i > last_sep_pos + 2 || (i < len &&
-		(fname[i] != '.') && fname[i] != ']')
-	   ) 
-	    continue;
-	/* have separator followed by -. or -] */
-	return true;
-    }
-    return false;
-}
-
-/* Answer the string to be used for combining a directory/device prefix */
-/* with a base file name. The prefix directory/device is examined to	*/
-/* determine if a separator is needed and may return an empty string	*/
-const char *
-gp_file_name_concat_string(const char *prefix, uint plen)
-{
-    /*  Full VAX/VMS paths are of the form:
-
-     *    device:[root.][directory.subdirectory]filename.extension;version
-     *    logical:filename.extension;version
-     *
-     *  Roots are fairly rare and associated typically with rooted logical
-     *  names.
-     *
-     *  Examples:
-     *
-     *    DUA1:[GHOSTSCRIPT]GHOST.PS;1
-     *    THOR_DEC:[DOOF.A.B.C.D]FILE.DAT;-3
-     *    LOG:GHOST.PS  (LOG is a logical defined as DUA1:[GHOSTSCRIPT])
-     *    LOG:DOOF.DAT  (LOG is defined as DUA1, current directory is
-     *                   is used as the directory spec.)
-     *
-     */
-    if (plen > 0)
-	switch (prefix[plen - 1]) {
-	    case ':':
-	    case ']':
-		return "";
-	};
-    return ":";
-}
-
 /* ------ Wild card file search procedures ------ */
 
 private void
@@ -374,10 +308,11 @@ gp_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
     file_enum *pfen;
     uint i, len;
     char *c, *newpat;
+    bool dot_in_filename = false;
 
     pfen = gs_alloc_struct(mem, file_enum, &st_file_enum,
 			   "GP_ENUM(file_enum)");
-    newpat = (char *)gs_alloc_bytes(mem, patlen, "GP_ENUM(pattern)");
+    newpat = (char *)gs_alloc_bytes(mem, patlen + 2, "GP_ENUM(pattern)");
     if (pfen == 0 || newpat == 0) {
 	gs_free_object(mem, newpat, "GP_ENUM(pattern)");
 	gs_free_object(mem, pfen, "GP_ENUM(file_enum)");
@@ -388,6 +323,11 @@ gp_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
      *  (VAX/VMS uses the wildcard '%' to represent exactly one character
      *  and '*' to represent zero or more characters.  Any combination and
      *  number of interspersed wildcards is permitted.)
+     *
+     *  Since VMS requires "*.*" to actually return all files, we add a
+     *  special check for a path ending in "*" and change it into "*.*"
+     *  if a "." wasn't part of the file spec. Thus "[P.A.T.H]*" becomes
+     *  "[P.A.T.H]*.*" but "[P.A.T.H]*.*" or "[P.A.T.H]*.X*" are unmodified.
      */
     c = newpat;
     for (i = 0; i < patlen; pat++, i++)
@@ -400,10 +340,18 @@ gp_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
 		if (i < patlen)
 		    *c++ = *++pat;
 		break;
+	    case '.':
+	    case ']':
+		dot_in_filename = *pat == '.'; 
 	    default:
 		*c++ = *pat;
 		break;
 	}
+    /* Check for trailing "*" and see if we need to add ".*" */
+    if (pat[-1] == '*' && !dot_in_filename) {
+	*c++ = '.';
+	*c++ = '*';
+    }
     len = c - newpat;
 
     /* Pattern may not exceed 255 characters */
@@ -474,3 +422,232 @@ gp_strerror(int errnum)
 {
     return NULL;
 }
+
+/* -------------- Helpers for gp_file_name_combine_generic ------------- */
+
+uint gp_file_name_root(const char *fname, uint len)
+{   
+    /*
+     *    The root for device:[root.][directory.subdirectory]filename.extension;version
+     *	    is device:[root.][
+     *    The root for device:[directory.subdirectory]filename.extension;version
+     *	    is device:[
+     *    The root for logical:filename.extension;version
+     *	    is logical:
+     */
+    int i, j;
+
+    if (len == 0)
+	return 0;
+    /* Search for ':' */
+    for (i = 0; i < len; i++)
+	if (fname[i] == ':')
+	    break;
+    if (i == len)
+	return 0; /* No root. */
+    if (fname[i] == ':')
+	i++;
+    if (i == len || fname[i] != '[')
+	return i; 
+    /* Search for ']' */
+    i++;
+    for (j = i; j < len; j++)
+	if (fname[j] == ']')
+	    break;
+    if (j == len)
+	return i; /* No ']'. Allowed as a Ghostscript specifics. */
+    j++;
+    if (j == len)
+	return i; /* Appending "device:[directory.subdirectory]" with "filename.extension;version". */
+    if (fname[j] != '[')
+	return i; /* Can't append anything, but pass through for checking an absolute path. */
+    return j + 1; /* device:[root.][ */
+}
+
+uint gs_file_name_check_separator(const char *fname, int len, const char *item)
+{   
+    if (len > 0) {
+	/* 
+	 * Ghostscript specifics : an extended syntax like Mac OS.
+	 * We intentionally don't consider ':' and '[' as separators
+	 * in forward search, see gp_file_name_combine. 
+	 */
+	if (fname[0] == ']')
+	    return 1; /* It is a file separator. */
+	if (fname[0] == '.')
+	    return 1; /* It is a directory separator. */
+	if (fname[0] == '-') {
+	    if (fname == item + 1 && item[0] == '-')
+		return 1; /* Two or more parents, cut the first one. */
+	    return 1;
+	}
+    } else if (len < 0) {
+	if (fname[-1] == '.' || fname[-1] == ':' || fname[-1] == '[')
+	    return 1;
+    }
+    return 0;
+}
+
+bool gp_file_name_is_parent(const char *fname, uint len)
+{   /* Ghostscript specifics : an extended syntax like Mac OS. */
+    return len == 1 && fname[0] == '-';
+}
+
+bool gp_file_name_is_current(const char *fname, uint len)
+{   /* Ghostscript specifics : an extended syntax like Mac OS. */
+    return len == 0;
+}
+
+const char *gp_file_name_separator(void)
+{   return "]";
+}
+
+const char *gp_file_name_directory_separator(void)
+{   return ".";
+}
+
+const char *gp_file_name_parent(void)
+{   return "-";
+}
+
+const char *gp_file_name_current(void)
+{   return "";
+}
+
+bool gp_file_name_is_partent_allowed(void)
+{   return false;
+}
+
+bool gp_file_name_is_empty_item_meanful(void)
+{   return true;
+}
+
+gp_file_name_combine_result
+gp_file_name_combine(const char *prefix, uint plen, const char *fname, uint flen, 
+		    bool no_sibling, char *buffer, uint *blen)
+{
+    /*
+     * Reduce it to the general case.
+     *
+     * Implementation restriction : fname must not contain a part of 
+     * "device:[root.]["
+     */
+    uint rlen, flen1 = flen, plen1 = plen;
+    const char *fname1 = fname;
+  
+   if ( plen > 0 && prefix[plen-1] == '\0' )
+     plen--;
+   
+    if (plen == 0 && flen == 0) {
+	/* Not sure that we need this case. */
+	if (*blen == 0)
+	    return gp_combine_small_buffer;
+	buffer[0] = '.';
+	*blen = 1;
+    }
+    rlen = gp_file_name_root(fname, flen);
+    if (rlen > 0 || plen == 0 || flen == 0) {
+	if (rlen == 0 && plen != 0) {
+	    fname1 = prefix;
+	    flen1 = plen;
+	}
+	if (flen1 + 1 > *blen)
+	    return gp_combine_small_buffer;
+	memcpy(buffer, fname1, flen1);
+	buffer[flen1] = 0;
+	*blen = flen1;
+	return gp_combine_success;
+    }
+   
+   if ( prefix[plen - 1] == ']' && fname[ 0 ] == '-' )
+     {
+	memcpy(buffer, prefix, plen - 1 );
+	fname1 = fname + 1;
+	flen1 = flen - 1;
+	memcpy(buffer + plen - 1 , fname1, flen1);
+	memcpy(buffer + plen + flen1 - 1 , "]" , 1 );
+	buffer[plen + flen1] = 0;
+	*blen = plen + flen1;
+	return gp_combine_success;
+     }
+
+   if ( prefix[plen - 1] == ':' || (prefix[plen - 1] == ']' &&
+				     memchr(fname, ']', flen) == 0) )
+       {
+	/* Just concatenate. */
+	if (plen + flen + 1 > *blen)
+	    return gp_combine_small_buffer;
+	memcpy(buffer, prefix, plen);
+	memcpy(buffer + plen, fname, flen);
+	buffer[plen + flen] = 0;
+	*blen = plen + flen;
+	return gp_combine_success;
+    }
+   if ( memchr( prefix , '[' , plen ) == 0 &&
+	memchr( prefix , '.' , plen ) == 0 )
+     {
+	char* tmp_prefix;
+	int tmp_plen;
+	
+	if ( prefix[0] == '/' )
+	  {
+	     tmp_prefix = prefix + 1;
+	     tmp_plen = plen - 1;
+	  }
+	else
+	  {
+	     tmp_prefix = prefix;
+	     tmp_plen = plen;
+	  }
+	if ( tmp_plen + flen + 2 > *blen)
+	    return gp_combine_small_buffer;
+	memcpy(buffer, tmp_prefix, tmp_plen);
+	memcpy(buffer + tmp_plen , ":" , 1 );
+	memcpy(buffer + tmp_plen + 1, fname, flen);
+	if ( memchr( fname , '.' , flen ) != 0 )
+	  {
+	     buffer[ tmp_plen + flen + 1] = 0;
+	     *blen = tmp_plen + flen + 1;
+	  }
+	else
+	  {
+	     memcpy(buffer + tmp_plen + flen + 1 , "." , 1 );
+	     buffer[ tmp_plen + flen + 2] = 0;
+	     *blen = tmp_plen + flen + 2;
+	  }
+	return gp_combine_success;
+     }
+    if (prefix[plen - 1] != ']' && fname[0] == '[')
+        return gp_combine_cant_handle;
+    /* Unclose "][" :*/
+    if (fname[0] == '[') {
+	fname1 = fname + 1;
+	flen1 = flen - 1;
+    }
+    if (prefix[plen - 1] == ']')
+        plen1 = plen - 1;
+    return gp_file_name_combine_generic(prefix, plen1, 
+	    fname1, flen1, no_sibling, buffer, blen);
+}
+
+/* ------ Font enumeration ------ */
+ 
+ /* This is used to query the native os for a list of font names and
+  * corresponding paths. The general idea is to save the hassle of
+  * building a custom fontmap file.
+  */
+ 
+void *gp_enumerate_fonts_init(gs_memory_t *mem)
+{
+    return NULL;
+}
+         
+int gp_enumerate_fonts_next(void *enum_state, char **fontname, char **path)
+{
+    return 0;
+}
+                         
+void gp_enumerate_fonts_free(void *enum_state)
+{
+}
+
