@@ -1,20 +1,17 @@
-/* Copyright (C) 1998 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gpgetenv.h,v 1.2.6.1 2002/02/22 19:45:57 ray Exp $ */
+/* $Id: gpgetenv.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Interface to platform-specific getenv routine */
 
 #ifndef gpgetenv_INCLUDED
@@ -37,6 +34,6 @@
  * because of the terminating \0, the maximum string length is 1 less than
  * the size of the buffer.
  */
-int gp_getenv(P3(const char *key, char *ptr, int *plen));
+int gp_getenv(const char *key, char *ptr, int *plen);
 
 #endif /* gpgetenv_INCLUDED */
