@@ -1,23 +1,17 @@
-/* Copyright (C) 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gscrdp.c,v $ $Revision: 1.2.2.1 $ */
+/* $Id: gscrdp.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* CIE color rendering dictionary creation */
 #include "math_.h"
 #include "memory_.h"
@@ -251,7 +245,7 @@ param_put_cie_render1(gs_param_list * plist, gs_cie_render * pcrd,
 		    for (j = 0; j < size; ++j)
 			values[i * size + j] =
 			    frac2float((*pcrd->RenderTable.T.procs[i])
-				       (j * scale, pcrd));
+				       ((byte)(j * scale), pcrd));
 		}
 		fa.data = values;
 		fa.size = size * m;
@@ -542,7 +536,7 @@ param_get_cie_render1(gs_cie_render * pcrd, gs_param_list * plist,
 	    if (pname.size < 1 || pname.data[pname.size - 1] != 0)
 		return_error(gs_error_rangecheck);
 	    pcrd->TransformPQR.proc = TransformPQR_lookup_proc_name;
-	    pcrd->TransformPQR.proc_name = (char *)pname.data;
+	    pcrd->TransformPQR.proc_name = (const char *)pname.data;
 	    switch (code = param_read_string(plist, "TransformPQRData", &pdata)) {
 		default:	/* error */
 		    return code;
