@@ -1,20 +1,17 @@
-/* Copyright (C) 1994, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gscsepr.c,v 1.27 2004/08/04 19:36:12 stefan Exp $ */
+/* $Id: gscsepr.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Separation color space and operation definition */
 #include "memory_.h"
 #include "gx.h"
@@ -129,8 +126,30 @@ check_Separation_component_name(const gs_color_space * pcs, gs_state * pgs);
 private int
 gx_install_Separation(const gs_color_space * pcs, gs_state * pgs)
 {
-    int code = check_Separation_component_name(pcs, pgs);
+    int code;
+#if ENABLE_NAMED_COLOR_CALLBACK
+    /*
+     * Check if we want to use the named color callback color processing for
+     * this color space.
+     */
+    bool use_named_color_callback =
+	    named_color_callback_install_Separation(pgs->color_space, pgs);
 
+    pgs->color_component_map.use_named_color_callback =
+	   					 use_named_color_callback;
+    if (use_named_color_callback) {
+	/*
+	 * We are using the callback instead of the alternate tint transform
+	 * for this color space.
+	 */
+        pgs->color_space->params.separation.use_alt_cspace =
+            pgs->color_component_map.use_alt_cspace = false;
+        pgs->color_component_map.cspace_id = pcs->id;
+        return 0;
+    }
+#endif
+
+    code = check_Separation_component_name(pcs, pgs);
     if (code < 0)
        return code;
     pgs->color_space->params.separation.use_alt_cspace =
@@ -355,7 +374,7 @@ gx_concretize_Separation(const gs_client_color *pc, const gs_color_space *pcs,
 	(const gs_color_space *)&pcs->params.separation.alt_space;
     
     if (pcs->params.separation.sep_type == SEP_OTHER &&
-        pcs->params.separation.use_alt_cspace) {
+	pcs->params.separation.use_alt_cspace) {
         gs_device_n_map *map = pcs->params.separation.map;
 
 	/* Check the 1-element cache first. */
@@ -392,6 +411,12 @@ gx_remap_concrete_Separation(const frac * pconc,  const gs_color_space * pcs,
 	dprintf("gx_remap_concrete_Separation: color space id mismatch");
 #endif
 
+#if ENABLE_NAMED_COLOR_CALLBACK
+    if (pis->color_component_map.use_named_color_callback) {
+	return gx_remap_concrete_named_color_Separation(pconc, pcs, pdc,
+						       	pis, dev, select);
+    }
+#endif
     if (pis->color_component_map.use_alt_cspace) {
         const gs_color_space *pacs =
 	    (const gs_color_space *)&pcs->params.separation.alt_space;
