@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsfname.h,v 1.2.6.1 2002/02/22 19:45:57 ray Exp $ */
+/* $Id: gsfname.h 6651 2006-03-13 16:18:19Z stefan $ */
 
 #ifndef gsfname_INCLUDED
 #  define gsfname_INCLUDED
@@ -42,17 +39,17 @@ typedef struct gs_parsed_file_name_s {
 } gs_parsed_file_name_t;
 
 /* Parse a file name into device and individual name. */
-int gs_parse_file_name(P3(gs_parsed_file_name_t *, const char *, uint));
+int gs_parse_file_name(gs_parsed_file_name_t *, const char *, uint);
 
 /* Parse a real (non-device) file name and convert to a C string. */
-int gs_parse_real_file_name(P5(gs_parsed_file_name_t *, const char *, uint,
-			       gs_memory_t *, client_name_t));
+int gs_parse_real_file_name(gs_parsed_file_name_t *, const char *, uint,
+			    gs_memory_t *, client_name_t);
 
 /* Convert a file name to a C string by adding a null terminator. */
-int gs_terminate_file_name(P3(gs_parsed_file_name_t *, gs_memory_t *,
-			      client_name_t));
+int gs_terminate_file_name(gs_parsed_file_name_t *, gs_memory_t *,
+			   client_name_t);
 
 /* Free a file name that was copied to a C string. */
-void gs_free_file_name(P2(gs_parsed_file_name_t *, client_name_t));
+void gs_free_file_name(gs_parsed_file_name_t *, client_name_t);
 
 #endif /* gsfname_INCLUDED */
