@@ -1,22 +1,20 @@
-/* Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsht1.c,v 1.3.2.1 2002/02/22 19:45:57 ray Exp $ */
+/*$Id: gsht1.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Extended halftone operators for Ghostscript library */
 #include "memory_.h"
+#include "string_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gsstruct.h"
@@ -25,29 +23,24 @@
 #include "gxdevice.h"		/* for gzht.h */
 #include "gzht.h"
 
-/* Define the size of the halftone tile cache. */
-#define max_tile_bytes_LARGE 4096
-#define max_tile_bytes_SMALL 512
-#if arch_small_memory
-#  define max_tile_cache_bytes max_tile_bytes_SMALL
-#else
-#  define max_tile_cache_bytes\
-     (gs_debug_c('.') ? max_tile_bytes_SMALL : max_tile_bytes_LARGE)
-#endif
+#include "gxwts.h"
+#include "gswts.h"
 
 /* Imports from gscolor.c */
-void load_transfer_map(P3(gs_state *, gx_transfer_map *, floatp));
+void load_transfer_map(gs_state *, gx_transfer_map *, floatp);
 
 /* Forward declarations */
-private int process_spot(P4(gx_ht_order *, gs_state *,
-			    gs_spot_halftone *, gs_memory_t *));
-private int process_threshold(P4(gx_ht_order *, gs_state *,
-				 gs_threshold_halftone *, gs_memory_t *));
-private int process_threshold2(P4(gx_ht_order *, gs_state *,
-				  gs_threshold2_halftone *,
-				  gs_memory_t *));
-private int process_client_order(P4(gx_ht_order *, gs_state *,
-				gs_client_order_halftone *, gs_memory_t *));
+private int process_spot(gx_ht_order *, gs_state *,
+			 gs_spot_halftone *, gs_memory_t *);
+private int process_threshold(gx_ht_order *, gs_state *,
+			      gs_threshold_halftone *, gs_memory_t *);
+private int process_threshold2(gx_ht_order *, gs_state *,
+			       gs_threshold2_halftone *, gs_memory_t *);
+private int process_client_order(gx_ht_order *, gs_state *,
+				 gs_client_order_halftone *, gs_memory_t *);
+private int
+gs_sethalftone_try_wts(gs_halftone *pht, gs_state *pgs,
+		       gx_device_halftone *pdht);
 
 /* Structure types */
 public_st_halftone_component();
@@ -165,8 +158,11 @@ gs_sethalftone_allocated(gs_state * pgs, gs_halftone * pht)
     if (code < 0)
 	return code;
     dev_ht.rc.memory = pht->rc.memory;
-    return gx_ht_install(pgs, pht, &dev_ht);
+    if ((code = gx_ht_install(pgs, pht, &dev_ht)) < 0)
+        gx_device_halftone_release(&dev_ht, pht->rc.memory);
+    return code;
 }
+
 /* Prepare the halftone, but don't install it. */
 int
 gs_sethalftone_prepare(gs_state * pgs, gs_halftone * pht,
@@ -176,18 +172,16 @@ gs_sethalftone_prepare(gs_state * pgs, gs_halftone * pht,
     gx_ht_order_component *pocs = 0;
     int code = 0;
 
+    if (gs_currentusewts() && gs_sethalftone_try_wts(pht, pgs, pdht) == 0)
+	return 0;
+
     switch (pht->type) {
 	case ht_type_colorscreen:
 	    {
 		gs_screen_halftone *phc =
-		pht->params.colorscreen.screens.indexed;
-		static const gs_ht_separation_name cnames[4] =
-		{
-		    gs_ht_separation_Default, gs_ht_separation_Red,
-		    gs_ht_separation_Green, gs_ht_separation_Blue
-		};
-		static const int cindex[4] =
-		{3, 0, 1, 2};
+		    pht->params.colorscreen.screens.indexed;
+		static const int cindex[4] = {3, 0, 1, 2};
+		static const char * color_names[4] = {"Gray", "Red", "Green", "Blue"};
 		int i;
 
 		pocs = gs_alloc_struct_array(mem, 4,
@@ -206,25 +200,11 @@ gs_sethalftone_prepare(gs_state * pgs, gs_halftone * pht,
 		    if (code < 0)
 			break;
 		    poc->corder = senum.order;
-		    poc->cname = cnames[i];
+		    poc->comp_number = gs_color_name_component_number(pgs->device,
+				color_names[i], strlen(color_names[i]), pht->type);
+		    poc->cname = 0;  /* name index values are not known (or needed) */
 		    if (i == 0)	/* Gray = Default */
-			pdht->order = senum.order;
-		    else {
-			uint tile_bytes = senum.order.raster *
-			    (senum.order.num_bits / senum.order.width);
-			uint num_tiles =
-			    max_tile_cache_bytes / tile_bytes + 1;
-			gx_ht_cache *pcache =
-			    gx_ht_alloc_cache(mem, num_tiles,
-					      tile_bytes * num_tiles);
-
-			if (pcache == 0) {
-			    code = gs_note_error(gs_error_VMerror);
-			    break;
-			}
-			poc->corder.cache = pcache;
-			gx_ht_init_cache(pcache, &poc->corder);
-		    }
+			pdht->order = poc->corder;	/* Save default value */
 		}
 		if (code < 0)
 		    break;
@@ -276,9 +256,9 @@ gs_sethalftone_prepare(gs_state * pgs, gs_halftone * pht,
 		    return_error(gs_error_VMerror);
 		poc_next = pocs + 1;
 		for (i = 0; i < count; i++, phc++) {
-		    gx_ht_order_component *poc;
+		    gx_ht_order_component *poc = poc_next;		    
 
-		    if (phc->cname == gs_ht_separation_Default) {
+		    if (phc->comp_number == GX_DEVICE_COLOR_MAX_COMPONENTS) {
 			if (have_Default) {
 			    /* Duplicate Default */
 			    code = gs_note_error(gs_error_rangecheck);
@@ -292,6 +272,8 @@ gs_sethalftone_prepare(gs_state * pgs, gs_halftone * pht,
 			break;
 		    } else
 			poc = poc_next++;
+
+		    poc->comp_number = phc->comp_number;
 		    poc->cname = phc->cname;
 		    switch (phc->type) {
 			case ht_type_spot:
@@ -316,20 +298,6 @@ gs_sethalftone_prepare(gs_state * pgs, gs_halftone * pht,
 		    }
 		    if (code < 0)
 			break;
-		    if (poc != pocs) {
-			gx_ht_cache *pcache =
-			gx_ht_alloc_cache(mem, 4,
-					  poc->corder.raster *
-					  (poc->corder.num_bits /
-					   poc->corder.width) * 4);
-
-			if (pcache == 0) {
-			    code = gs_note_error(gs_error_VMerror);
-			    break;
-			}
-			poc->corder.cache = pcache;
-			gx_ht_init_cache(pcache, &poc->corder);
-		    }
 		}
 		if (code < 0)
 		    break;
@@ -366,13 +334,16 @@ process_transfer(gx_ht_order * porder, gs_state * pgs,
 
     if (proc == 0 && pmc->proc == 0)
 	return 0;
-    pmap = gs_alloc_struct(mem, gx_transfer_map, &st_transfer_map,
-			   "process_transfer");
-    if (pmap == 0)
-	return_error(gs_error_VMerror);
+    /*
+     * The transfer funtion is referenced by the order, so start the
+     * reference count at 1.
+     */
+    rc_alloc_struct_1(pmap, gx_transfer_map, &st_transfer_map, mem,
+		      return_error(gs_error_VMerror),
+		      "process_transfer");
     pmap->proc = proc;		/* 0 => use closure */
     pmap->closure = *pmc;
-    pmap->id = gs_next_ids(1);
+    pmap->id = gs_next_ids(mem, 1);
     load_transfer_map(pgs, pmap, 0.0);
     porder->transfer = pmap;
     return 0;
@@ -575,3 +546,132 @@ process_client_order(gx_ht_order * porder, gs_state * pgs,
     return process_transfer(porder, pgs, NULL,
 			    &phcop->transfer_closure, mem);
 }
+
+private const gx_ht_order_procs_t wts_order_procs = { 0
+};
+
+/**
+ * gs_sethalftone_try_wts: Try creating a wts-based device halftone.
+ * @pht: Client halftone.
+ * @pdht: Device halftone to initialize.
+ *
+ * Tries initializing @pdht based on data from @pht, using WTS.
+ *
+ * Return value: 0 on success, 1 to indicate that the initialization
+ * was not done, and that the legacy initialization code path should
+ * be used.
+ **/
+private int
+gs_sethalftone_try_wts(gs_halftone *pht, gs_state *pgs,
+		       gx_device_halftone *pdht)
+{
+    gx_device *dev = pgs->device;
+    int num_comps = dev->color_info.num_components;
+    int depth = dev->color_info.depth;
+
+    if (pht->type != ht_type_multiple)
+	/* Only work with Type 5 halftones. todo: we probably want
+	   to relax this. */
+	return 1;
+
+    if_debug2('h', "[h]%s, num_comp = %d\n",
+	      dev->color_info.separable_and_linear == GX_CINFO_SEP_LIN ? "Separable and linear" : "Not separable and linear!",
+	      pht->params.multiple.num_comp);
+
+    if (dev->color_info.separable_and_linear != GX_CINFO_SEP_LIN &&
+	pht->params.multiple.num_comp > 1)
+	/* WTS is only enabled for separable or monochrome devices. */
+	return 1;
+
+    /* only work with bilevel (not multilevel) devices */
+    if (depth > num_comps) {
+        if (depth >= 2 * num_comps)
+	    return 1;
+        if (dev->color_info.gray_index != GX_CINFO_COMP_NO_INDEX &&
+            (dev->color_info.max_gray > 1 || 
+	    (num_comps > 1 && dev->color_info.max_color > 1)))
+            return 1;
+    }
+
+    if (pht->type == ht_type_multiple) {
+	gs_halftone_component *components = pht->params.multiple.components;
+	uint num_comp = pht->params.multiple.num_comp;
+	int i;
+	gx_ht_order_component *pocs;
+	gx_ht_order_component *poc_next;
+	int code = 0;
+	bool have_Default = false;
+
+	for (i = 0; i < num_comp; i++) {
+	    if (components[i].type != ht_type_spot)
+		return 1;
+	    else {
+		gs_spot_halftone *spot = &components[i].params.spot;
+		if (!spot->accurate_screens)
+		    return 1;
+	    }
+	}
+
+	pocs = gs_alloc_struct_array( pgs->memory,
+                                      num_comp,
+                                      gx_ht_order_component,
+                                      &st_ht_order_component_element,
+                                      "gs_sethalftone_try_wts" );
+	/* pocs = malloc(num_comp * sizeof(gx_ht_order_component)); */
+	poc_next = &pocs[1];
+	for (i = 0; i < num_comp; i++) {
+	    gs_halftone_component *component = &components[i];
+	    gs_spot_halftone *spot = &component->params.spot;
+	    gs_screen_halftone *h = &spot->screen;
+	    gx_wts_cell_params_t *wcp;
+	    gs_wts_screen_enum_t *wse;
+	    gs_matrix imat;
+	    gx_ht_order_component *poc;
+
+	    if (component->comp_number == GX_DEVICE_COLOR_MAX_COMPONENTS) {
+		if (have_Default) {
+		    /* Duplicate Default */
+		    code = gs_note_error(gs_error_rangecheck);
+		    break;
+		}
+		poc = pocs;
+		have_Default = true;
+	    } else if (i == num_comp - 1 && !have_Default) {
+		/* No Default */
+		code = gs_note_error(gs_error_rangecheck);
+		break;
+	    } else
+		poc = poc_next++;
+
+	    gs_deviceinitialmatrix(gs_currentdevice(pgs), &imat);
+
+	    wcp = wts_pick_cell_size(h, &imat);
+	    wse = gs_wts_screen_enum_new(wcp);
+
+	    poc->corder.wse = wse;
+	    poc->corder.wts = NULL;
+	    poc->corder.procs = &wts_order_procs;
+	    poc->corder.data_memory = NULL;
+	    poc->corder.num_levels = 0;
+	    poc->corder.num_bits = 0;
+	    poc->corder.levels = NULL;
+	    poc->corder.bit_data = NULL;
+	    poc->corder.cache = NULL;
+	    poc->corder.transfer = NULL;
+	    poc->comp_number = component->comp_number;
+	    poc->cname = component->cname;
+	    code = process_transfer( &poc->corder,
+                                     pgs,
+                                     spot->transfer,
+                                     &spot->transfer_closure,
+                                     pgs->memory );
+	    if (code < 0)
+		break;
+	}
+	/* todo: cleanup on error */
+	pdht->components = pocs;
+	pdht->num_comp = num_comp;
+	return code;
+    }
+    return 1;
+}
