@@ -1,20 +1,17 @@
-/* Copyright (C) 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsimage.c,v 1.12 2004/08/04 19:36:12 stefan Exp $ */
+/* $Id: gsimage.c 6673 2006-03-22 08:43:11Z leonardo $ */
 /* Image setup procedures for Ghostscript library */
 #include "memory_.h"
 #include "gx.h"
@@ -284,19 +281,15 @@ begin_planes(gs_image_enum *penum)
     next_plane(penum);
 }
 
-/* Start processing a general image. */
-int
-gs_image_enum_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
-		   const gs_data_image_t * pim, gs_state *pgs)
-{
-    return gs_image_common_init(penum, pie, pim, pgs->memory,
-				(pgs->in_charpath ? NULL :
-				 gs_currentdevice_inline(pgs)));
-}
-int
+private int
 gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
-	    const gs_data_image_t * pim, gs_memory_t * mem, gx_device * dev)
+	    const gs_data_image_t * pim, gx_device * dev)
 {
+    /*
+     * HACK : For a compatibility with gs_image_cleanup_and_free_enum,
+     * penum->memory must be initialized in advance 
+     * with the memory heap that owns *penum.
+     */
     int i;
 
     if (pim->Width == 0 || pim->Height == 0) {
@@ -304,7 +297,6 @@ gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
 	return 1;
     }
     image_enum_init(penum);
-    penum->memory = mem;
     penum->dev = dev;
     penum->info = pie;
     penum->num_planes = pie->num_planes;
@@ -332,6 +324,18 @@ gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
     return 0;
 }
 
+/* Initialize an enumerator for a general image. 
+   penum->memory must be initialized in advance.
+*/
+int
+gs_image_enum_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
+		   const gs_data_image_t * pim, gs_state *pgs)
+{
+    return gs_image_common_init(penum, pie, pim,
+				(pgs->in_charpath ? NULL :
+				 gs_currentdevice_inline(pgs)));
+}
+
 /* Return the set of planes wanted. */
 const byte *
 gs_image_planes_wanted(gs_image_enum *penum)
@@ -558,3 +562,13 @@ gs_image_cleanup(gs_image_enum * penum)
     /* Don't free the local enumerator -- the client does that. */
     return code;
 }
+
+/* Clean up after processing an image and free the enumerator. */
+int
+gs_image_cleanup_and_free_enum(gs_image_enum * penum)
+{
+    int code = gs_image_cleanup(penum);
+
+    gs_free_object(penum->memory, penum, "gs_image_cleanup_and_free_enum");
+    return code;
+}
