@@ -1,23 +1,17 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gsmisc.c,v $ $Revision: 1.8.2.2 $ */
+/* $Id: gsmisc.c 6656 2006-03-13 21:11:43Z stefan $ */
 /* Miscellaneous utilities for Ghostscript library */
 
 /*
@@ -52,8 +46,50 @@ orig_sqrt(double x)
 #include "gxfarith.h"
 #include "gxfixed.h"
 
-/* Define private replacements for stdin, stdout, and stderr. */
-FILE *gs_stdio[3];
+/* ------ Redirected stdout and stderr  ------ */
+
+#include <stdarg.h>
+#define PRINTF_BUF_LENGTH 1024
+
+int outprintf(const gs_memory_t *mem, const char *fmt, ...)
+{
+    int count;
+    char buf[PRINTF_BUF_LENGTH];
+    va_list args;
+
+    va_start(args, fmt);
+
+    count = vsprintf(buf, fmt, args);
+    outwrite(mem, buf, count);
+    if (count >= PRINTF_BUF_LENGTH) {
+	count = sprintf(buf, 
+	    "PANIC: printf exceeded %d bytes.  Stack has been corrupted.\n", 
+	    PRINTF_BUF_LENGTH);
+	outwrite(mem, buf, count);
+    }
+    va_end(args);
+    return count;
+}
+
+int errprintf(const char *fmt, ...)
+{
+    int count;
+    char buf[PRINTF_BUF_LENGTH];
+    va_list args;
+
+    va_start(args, fmt);
+
+    count = vsprintf(buf, fmt, args);
+    errwrite(buf, count);
+    if (count >= PRINTF_BUF_LENGTH) {
+	count = sprintf(buf, 
+	    "PANIC: printf exceeded %d bytes.  Stack has been corrupted.\n", 
+	    PRINTF_BUF_LENGTH);
+	errwrite(buf, count);
+    }
+    va_end(args);
+    return count;
+}
 
 /* ------ Debugging ------ */
 
@@ -79,13 +115,13 @@ const char *const dprintf_file_only_format = "%10s(unkn): ";
 /*
  * Define the trace printout procedures.  We always include these, in case
  * other modules were compiled with DEBUG set.  Note that they must use
- * fprintf, not fput[cs], because of the way that stdout is implemented on
- * Windows platforms.
+ * out/errprintf, not fprintf nor fput[cs], because of the way that 
+ * stdout/stderr are implemented on DLL/shared library builds.
  */
 void
 dflush(void)
 {
-    fflush(dstderr);
+    errflush();
 }
 private const char *
 dprintf_file_tail(const char *file)
@@ -100,53 +136,56 @@ dprintf_file_tail(const char *file)
 }
 #if __LINE__			/* compiler provides it */
 void
-dprintf_file_and_line(FILE * f, const char *file, int line)
+dprintf_file_and_line(const char *file, int line)
 {
     if (gs_debug['/'])
-	fprintf(f, dprintf_file_and_line_format,
+	dpf(dprintf_file_and_line_format,
 		dprintf_file_tail(file), line);
 }
 #else
 void
-dprintf_file_only(FILE * f, const char *file)
+dprintf_file_only(const char *file)
 {
     if (gs_debug['/'])
-	fprintf(f, dprintf_file_only_format, dprintf_file_tail(file));
+	dpf(dprintf_file_only_format, dprintf_file_tail(file));
 }
 #endif
 void
-printf_program_ident(FILE * f, const char *program_name,
-		     long revision_number)
+printf_program_ident(const gs_memory_t *mem, const char *program_name, long revision_number)
 {
     if (program_name)
-	fprintf(f, (revision_number ? "%s " : "%s"), program_name);
+        outprintf(mem, (revision_number ? "%s " : "%s"), program_name);
     if (revision_number) {
 	int fpart = revision_number % 100;
 
-	fprintf(f, (fpart == 0 ? "%d.%d" : "%d.%02d"),
-		(int)(revision_number / 100), fpart);
+	outprintf(mem, "%d.%02d", (int)(revision_number / 100), fpart);
     }
 }
 void
-eprintf_program_ident(FILE * f, const char *program_name,
+eprintf_program_ident(const char *program_name,
 		      long revision_number)
 {
     if (program_name) {
-	printf_program_ident(f, program_name, revision_number);
-	fprintf(f, ": ");
+	epf((revision_number ? "%s " : "%s"), program_name);
+	if (revision_number) {
+	    int fpart = revision_number % 100;
+
+	    epf("%d.%02d", (int)(revision_number / 100), fpart);
+	}
+	epf(": ");
     }
 }
 #if __LINE__			/* compiler provides it */
 void
-lprintf_file_and_line(FILE * f, const char *file, int line)
+lprintf_file_and_line(const char *file, int line)
 {
-    fprintf(f, "%s(%d): ", file, line);
+    epf("%s(%d): ", file, line);
 }
 #else
 void
 lprintf_file_only(FILE * f, const char *file)
 {
-    fprintf(f, "%s(?): ", file);
+    epf("%s(?): ", file);
 }
 #endif
 
@@ -168,18 +207,82 @@ gs_log_error(int err, const char *file, int line)
 
 /* Check for interrupts before a return. */
 int
-gs_return_check_interrupt(int code)
+gs_return_check_interrupt(const gs_memory_t *mem, int code)
 {
     if (code < 0)
 	return code;
     {
-	int icode = gp_check_interrupts();
+	int icode = gp_check_interrupts(mem);
 
 	return (icode == 0 ? code :
 		gs_note_error((icode > 0 ? gs_error_interrupt : icode)));
     }
 }
 
+
+int gs_throw_imp(const char *func, const char *file, int line, int op, int code, const char *fmt, ...)
+{
+    char msg[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    vsprintf(msg, fmt, ap);
+    msg[sizeof(msg) - 1] = 0;
+    va_end(ap);
+
+    if (!gs_debug_c('#')) {
+	; /* NB: gs_log_errors  
+	   * we could disable these printfs, and probably will when, 
+	   * the code becomes more stable: 
+	   * return code;
+	   */
+    }
+       
+
+    /* throw */
+    if (op == 0)
+	errprintf(NULL, "+ %s:%d: %s(): %s\n", file, line, func, msg);
+
+    /* rethrow */
+    if (op == 1)
+	errprintf(NULL, "| %s:%d: %s(): %s\n", file, line, func, msg);
+
+    /* catch */
+    if (op == 2)
+	errprintf(NULL, "- %s:%d: %s(): %s\n", file, line, func, msg);
+
+    /* warn */
+    if (op == 3)
+	errprintf(NULL, "  %s:%d: %s(): %s\n", file, line, func, msg);
+
+    return code;
+}
+
+const char *gs_errstr(int code)
+{
+    switch (code) {
+    default:
+    case gs_error_unknownerror: return "unknownerror";
+    case gs_error_interrupt: return "interrupt";
+    case gs_error_invalidaccess: return "invalidaccess";
+    case gs_error_invalidfileaccess: return "invalidfileaccess";
+    case gs_error_invalidfont: return "invalidfont";
+    case gs_error_ioerror: return "ioerror";
+    case gs_error_limitcheck: return "limitcheck";
+    case gs_error_nocurrentpoint: return "nocurrentpoint";
+    case gs_error_rangecheck: return "rangecheck";
+    case gs_error_typecheck: return "typecheck";
+    case gs_error_undefined: return "undefined";
+    case gs_error_undefinedfilename: return "undefinedfilename";
+    case gs_error_undefinedresult: return "undefinedresult";
+    case gs_error_VMerror: return "vmerror";
+    case gs_error_unregistered: return "unregistered";
+    case gs_error_hit_detected: return "hit_detected";
+    case gs_error_Fatal: return "Fatal";
+    }
+}
+
+
 /* ------ Substitutes for missing C library functions ------ */
 
 #ifdef MEMORY__NEED_MEMMOVE	/* see memory_.h */
@@ -489,7 +592,7 @@ ilog2(int n)
     while (m >= 16)
 	m >>= 4, l += 4;
     return
-	(m <= 1 ? 0 :
+	(m <= 1 ? l :
 	 "\000\000\001\001\002\002\002\002\003\003\003\003\003\003\003\003"[m] + l);
 }
 
@@ -669,8 +772,14 @@ set_fixed2double_(double *pd, fixed x, int frac_bits)
 /*
  * If doubles aren't wide enough, we lose too much precision by using double
  * arithmetic: we have to use the slower, accurate fixed-point algorithm.
+ * See the simpler implementation below for more information.
  */
-#if USE_FPU_FIXED || (arch_double_mantissa_bits < arch_sizeof_long * 12)
+#define MAX_OTHER_FACTOR_BITS\
+  (ARCH_DOUBLE_MANTISSA_BITS - ARCH_SIZEOF_FIXED * 8)
+#define ROUND_BITS\
+  (ARCH_SIZEOF_FIXED * 8 * 2 - ARCH_DOUBLE_MANTISSA_BITS)
+
+#if USE_FPU_FIXED || ROUND_BITS >= MAX_OTHER_FACTOR_BITS - 1
 
 #ifdef DEBUG
 struct {
@@ -826,38 +935,58 @@ fixed_mult_quo(fixed signed_A, fixed B, fixed C)
     }
 }
 
-#else				/* can approximate using doubles */
+#else				/* use doubles */
 
 /*
- * Compute A * B / C as above.  Since a double doesn't have enough bits to
- * represent the product of two longs, we have to do it in two steps.
+ * Compute A * B / C as above using doubles.  If floating point is
+ * reasonably fast, this is much faster than the fixed-point algorithm.
  */
 fixed
 fixed_mult_quo(fixed signed_A, fixed B, fixed C)
 {
-#define MAX_OTHER_FACTOR\
-  (1L << (arch_double_mantissa_bits - sizeof(fixed) * 8))
+    /*
+     * Check whether A * B will fit in the mantissa of a double.
+     */
+#define MAX_OTHER_FACTOR (1L << MAX_OTHER_FACTOR_BITS)
     if (B < MAX_OTHER_FACTOR || any_abs(signed_A) < MAX_OTHER_FACTOR) {
-	/* The double computation will be exact. */
-	return (fixed)floor((double)signed_A * B / C);
-    }
 #undef MAX_OTHER_FACTOR
-    {
-	/* Use 2 double steps. */
-	fixed bhi = B >> half_bits;
-	fixed qhi = (fixed)floor((double)signed_A * bhi / C);
-	fixed rhi = signed_A * bhi - qhi * C;
-	fixed blo = B & half_mask;
-	fixed qlo =
-	    (fixed)floor(((double)rhi * (1L << half_bits) +
-			  (double)signed_A * blo) / C);
+	/*
+	 * The product fits, so a straightforward double computation
+	 * will be exact.
+	 */
+	return (fixed)floor((double)signed_A * B / C);
+    } else {
+	/*
+	 * The product won't fit.  However, the approximate product will
+	 * only be off by at most +/- 1/2 * (1 << ROUND_BITS) because of
+	 * rounding.  If we add 1 << ROUND_BITS to the value of the product
+	 * (i.e., 1 in the least significant bit of the mantissa), the
+	 * result is always greater than the correct product by between 1/2
+	 * and 3/2 * (1 << ROUND_BITS).  We know this is less than C:
+	 * because of the 'if' just above, we know that B >=
+	 * MAX_OTHER_FACTOR; since B <= C, we know C >= MAX_OTHER_FACTOR;
+	 * and because of the #if that chose between the two
+	 * implementations, we know that C >= 2 * (1 << ROUND_BITS).  Hence,
+	 * the quotient after dividing by C will be at most 1 too large.
+	 */
+	fixed q =
+	    (fixed)floor(((double)signed_A * B + (1L << ROUND_BITS)) / C);
 
-	return (qhi << half_bits) + qlo;
+	/*
+	 * Compute the remainder R.  If the quotient was correct,
+	 * 0 <= R < C.  If the quotient was too high, -C <= R < 0.
+	 */
+	if (signed_A * B - q * C < 0)
+	    --q;
+	return q;
     }
 }
 
 #endif
 
+#undef MAX_OTHER_FACTOR_BITS
+#undef ROUND_BITS
+
 #undef num_bits
 #undef half_bits
 #undef half_mask
@@ -1069,10 +1198,16 @@ gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 static const int isincos[5] =
 {0, 1, 0, -1, 0};
 
+/* GCC with -ffast-math compiles ang/90. as ang*(1/90.), losing precission.
+ * This doesn't happen when the numeral is replaced with a non-const variable.
+ * So we define the variable to work around the GCC problem. 
+ */
+static double const_90_degrees = 90.;
+
 double
 gs_sin_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/*
@@ -1087,7 +1222,7 @@ gs_sin_degrees(double ang)
 double
 gs_cos_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
@@ -1099,7 +1234,7 @@ gs_cos_degrees(double ang)
 void
 gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
