@@ -1,20 +1,17 @@
-/* Copyright (C) 1998 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsparams.h,v 1.2.6.1 2002/02/22 19:45:58 ray Exp $ */
+/* $Id: gsparams.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Serializer/expander for gs_parm_list's */
 
 #ifndef gsparams_INCLUDED
@@ -36,13 +33,13 @@
  * Serialize the contents of a gs_param_list, including sub-collections,
  * onto a stream.  The list must be in READ mode.
  */
-int gs_param_list_puts(P2(stream *dest, gs_param_list *list));
+int gs_param_list_puts(stream *dest, gs_param_list *list);
 
 /*
  * Unserialize a parameter list, including sub-collections, from a stream.
  * The list must be in WRITE mode.
  */
-int gs_param_list_gets(P3(stream *src, gs_param_list *list, gs_memory_t *mem));
+int gs_param_list_gets(stream *src, gs_param_list *list, gs_memory_t *mem);
 
 #else	/****************/
 
@@ -55,14 +52,14 @@ int gs_param_list_gets(P3(stream *src, gs_param_list *list, gs_memory_t *mem));
  * less than or equal to the buffer size.  Note that the buffer may be
  * NULL, in which case nothing is stored (but the size is still returned).
  */
-int gs_param_list_serialize(P3(gs_param_list *list, byte *buf, int buf_size));
+int gs_param_list_serialize(gs_param_list *list, byte *buf, int buf_size);
 
 /*
  * Unserialize a parameter list from a buffer.  Return the actual number
  * of bytes occupied by the list, or a negative error code.  The buffer
  * must be void * aligned.
  */
-int gs_param_list_unserialize(P2(gs_param_list *list, const byte *buf));
+int gs_param_list_unserialize(gs_param_list *list, const byte *buf);
 
 #endif	/****************/
 
