@@ -1,20 +1,17 @@
-/* Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsptype1.c,v 1.21 2005/03/23 19:45:15 dan Exp $ */
+/* $Id: gsptype1.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* PatternType 1 pattern implementation */
 #include "math_.h"
 #include "gx.h"
@@ -697,10 +694,9 @@ bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
     else
 	for (n = pim->Height; n > 0 && code >= 0; dp += raster, --n)
 	    code = gs_image_next(pen, dp, nbytes, &used);
-    code1 = gs_image_cleanup(pen);
+    code1 = gs_image_cleanup_and_free_enum(pen);
     if (code >= 0 && code1 < 0)
 	code = code1;
-    gs_free_object(gs_state_memory(pgs), pen, "bitmap_paint");
     return code;
 }
 
@@ -996,8 +992,16 @@ gx_dc_pattern_save_dc(
     gx_device_color_saved * psdc )
 {
     psdc->type = pdevc->type;
-    psdc->colors.pattern.id = pdevc->ccolor.pattern->pattern_id;
-    psdc->colors.pattern.phase = pdevc->phase;
+    if (pdevc->ccolor_valid) {
+	psdc->colors.pattern.id = pdevc->ccolor.pattern->pattern_id;
+	psdc->colors.pattern.phase = pdevc->phase;
+    } {
+	/* The client color has been changed to a non-pattern color,
+	   but device color has not been created yet. 
+	 */
+	psdc->colors.pattern.id = gs_no_id;
+	psdc->colors.pattern.phase.x = psdc->colors.pattern.phase.y = 0;
+    }
 }
 
 /*
