@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsshade.h,v 1.8 2004/09/13 15:52:49 igor Exp $ */
+/* $Id: gsshade.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Definitions for shading */
 
 #ifndef gsshade_INCLUDED
@@ -74,11 +71,12 @@ typedef struct gx_device_s gx_device;
  * clipping device if necessary.
  */
 #define SHADING_FILL_RECTANGLE_PROC(proc)\
-  int proc(const gs_shading_t *psh, const gs_rect *prect, gx_device *dev,\
+  int proc(const gs_shading_t *psh, const gs_rect *prect,\
+	   const gs_fixed_rect *prect_clip, gx_device *dev,\
 	   gs_imager_state *pis)
 typedef SHADING_FILL_RECTANGLE_PROC((*shading_fill_rectangle_proc_t));
-#define gs_shading_fill_rectangle(psh, prect, dev, pis)\
-  ((psh)->head.procs.fill_rectangle(psh, prect, dev, pis))
+#define gs_shading_fill_rectangle(psh, prect, prect_clip, dev, pis)\
+  ((psh)->head.procs.fill_rectangle(psh, prect, prect_clip, dev, pis))
 
 /* Define the generic shading structures. */
 typedef struct gs_shading_procs_s {
