@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxccache.c,v 1.33 2005/03/31 06:12:42 dan Exp $ */
+/* $Id: gxccache.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Fast case character cache routines for Ghostscript library */
 #include "memory_.h"
 #include "gx.h"
@@ -85,9 +82,8 @@ gx_lookup_fm_pair(gs_font * pfont, const gs_matrix *char_tm,
     float mxx, mxy, myx, myy;
     gs_font *font = pfont;
     register gs_font_dir *dir = font->dir;
-    register cached_fm_pair *pair =
-    dir->fmcache.mdata + dir->fmcache.mnext;
-    int count = dir->fmcache.mmax;
+    register cached_fm_pair *pair = dir->fmcache.mdata + dir->fmcache.used;
+    int count = dir->fmcache.msize;
     gs_uid uid;
 
     gx_compute_ccache_key(pfont, char_tm, log2_scale, design_grid,
@@ -99,10 +95,7 @@ gx_lookup_fm_pair(gs_font * pfont, const gs_matrix *char_tm,
 	if (uid_is_valid(&uid))
 	    font = 0;
     }
-    while (count--) {
-	if (pair == dir->fmcache.mdata)
-	    pair += dir->fmcache.mmax;
-	pair--;
+    for (;count--; pair = dir->fmcache.mdata + pair->next) {
 	/* We have either a non-zero font and an invalid UID, */
 	/* or a zero font and a valid UID. */
 	/* We have to break up the test */
@@ -119,6 +112,8 @@ gx_lookup_fm_pair(gs_font * pfont, const gs_matrix *char_tm,
 	if (pair->mxx == mxx && pair->mxy == mxy &&
 	    pair->myx == myx && pair->myy == myy
 	    && pair->design_grid == design_grid) {
+	    int code; 
+
 	    if (pair->font == 0) {
 		pair->font = pfont;
 		if_debug2('k', "[k]updating pair 0x%lx with font 0x%lx\n",
@@ -127,6 +122,9 @@ gx_lookup_fm_pair(gs_font * pfont, const gs_matrix *char_tm,
 		if_debug2('k', "[k]found pair 0x%lx: font=0x%lx\n",
 			  (ulong) pair, (ulong) pair->font);
 	    }
+	    code = gx_touch_fm_pair(dir, pair);
+	    if (code < 0)
+		return code;
 	    *ppair = pair;
 	    return 0;
 	}
@@ -163,10 +161,12 @@ gx_lookup_cached_char(const gs_font * pfont, const cached_fm_pair * pair,
 }
 
 /* Look up a character in an external font. */
-/* Return the cached_char or 0. */
-cached_char *
+/* If found, stores a pointer to the cached_char into *pcc and returns 1 */
+/* Otherwise stores NULL into *pcc and returns 0 or error. */
+/* Note it must initialise *pcc in any case. */
+int
 gx_lookup_xfont_char(const gs_state * pgs, cached_fm_pair * pair,
-		     gs_char chr, gs_glyph glyph, int wmode)
+		     gs_char chr, gs_glyph glyph, int wmode, cached_char **pcc)
 {
     gs_font *font = pair->font;
     int enc_index;
@@ -176,9 +176,11 @@ gx_lookup_xfont_char(const gs_state * pgs, cached_fm_pair * pair,
     gs_point wxy;
     gs_int_rect bbox;
     cached_char *cc;
+    int code;
 
+    *pcc = NULL;
     if (font == 0)
-	return NULL;
+	return 0;
     enc_index =
 	(font->FontType == ft_composite ? -1 :
 	 ((gs_font_base *) font)->nearest_encoding_index);
@@ -188,14 +190,14 @@ gx_lookup_xfont_char(const gs_state * pgs, cached_fm_pair * pair,
     }
     xf = pair->xfont;
     if (xf == 0)
-	return NULL;
+	return 0;
     {
 	const gx_xfont_procs *procs = xf->common.procs;
 	gs_const_string gstr;
 	int code = font->procs.glyph_name(font, glyph, &gstr);
 
 	if (code < 0)
-	    return NULL;
+	    return 0;
 	if (enc_index >= 0 && ((gs_font_base *)font)->encoding_index < 0) {
 	    /*
 	     * Use the registered encoding only if this glyph
@@ -211,16 +213,16 @@ gx_lookup_xfont_char(const gs_state * pgs, cached_fm_pair * pair,
 	}
 	xg = procs->char_xglyph(xf, chr, enc_index, glyph, &gstr);
 	if (xg == gx_no_xglyph)
-	    return NULL;
+	    return 0;
 	if ((*procs->char_metrics) (xf, xg, wmode, &wxy, &bbox) < 0)
-	    return NULL;
+	    return 0;
     }
     log2_scale.x = log2_scale.y = 1;
     cc = gx_alloc_char_bits(font->dir, NULL, NULL, 
 		(ushort)(bbox.q.x - bbox.p.x), (ushort)(bbox.q.y - bbox.p.y), 
 		&log2_scale, 1);
     if (cc == 0)
-	return NULL;
+	return 0;
     /* Success.  Make the cache entry. */
     cc->code = glyph;
     cc->wmode = wmode;
@@ -235,8 +237,11 @@ gx_lookup_xfont_char(const gs_state * pgs, cached_fm_pair * pair,
 	      (ulong) glyph, (ulong) xg);
     if_debug6('k', "     wxy=(%g,%g) bbox=(%d,%d),(%d,%d)\n",
 	      wxy.x, wxy.y, bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
-    gx_add_cached_char(font->dir, NULL, cc, pair, &scale_log2_1);
-    return cc;
+    code = gx_add_cached_char(font->dir, NULL, cc, pair, &scale_log2_1);
+    if (code < 0)
+	return code;
+    *pcc = cc;
+    return 1;
 }
 
 /* Copy a cached character to the screen. */
@@ -443,11 +448,10 @@ gx_image_cached_char(register gs_show_enum * penum, register cached_char * cc)
 		for (iy = 0; iy < h && code >= 0; iy++)
 		    code = gs_image_next(pie, bits + iy * raster,
 					 (w + 7) >> 3, &used);
-		code1 = gs_image_cleanup(pie);
-		if (code >= 0 && code1 < 0)
-		    code = code1;
 	}
-	gs_free_object(mem, pie, "image_char(image_enum)");
+	code1 = gs_image_cleanup_and_free_enum(pie);
+	if (code >= 0 && code1 < 0)
+	    code = code1;
     }
   done:if (bits != cc_bits(cc))
 	gs_free_object(penum->memory->non_gc_memory, bits, "compress_alpha_bits");
