@@ -1,23 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gxccman.c,v $ $Revision: 1.2.2.1 $ */
+/* $Id: gxccman.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Character cache management routines for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -36,6 +30,8 @@
 #include "gxfont.h"
 #include "gxfcache.h"
 #include "gxxfont.h"
+#include "gxttfb.h"
+#include "gxfont42.h"
 
 /* Define the descriptors for the cache structures. */
 private_st_cached_fm_pair();
@@ -61,11 +57,11 @@ private RELOC_PTRS_BEGIN(cc_ptr_reloc_ptrs)
 RELOC_PTRS_END
 
 /* Forward references */
-private gx_xfont * lookup_xfont_by_name(P6(gx_device *, const gx_xfont_procs *, gs_font_name *, int, const cached_fm_pair *, const gs_matrix *));
-private cached_char *alloc_char(P2(gs_font_dir *, ulong));
-private cached_char *alloc_char_in_chunk(P2(gs_font_dir *, ulong));
-private void hash_remove_cached_char(P2(gs_font_dir *, uint));
-private void shorten_cached_char(P3(gs_font_dir *, cached_char *, uint));
+private gx_xfont * lookup_xfont_by_name(gx_device *, const gx_xfont_procs *, gs_font_name *, int, const cached_fm_pair *, const gs_matrix *);
+private cached_char *alloc_char(gs_font_dir *, ulong);
+private cached_char *alloc_char_in_chunk(gs_font_dir *, ulong);
+private void hash_remove_cached_char(gs_font_dir *, uint);
+private void shorten_cached_char(gs_font_dir *, cached_char *, uint);
 
 /* ====== Initialization ====== */
 
@@ -119,17 +115,20 @@ gx_char_cache_init(register gs_font_dir * dir)
 			     "initial_chunk");
 
     dir->fmcache.msize = 0;
-    dir->fmcache.mnext = 0;
+    dir->fmcache.used = dir->fmcache.mmax;
+    dir->fmcache.free = dir->fmcache.mmax;
+    dir->fmcache.unused = 0;
     gx_bits_cache_chunk_init(cck, NULL, 0);
     gx_bits_cache_init((gx_bits_cache *) & dir->ccache, cck);
     dir->ccache.bspace = 0;
     memset((char *)dir->ccache.table, 0,
 	   (dir->ccache.table_mask + 1) * sizeof(cached_char *));
     for (i = 0, pair = dir->fmcache.mdata;
-	 i < dir->fmcache.mmax; i++, pair++
-	) {
+	 i < dir->fmcache.mmax; i++, pair++) {
 	pair->index = i;
 	fm_pair_init(pair);
+	pair->ttf = 0;
+	pair->ttr = 0;
     }
 }
 
@@ -139,7 +138,9 @@ gx_char_cache_init(register gs_font_dir * dir)
 /* a client-supplied procedure. */
 void
 gx_purge_selected_cached_chars(gs_font_dir * dir,
-		   bool(*proc) (P2(cached_char *, void *)), void *proc_data)
+			       bool(*proc) (const gs_memory_t *mem, 
+					    cached_char *, void *), 
+			       void *proc_data)
 {
     int chi;
     int cmax = dir->ccache.table_mask;
@@ -147,7 +148,7 @@ gx_purge_selected_cached_chars(gs_font_dir * dir,
     for (chi = 0; chi <= cmax;) {
 	cached_char *cc = dir->ccache.table[chi];
 
-	if (cc != 0 && (*proc) (cc, proc_data)) {
+	if (cc != 0 && (*proc) (dir->memory, cc, proc_data)) {
 	    hash_remove_cached_char(dir, chi);
 	    gx_free_cached_char(dir, cc);
 	} else
@@ -155,53 +156,171 @@ gx_purge_selected_cached_chars(gs_font_dir * dir,
     }
 }
 
+/* ====== font-matrix pair lists ====== */
+
+private int
+fm_pair_remove_from_list(gs_font_dir * dir, cached_fm_pair *pair, uint *head)
+{
+    if (dir->fmcache.mdata + pair->index != pair)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (pair->next == pair->index) {
+	/* The list consists of single element. */
+	if (pair->prev != pair->index)
+	    return_error(gs_error_unregistered); /* Must not happen. */
+	*head = dir->fmcache.mmax;
+    } else {
+	cached_fm_pair *next = dir->fmcache.mdata + pair->next;
+	cached_fm_pair *prev = dir->fmcache.mdata + pair->prev;
+
+	if (next->prev != pair->index)
+	    return_error(gs_error_unregistered); /* Must not happen. */
+	if (prev->next != pair->index)
+	    return_error(gs_error_unregistered); /* Must not happen. */
+	if (*head == pair->index)
+	    *head = next->index;
+	next->prev = prev->index;
+	prev->next = next->index;
+    }
+    return 0;
+}
+
+private int
+fm_pair_insert_into_list(gs_font_dir * dir, cached_fm_pair *pair, uint *head)
+{
+    if (dir->fmcache.mdata + pair->index != pair)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (*head >= dir->fmcache.mmax) {
+	*head = pair->next = pair->prev = pair->index;
+    } else {
+	cached_fm_pair *first = dir->fmcache.mdata + *head;
+	cached_fm_pair *last = dir->fmcache.mdata + first->prev;
+
+	if (first->prev != last->index)
+	    return_error(gs_error_unregistered); /* Must not happen. */
+	if (last->next != first->index)
+	    return_error(gs_error_unregistered); /* Must not happen. */
+	pair->next = first->index;
+	pair->prev = last->index;
+	first->prev = last->next = pair->index;
+	*head = pair->index;	
+    }
+    return 0;
+}
+
 /* ====== Font-level routines ====== */
 
 /* Add a font/matrix pair to the cache. */
 /* (This is only exported for gxccache.c.) */
-cached_fm_pair *
+int
 gx_add_fm_pair(register gs_font_dir * dir, gs_font * font, const gs_uid * puid,
-	       const gs_state * pgs)
+	       const gs_matrix * char_tm, const gs_log2_scale_point *log2_scale,
+	       bool design_grid, cached_fm_pair **ppair)
 {
-    register cached_fm_pair *pair =
-    dir->fmcache.mdata + dir->fmcache.mnext;
-    cached_fm_pair *mend =
-    dir->fmcache.mdata + dir->fmcache.mmax;
+    float mxx, mxy, myx, myy;
+    register cached_fm_pair *pair;
+    int code; 
 
-    if (dir->fmcache.msize == dir->fmcache.mmax) {	/* cache is full *//* Prefer an entry with num_chars == 0, if any. */
-	int count;
-
-	for (count = dir->fmcache.mmax;
-	     --count >= 0 && pair->num_chars != 0;
-	    )
-	    if (++pair == mend)
-		pair = dir->fmcache.mdata;
-	gs_purge_fm_pair(dir, pair, 0);
-    } else {			/* Look for an empty entry.  (We know there is one.) */
-	while (!fm_pair_is_free(pair))
-	    if (++pair == mend)
-		pair = dir->fmcache.mdata;
+    gx_compute_ccache_key(font, char_tm, log2_scale, design_grid,
+			    &mxx, &mxy, &myx, &myy);
+    if (dir->fmcache.msize == dir->fmcache.mmax) {	
+	/* cache is full, drop the older entry. */
+	/* gx_touch_fm_pair must be called whenever
+	   a pair is used to move it to the top of the list.
+	   Since we drop a pair from the list bottom,
+	   and since the list is long enough,
+	   with a high probability it won't drop a pair, 
+	   which currently is pointed by an active text enumerator.
+	   
+	   Note that with Type 3 fonts multiple text enumerators
+	   may be active (exist on estack) in same time,
+	   therefore the list length sets a constraint for
+	   the number of font-matrix pairs used within a charproc.
+	   If it uses too many ones, the outer text enumerator
+	   will fail with 'invalidfont' in gx_add_cached_char.
+	*/
+	pair = dir->fmcache.mdata + dir->fmcache.used;
+	pair = dir->fmcache.mdata + pair->prev; /* last touched. */
+	code = gs_purge_fm_pair(dir, pair, 0);
+	if (code < 0)
+	    return code;
+    }
+    if (dir->fmcache.free < dir->fmcache.mmax) {
+	/* use a free entry. */
+	pair = dir->fmcache.mdata + dir->fmcache.free;
+	code = fm_pair_remove_from_list(dir, pair, &dir->fmcache.free);
+	if (code < 0)
+	    return code;
+    } else {
+	/* reserve a new entry. */
+	pair = dir->fmcache.mdata + dir->fmcache.unused;
+	dir->fmcache.unused++;
     }
     dir->fmcache.msize++;
-    dir->fmcache.mnext = pair + 1 - dir->fmcache.mdata;
-    if (dir->fmcache.mnext == dir->fmcache.mmax)
-	dir->fmcache.mnext = 0;
+    code = fm_pair_insert_into_list(dir, pair, &dir->fmcache.used);
+    if (code < 0)
+	return code;
     pair->font = font;
     pair->UID = *puid;
     pair->FontType = font->FontType;
     /* The OSF/1 compiler doesn't like casting a pointer to */
     /* a shorter int.... */
     pair->hash = (uint) (ulong) pair % 549;	/* scramble bits */
-    pair->mxx = pgs->char_tm.xx, pair->mxy = pgs->char_tm.xy;
-    pair->myx = pgs->char_tm.yx, pair->myy = pgs->char_tm.yy;
+    pair->mxx = mxx, pair->mxy = mxy;
+    pair->myx = myx, pair->myy = myy;
     pair->num_chars = 0;
     pair->xfont_tried = false;
     pair->xfont = 0;
+    pair->ttf = 0;
+    pair->ttr = 0;
+    pair->design_grid = false;
+    if (font->FontType == ft_TrueType || font->FontType == ft_CID_TrueType) 
+	if (((gs_font_type42 *)font)->FAPI==NULL) {
+	    float cxx, cxy, cyx, cyy;
+	    gs_matrix m;
+	    gx_compute_char_matrix(char_tm, log2_scale, &cxx, &cxy, &cyx, &cyy);
+
+	    pair->design_grid = design_grid;
+	    m.xx = cxx;
+	    m.xy = cxy;
+	    m.yx = cyx;
+	    m.yy = cyy;
+	    m.tx = m.ty = 0;
+	    pair->ttr = gx_ttfReader__create(dir->memory);
+	    if (!pair->ttr)
+		return_error(gs_error_VMerror);
+	    /*  We could use a single the reader instance for all fonts ... */
+	    pair->ttf = ttfFont__create(dir);
+	    if (!pair->ttf)
+		return_error(gs_error_VMerror);
+	    gx_ttfReader__set_font(pair->ttr, (gs_font_type42 *)font);
+	    code = ttfFont__Open_aux(pair->ttf, dir->tti, pair->ttr, 
+			(gs_font_type42 *)font, &m, log2_scale, design_grid);
+	    gx_ttfReader__set_font(pair->ttr, NULL);
+	    if (code < 0)
+		return code;
+	}
+    pair->memory = 0;
     if_debug8('k', "[k]adding pair 0x%lx: font=0x%lx [%g %g %g %g] UID %ld, 0x%lx\n",
 	      (ulong) pair, (ulong) font,
 	      pair->mxx, pair->mxy, pair->myx, pair->myy,
 	      (long)pair->UID.id, (ulong) pair->UID.xvalues);
-    return pair;
+    *ppair = pair;
+    return 0;
+}
+
+/* Update the pointer to the last used font/matrix pair. */
+int
+gx_touch_fm_pair(gs_font_dir *dir, cached_fm_pair *pair)
+{
+    if (pair->index != dir->fmcache.used) {
+	int code; 
+
+	code = fm_pair_remove_from_list(dir, pair, &dir->fmcache.used);
+	if (code < 0)
+	    return code;
+	return fm_pair_insert_into_list(dir, pair, &dir->fmcache.used);
+    }
+    return 0;
 }
 
 /* Look up the xfont for a font/matrix pair. */
@@ -274,17 +393,17 @@ gx_lookup_xfont(const gs_state * pgs, cached_fm_pair * pair, int encoding_index)
 /* or just characters that depend on its xfont. */
 #define cpair ((cached_fm_pair *)vpair)
 private bool
-purge_fm_pair_char(cached_char * cc, void *vpair)
+purge_fm_pair_char(const gs_memory_t *mem, cached_char * cc, void *vpair)
 {
     return cc_pair(cc) == cpair;
 }
 private bool
-purge_fm_pair_char_xfont(cached_char * cc, void *vpair)
+purge_fm_pair_char_xfont(const gs_memory_t *mem, cached_char * cc, void *vpair)
 {
     return cc_pair(cc) == cpair && cpair->xfont == 0 && !cc_has_bits(cc);
 }
 #undef cpair
-void
+int
 gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 {
     if_debug2('k', "[k]purging pair 0x%lx%s\n",
@@ -299,7 +418,15 @@ gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 				   (xfont_only ? purge_fm_pair_char_xfont :
 				    purge_fm_pair_char),
 				   pair);
+    if (pair->ttr)
+	gx_ttfReader__destroy(pair->ttr);
+    pair->ttr = 0;
+    if (pair->ttf)
+	ttfFont__destroy(pair->ttf, dir);
+    pair->ttf = 0;
     if (!xfont_only) {
+	int code;
+
 #ifdef DEBUG
 	if (pair->num_chars != 0) {
 	    lprintf1("Error in gs_purge_fm_pair: num_chars =%d\n",
@@ -307,8 +434,15 @@ gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 	}
 #endif
 	fm_pair_set_free(pair);
+	code = fm_pair_remove_from_list(dir, pair, &dir->fmcache.used);
+	if (code < 0)
+	    return code;
+	code = fm_pair_insert_into_list(dir, pair, &dir->fmcache.free);
+	if (code < 0)
+	    return code;
 	dir->fmcache.msize--;
     }
+    return 0;
 }
 
 /* Look up an xfont by name. */
@@ -441,6 +575,8 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
     cc_set_raster(cc, gdev_mem_raster(pdev2));
     cc_set_pair_only(cc, 0);	/* not linked in yet */
     cc->id = gx_no_bitmap_id;
+    cc->subpix_origin.x = cc->subpix_origin.y = 0;
+    cc->linked = false;
 
     /* Open the cache device(s). */
 
@@ -481,16 +617,15 @@ gx_free_cached_char(gs_font_dir * dir, cached_char * cc)
 
     dir->ccache.chunks = cck;
     dir->ccache.cnext = (byte *) cc - cck->data;
-    if (cc_pair(cc) != 0) {	/* might be allocated but not added to table yet */
+    if (cc->linked)
 	cc_pair(cc)->num_chars--;
-    }
     if_debug2('k', "[k]freeing char 0x%lx, pair=0x%lx\n",
 	      (ulong) cc, (ulong) cc_pair(cc));
     gx_bits_cache_free((gx_bits_cache *) & dir->ccache, &cc->head, cck);
 }
 
 /* Add a character to the cache */
-void
+int
 gx_add_cached_char(gs_font_dir * dir, gx_device_memory * dev,
 cached_char * cc, cached_fm_pair * pair, const gs_log2_scale_point * pscale)
 {
@@ -514,9 +649,26 @@ cached_char * cc, cached_fm_pair * pair, const gs_log2_scale_point * pscale)
 	while (dir->ccache.table[chi &= dir->ccache.table_mask] != 0)
 	    chi++;
 	dir->ccache.table[chi] = cc;
+	if (cc->pair == NULL) {
+	    /* gx_show_text_retry could reset it when bbox_draw
+	       discovered an insufficient FontBBox and enlarged it. 
+	       Glyph raster params could change then. */
+	    cc->pair = pair;
+	} else if (cc->pair != pair) {
+	    /* gx_add_fm_pair could drop the active font-matrix pair
+	       due to cache overflow during a charproc interpretation. 
+	       Likely a single charproc renders too many characters
+	       for generating the character image.
+	       We have no mechanizm for locking font-matrix pairs in cache
+	       to avoud their dissipation. Therefore we consider this failure 
+	       as an implementation limitation. */
+	    return_error(gs_error_invalidfont);
+	}
+	cc->linked = true;
 	cc_set_pair(cc, pair);
 	pair->num_chars++;
     }
+    return 0;
 }
 
 /* Adjust the bits of a newly-rendered character, by unscaling */
@@ -646,11 +798,11 @@ gx_add_char_bits(gs_font_dir * dir, cached_char * cc,
 
     /* Assign a bitmap id. */
 
-    cc->id = gs_next_ids(1);
+    cc->id = gs_next_ids(dir->memory, 1);
 }
 
 /* Purge from the caches all references to a given font. */
-void
+int
 gs_purge_font_from_char_caches(gs_font_dir * dir, const gs_font * font)
 {
     cached_fm_pair *pair = dir->fmcache.mdata;
@@ -662,11 +814,16 @@ gs_purge_font_from_char_caches(gs_font_dir * dir, const gs_font * font)
 	if (pair->font == font) {
 	    if (uid_is_valid(&pair->UID)) {	/* Keep the entry. */
 		pair->font = 0;
-	    } else
-		gs_purge_fm_pair(dir, pair, 0);
+	    } else {
+		int code = gs_purge_fm_pair(dir, pair, 0);
+
+		if (code < 0)
+		    return code;
+	    }
 	}
 	pair++;
     }
+    return 0;
 }
 
 /* ------ Internal routines ------ */
