@@ -1,22 +1,18 @@
-/* Copyright (C) 1993, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxcht.c,v 1.15 2004/12/08 21:35:13 stefan Exp $ */
+/*$Id: gxcht.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Color halftone rendering for Ghostscript imaging library */
-#include <assert.h>
 #include "memory_.h"
 #include "gx.h"
 #include "gserrors.h"
@@ -202,7 +198,8 @@ gx_dc_ht_colored_write(
     byte *                          pdata0 = pdata;
 
     /* sanity check */
-    assert(pdevc->colors.colored.num_components == num_comps);
+    if (pdevc->colors.colored.num_components != num_comps)
+	return_error(gs_error_unregistered); /* Must not happen. */
 
     /* check if saved color is of the same type */
     if (psdc != 0 && psdc->type != pdevc->type)
@@ -514,7 +511,7 @@ typedef struct color_values_pair_s {
 #define SET_HT_COLORS_PROC(proc)\
   int proc(\
 	      color_values_pair_t *pvp,\
-	      gx_color_index colors[1 << MAX_DCC],\
+	      gx_color_index colors[MAX_DCC_16],\
 	      const gx_const_strip_bitmap *sbits[MAX_DCC],\
 	      const gx_device_color *pdevc,\
 	      gx_device *dev,\
@@ -833,7 +830,7 @@ private const gx_const_strip_bitmap ht_no_bitmap = {
 /* Set up the colors and the individual plane halftone bitmaps. */
 private int
 set_ht_colors_le_4(color_values_pair_t *pvp /* only used internally */,
-		   gx_color_index colors[MAX_DCC_16],
+		   gx_color_index colors[MAX_DCC_16] /* 16 used */,
 		   const gx_const_strip_bitmap * sbits[MAX_DCC],
 		   const gx_device_color * pdc, gx_device * dev,
 		   gx_ht_cache * caches[MAX_DCC], int nplanes)
@@ -957,7 +954,7 @@ c2:	    case 2:
 /* Set up colors using the standard 1-bit CMYK mapping. */
 private int
 set_cmyk_1bit_colors(color_values_pair_t *ignore_pvp,
-		     gx_color_index colors[MAX_DCC_16 /*16 used*/],
+		     gx_color_index colors[MAX_DCC_16] /*2 used*/,
 		     const gx_const_strip_bitmap * sbits[MAX_DCC /*4 used*/],
 		     const gx_device_color * pdc, gx_device * dev,
 		     gx_ht_cache * caches[MAX_DCC /*4 used*/],
@@ -1021,7 +1018,7 @@ set_cmyk_1bit_colors(color_values_pair_t *ignore_pvp,
  */
 private int
 set_ht_colors_gt_4(color_values_pair_t *pvp,
-		   gx_color_index colors[MAX_DCC_16],
+		   gx_color_index colors[MAX_DCC_16 /* 2 * nplanes */],
 		   const gx_const_strip_bitmap * sbits[MAX_DCC],
 		   const gx_device_color * pdc, gx_device * dev,
 		   gx_ht_cache * caches[MAX_DCC], int nplanes)
@@ -1095,7 +1092,7 @@ init_tile_cursor(int i, tile_cursor_t *ptc, const gx_const_strip_bitmap *btile,
     ptc->xbits = ((tw - 1) & 7) + 1;
     ptc->tdata = btile->data;
     ptc->raster = btile->raster;
-    ptc->row = ptc->tdata + by * ptc->raster;
+    ptc->row = ptc->tdata + by * (int)ptc->raster;
     ptc->data = ptc->row + ptc->xoffset;
     ptc->bit_shift = ptc->xshift;
     if_debug6('h', "[h]plane %d: size=%d,%d shift=%d bx=%d by=%d\n",
