@@ -1,20 +1,17 @@
-/* Copyright (C) 1995, 1996, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxobj.h,v 1.2.6.1 2002/02/22 19:45:59 ray Exp $ */
+/* $Id: gxobj.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Memory manager implementation structures for Ghostscript */
 
 #ifndef gxobj_INCLUDED
@@ -22,6 +19,12 @@
 
 #include "gxbitmap.h"
 
+#ifdef DEBUG
+#define IGC_PTR_STABILITY_CHECK 0
+#else
+#define IGC_PTR_STABILITY_CHECK 0
+#endif
+
 /* ================ Objects ================ */
 
 /*
@@ -91,6 +94,9 @@ typedef struct obj_header_data_s {
 	gs_memory_type_ptr_t type;
 	uint reloc;
     } t;
+#   if IGC_PTR_STABILITY_CHECK
+    unsigned space_id:3; /* r_space_bits + 1 bit for "instability". */
+#   endif
 } obj_header_data_t;
 
 /*
@@ -98,10 +104,14 @@ typedef struct obj_header_data_s {
  * alignment values are powers of 2; we can avoid nested 'max'es that way.
  * The final | is because back pointer values are divided by obj_back_scale,
  * so objects must be aligned at least 0 mod obj_back_scale.
+ *
+ * Note: OBJECTS ARE NOT GUARANTEED to be aligned any more strictly than
+ * required by the hardware, regardless of the value of obj_align_mod.
+ * See gsmemraw.h for more information about this.
  */
 #define obj_align_mod\
-  (((arch_align_long_mod - 1) | (arch_align_ptr_mod - 1) |\
-    (arch_align_double_mod - 1) | (align_bitmap_mod - 1) |\
+  (((ARCH_ALIGN_MEMORY_MOD - 1) |\
+    (align_bitmap_mod - 1) |\
     (obj_back_scale - 1)) + 1)
 /* The only possible values for obj_align_mod are 4, 8, or 16.... */
 #if obj_align_mod == 4
