@@ -1,26 +1,24 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxpath.c,v 1.10 2004/03/13 18:28:52 igor Exp $ */
+/*$Id: gxpath.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Internal path management routines for Ghostscript library */
 #include "gx.h"
 #include "gserrors.h"
 #include "gsstruct.h"
 #include "gxfixed.h"
 #include "gzpath.h"
+#include "vdtrace.h"
 
 /* These routines all assume that all points are */
 /* already in device coordinates, and in fixed representation. */
@@ -737,6 +735,11 @@ fixed x3, fixed y3, fixed xt, fixed yt, floatp fraction, segment_notes notes)
 {
     fixed x0 = ppath->position.x, y0 = ppath->position.y;
 
+    vd_curveto(x0 + (fixed) ((xt - x0) * fraction),
+				   y0 + (fixed) ((yt - y0) * fraction),
+				   x3 + (fixed) ((xt - x3) * fraction),
+				   y3 + (fixed) ((yt - y3) * fraction),
+				   x3, y3);
     return gx_path_add_curve_notes(ppath,
 				   x0 + (fixed) ((xt - x0) * fraction),
 				   y0 + (fixed) ((yt - y0) * fraction),
