@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gzht.h,v 1.4.4.1 2002/02/22 19:45:59 ray Exp $ */
+/* $Id: gzht.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Internal procedures for halftones */
 /* Requires gxdevice.h, gxdcolor.h */
 
@@ -28,24 +25,24 @@
 #include "gxhttile.h"
 
 /* Sort a sampled halftone order by sample value. */
-void gx_sort_ht_order(P2(gx_ht_bit *, uint));
+void gx_sort_ht_order(gx_ht_bit *, uint);
 
 /* (Internal) procedures for constructing halftone orders. */
-int gx_ht_alloc_ht_order(P8(gx_ht_order * porder, uint width, uint height,
-			    uint num_levels, uint num_bits, uint strip_shift,
-			    const gx_ht_order_procs_t *procs,
-			    gs_memory_t * mem));
-int gx_ht_alloc_order(P6(gx_ht_order * porder, uint width, uint height,
-			 uint strip_shift, uint num_levels, gs_memory_t *mem));
-int gx_ht_alloc_threshold_order(P5(gx_ht_order * porder, uint width,
-				   uint height, uint num_levels,
-				   gs_memory_t * mem));
-int gx_ht_alloc_client_order(P6(gx_ht_order * porder, uint width, uint height,
-			uint num_levels, uint num_bits, gs_memory_t * mem));
-void gx_ht_construct_spot_order(P1(gx_ht_order *));
-int gx_ht_construct_threshold_order(P2(gx_ht_order *, const byte *));
-void gx_ht_construct_bit(P3(gx_ht_bit * bit, int width, int bit_num));
-void gx_ht_construct_bits(P1(gx_ht_order *));
+int gx_ht_alloc_ht_order(gx_ht_order * porder, uint width, uint height,
+			 uint num_levels, uint num_bits, uint strip_shift,
+			 const gx_ht_order_procs_t *procs,
+			 gs_memory_t * mem);
+int gx_ht_alloc_order(gx_ht_order * porder, uint width, uint height,
+		      uint strip_shift, uint num_levels, gs_memory_t *mem);
+int gx_ht_alloc_threshold_order(gx_ht_order * porder, uint width,
+				uint height, uint num_levels,
+				gs_memory_t * mem);
+int gx_ht_alloc_client_order(gx_ht_order * porder, uint width, uint height,
+			     uint num_levels, uint num_bits, gs_memory_t * mem);
+void gx_ht_construct_spot_order(gx_ht_order *);
+int gx_ht_construct_threshold_order(gx_ht_order *, const byte *);
+void gx_ht_construct_bit(gx_ht_bit * bit, int width, int bit_num);
+void gx_ht_construct_bits(gx_ht_order *);
 
 /* Halftone enumeration structure */
 struct gs_screen_enum_s {
@@ -64,31 +61,31 @@ struct gs_screen_enum_s {
 /*    order.levels, order.bits, pgs) */
 
 /* Prepare a device halftone for installation, but don't install it. */
-int gs_sethalftone_prepare(P3(gs_state *, gs_halftone *,
-			      gx_device_halftone *));
+int gs_sethalftone_prepare(gs_state *, gs_halftone *,
+			   gx_device_halftone *);
 
 /* Allocate and initialize a spot screen. */
 /* This is the first half of gs_screen_init_accurate/memory. */
-int gs_screen_order_alloc(P2(gx_ht_order *, gs_memory_t *));
-int gs_screen_order_init_memory(P5(gx_ht_order *, const gs_state *,
-				   gs_screen_halftone *, bool, gs_memory_t *));
+int gs_screen_order_alloc(gx_ht_order *, gs_memory_t *);
+int gs_screen_order_init_memory(gx_ht_order *, const gs_state *,
+				gs_screen_halftone *, bool, gs_memory_t *);
 
 #define gs_screen_order_init(porder, pgs, phsp, accurate)\
   gs_screen_order_init_memory(porder, pgs, phsp, accurate, pgs->memory)
 
 /* Prepare to sample a spot screen. */
 /* This is the second half of gs_screen_init_accurate/memory. */
-int gs_screen_enum_init_memory(P5(gs_screen_enum *, const gx_ht_order *,
-				  gs_state *, const gs_screen_halftone *,
-				  gs_memory_t *));
+int gs_screen_enum_init_memory(gs_screen_enum *, const gx_ht_order *,
+			       gs_state *, const gs_screen_halftone *,
+			       gs_memory_t *);
 
 #define gs_screen_enum_init(penum, porder, pgs, phsp)\
   gs_screen_enum_init_memory(penum, porder, pgs, phsp, pgs->memory)
 
 /* Process an entire screen plane. */
-int gx_ht_process_screen_memory(P5(gs_screen_enum * penum, gs_state * pgs,
-				   gs_screen_halftone * phsp, bool accurate,
-				   gs_memory_t * mem));
+int gx_ht_process_screen_memory(gs_screen_enum * penum, gs_state * pgs,
+				gs_screen_halftone * phsp, bool accurate,
+				gs_memory_t * mem);
 
 #define gx_ht_process_screen(penum, pgs, phsp, accurate)\
   gx_ht_process_screen_memory(penum, pgs, phsp, accurate, pgs->memory)
@@ -124,9 +121,27 @@ struct gx_ht_cache_s {
 				/* 1 if fit */
     gx_bitmap_id base_id;	/* the base id, to which */
 				/* we add the halftone level */
-    gx_ht_tile *(*render_ht)(P2(gx_ht_cache *, int)); /* rendering procedure */
+    gx_ht_tile *(*render_ht)(gx_ht_cache *, int); /* rendering procedure */
 };
 
+/* Define the sizes of the halftone cache. */
+#define max_cached_tiles_HUGE 5000	/* not used */
+#define max_ht_bits_HUGE 1000000	/* not used */
+#define max_cached_tiles_LARGE 577
+#define max_ht_bits_LARGE 100000
+#define max_cached_tiles_SMALL 25
+#define max_ht_bits_SMALL 1000
+
+/* Define the size of the halftone tile cache. */
+#define max_tile_bytes_LARGE 65536
+#define max_tile_bytes_SMALL 512
+#if arch_small_memory
+#  define max_tile_cache_bytes max_tile_bytes_SMALL
+#else
+#  define max_tile_cache_bytes\
+     (gs_debug_c('.') ? max_tile_bytes_SMALL : max_tile_bytes_LARGE)
+#endif
+
 /* We don't mark from the tiles pointer, and we relocate the tiles en masse. */
 #define private_st_ht_tiles()	/* in gxht.c */\
   gs_private_st_composite(st_ht_tiles, gx_ht_tile, "ht tiles",\
@@ -148,10 +163,10 @@ extern const gx_color_value *const fc_color_quo[8];
 /* ------ Halftone cache procedures ------ */
 
 /* Allocate/free a halftone cache. */
-uint gx_ht_cache_default_tiles(P0());
-uint gx_ht_cache_default_bits(P0());
-gx_ht_cache *gx_ht_alloc_cache(P3(gs_memory_t *, uint, uint));
-void gx_ht_free_cache(P2(gs_memory_t *, gx_ht_cache *));
+uint gx_ht_cache_default_tiles(void);
+uint gx_ht_cache_default_bits(void);
+gx_ht_cache *gx_ht_alloc_cache(gs_memory_t *, uint, uint);
+void gx_ht_free_cache(gs_memory_t *, gx_ht_cache *);
 
 /* Clear a halftone cache. */
 #define gx_ht_clear_cache(pcache)\
@@ -159,18 +174,21 @@ void gx_ht_free_cache(P2(gs_memory_t *, gx_ht_cache *));
    (pcache)->ht_tiles[0].tiles.data = 0)
 
 /* Initialize a halftone cache with a given order. */
-void gx_ht_init_cache(P2(gx_ht_cache *, const gx_ht_order *));
+void gx_ht_init_cache(const gs_memory_t *mem, gx_ht_cache *, const gx_ht_order *);
+
+/* Check whether the tile cache corresponds to the current order */
+bool gx_check_tile_cache_current(const gs_imager_state * pis);
 
 /* Make the cache order current, and return whether */
 /* there is room for all possible tiles in the cache. */
-bool gx_check_tile_cache(P1(const gs_imager_state *));
+bool gx_check_tile_cache(const gs_imager_state *);
 
 /* Determine whether a given (width, y, height) might fit into a */
 /* single tile. If so, return the byte offset of the appropriate row */
 /* from the beginning of the tile, and set *ppx to the x phase offset */
 /* within the tile; if not, return -1. */
-int gx_check_tile_size(P6(const gs_imager_state * pis, int w, int y, int h,
-			  gs_color_select_t select, int *ppx));
+int gx_check_tile_size(const gs_imager_state * pis, int w, int y, int h,
+		       gs_color_select_t select, int *ppx);
 
 /* Make a given level current in a halftone cache. */
 #define gx_render_ht(pcache, b_level)\
@@ -180,31 +198,54 @@ int gx_check_tile_size(P6(const gs_imager_state * pis, int w, int y, int h,
 
 /* Release a gx_ht_order by freeing its components. */
 /* (Don't free the gx_device_halftone itself.) */
-void gx_ht_order_release(P3(gx_ht_order * porder, gs_memory_t * mem, bool free_cache));
+void gx_ht_order_release(gx_ht_order * porder, gs_memory_t * mem, bool free_cache);
 
 /*
  * Install a device halftone in an imager state.  Note that this does not
- * read or update the client halftone.  There is a special check for pdht ==
- * pis->dev_ht, for the benefit of the band rendering code.
+ * read or update the client halftone.
  */
-int gx_imager_dev_ht_install(P4(gs_imager_state * pis,
-				const gx_device_halftone * pdht,
-				gs_halftone_type type,
-				const gx_device * dev));
+int gx_imager_dev_ht_install(gs_imager_state * pis,
+			     gx_device_halftone * pdht,
+			     gs_halftone_type type,
+			     const gx_device * dev);
 
 /*
  * Install a new halftone in the graphics state.  Note that we copy the top
  * level of the gs_halftone and the gx_device_halftone, and take ownership
  * of any substructures.
  */
-int gx_ht_install(P3(gs_state *, const gs_halftone *,
-		     const gx_device_halftone *));
+int gx_ht_install(gs_state *, const gs_halftone *, gx_device_halftone *);
 
 /* Reestablish the effective transfer functions, taking into account */
 /* any overrides from halftone dictionaries. */
 /* Some compilers object to names longer than 31 characters.... */
-void gx_imager_set_effective_xfer(P1(gs_imager_state * pis));
-void gx_set_effective_transfer(P1(gs_state * pgs));
-
+void gx_imager_set_effective_xfer(gs_imager_state * pis);
+void gx_set_effective_transfer(gs_state * pgs);
 
+/*
+ * This routine will take a color name (defined by a ptr and size) and
+ * check if this is a valid colorant name for the current device.  If
+ * so then the device's colorant number is returned.
+ *
+ * Two other checks are also made.  If the name is "Default" then a value
+ * of GX_DEVICE_COLOR_MAX_COMPONENTS is returned.  This is done to
+ * simplify the handling of default halftones.
+ *
+ * If the halftone type is colorscreen or multiple colorscreen, then we
+ * also check for Red/Cyan, Green/Magenta, Blue/Yellow, and Gray/Black
+ * component name pairs.  This is done since the setcolorscreen and
+ * sethalftone types 2 and 4 imply the dual name sets.
+ *
+ * A negative value is returned if the color name is not found.
+ */
+int gs_color_name_component_number(gx_device * dev, const char * pname,
+				int name_size, int halftonetype);
+/*
+ * See gs_color_name_component_number for main description.
+ *
+ * This version converts a name index value into a string and size and
+ * then call gs_color_name_component_number.
+ */
+int gs_cname_to_colorant_number(gs_state * pgs, byte * pname, uint name_size,
+				 int halftonetype);
 #endif /* gzht_INCLUDED */
