@@ -1,36 +1,40 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: iref.h,v $ $Revision: 1.3.2.1 $ */
+/* $Id: iref.h 6651 2006-03-13 16:18:19Z stefan $ */
 /* Object structure and type definitions for Ghostscript */
 
 #ifndef iref_INCLUDED
 #  define iref_INCLUDED
 
+/*
+ * Note: this file defines a large number of macros.  Many of these are
+ * only used for internal purposes within this file, to help in the
+ * definition of other macros or data structures, and should never be
+ * referenced outside this file.  It is intended that all such internal
+ * macros have names beginning with an underscore (_).
+ */
+
 /* The typedef for object references */
 #ifndef ref_DEFINED
 typedef struct ref_s ref;
 #  define ref_DEFINED
 #endif
 
-/* The typedef for packed object references.  This is opaque here: */
-/* the details are in packed.h. */
+/*
+ * Define the type for packed object references.  This is opaque here:
+ * the details are in ipacked.h.
+ */
 typedef ushort ref_packed;
 
 #define log2_sizeof_ref_packed arch_log2_sizeof_short
@@ -45,6 +49,11 @@ typedef ushort ref_packed;
  * The types marked with + use the read/write/execute
  * attributes; the rest only use the executable attribute.
  * The types marked with # use the size field.
+ *
+ * Note that for the object types that support getinterval (array and
+ * string types), there is no way to tell whether a given reference
+ * designates an original object or a sub-interval.  This is a deliberate
+ * design decision.
  */
 typedef enum {
 
@@ -56,10 +65,10 @@ typedef enum {
  * type 0 for t__invalid, which will never appear in a real ref.
  *
  * The "invalid" type is only used in a few special places: the guard
- * entries at the bottom of the o-stack that detect stack underflow,
- * and (eventually) the ref that the cached value pointer in names points to
- * if the binding isn't known.  It never appears on a stack or in a
- * program-visible data structure.
+ * entries at the bottom of the o-stack that detect stack underflow, and
+ * (perhaps eventually) the ref that the cached value pointer in names
+ * points to if the binding isn't known.  It never appears on a stack or in
+ * a program-visible data structure.
  */
 
     t__invalid,			/*      (no value) */
@@ -72,10 +81,12 @@ typedef enum {
  * a multiple of 4, for the sake of r_is_array and r_is_proc (see below).
  */
 
-#define _t_array_span 4
+#define _REF_T_ARRAY_SPAN 4
     t_array,			/* @!+# value.refs */
-    /* The following are the two implementations of */
-    /* packed arrays. */
+    /*
+     * The following are the two implementations of packed arrays.
+     * See ipacked.h for details.
+     */
     t_mixedarray,		/* @!+# value.packed */
     t_shortarray,		/* @!+# value.packed */
     t_unused_array_,		/*      (an unused array type) */
@@ -99,7 +110,7 @@ typedef enum {
  * a multiple of 2, for the sake of r_has_stype (see below).
  */
 
-#define _t_struct_span 2
+#define _REF_T_STRUCT_SPAN 2
     t_struct,			/* @    value.pstruct */
     t_astruct,			/* @ +  value.pstruct */
 
@@ -115,7 +126,7 @@ typedef enum {
  */
     t_name,			/* @! # value.pname, uses size for index */
     t_null,			/*  ! # (value.opproc, uses size for mark */
-    /*        type, on e-stack only) */
+				/*        type, on e-stack only) */
 /*
  * Operator objects use the a_space field because they may actually be
  * disguised procedures.  (Real operators always have a_space = 0.)
@@ -123,7 +134,7 @@ typedef enum {
     t_operator,			/* @! # value.opproc, uses size for index */
     t_real,			/*      value.realval */
     t_save,			/*      value.saveid, see isave.h for why */
-    /*        this isn't a t_struct */
+				/*        this isn't a t_struct */
     t_string,			/* @!+# value.bytes */
 /*
  * The following are extensions to the PostScript type set.
@@ -153,57 +164,57 @@ typedef enum {
  * Define a table giving properties of types, similar to the table used
  * by the isxxx functions (macros) in <ctype.h>.
  */
-#define _rtype_uses_access 1	/* type uses w/r/x attributes */
-#define _rtype_uses_size 2
-#define _rtype_is_null 4
-#define _rtype_is_dictionary 8
+#define _REF_TYPE_USES_ACCESS 1	/* type uses w/r/x attributes ("+" above) */
+#define _REF_TYPE_USES_SIZE 2	/* type uses size attribute ("#" above) */
+#define _REF_TYPE_IS_NULL 4	/* type is t_null */
+#define _REF_TYPE_IS_DICTIONARY 8 /* type is t_dictionary */
 extern const byte ref_type_properties[1 << 6];	/* r_type_bits */
 
 #define REF_TYPE_PROPERTIES_DATA\
   0,				/* t__invalid */\
   0,				/* t_boolean */\
-  _rtype_uses_access | _rtype_is_dictionary, /* t_dictionary */\
-  _rtype_uses_access | _rtype_uses_size, /* t_file */\
-  _rtype_uses_access | _rtype_uses_size, /* t_array */\
-  _rtype_uses_access | _rtype_uses_size, /* t_mixedarray */\
-  _rtype_uses_access | _rtype_uses_size, /* t_shortarray */\
-  _rtype_uses_access | _rtype_uses_size, /* (unused array type) */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_IS_DICTIONARY, /* t_dictionary */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_USES_SIZE, /* t_file */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_USES_SIZE, /* t_array */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_USES_SIZE, /* t_mixedarray */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_USES_SIZE, /* t_shortarray */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_USES_SIZE, /* (unused array type) */\
   0,				/* t_struct */\
-  _rtype_uses_access,		/* t_astruct */\
+  _REF_TYPE_USES_ACCESS,		/* t_astruct */\
   0,				/* t_fontID */\
   0,				/* t_integer */\
   0,				/* t_mark */\
-  _rtype_uses_size,		/* t_name */\
-  _rtype_is_null,		/* t_null, uses size only on e-stack */\
-  _rtype_uses_size,		/* t_operator */\
+  _REF_TYPE_USES_SIZE,		/* t_name */\
+  _REF_TYPE_IS_NULL,		/* t_null, uses size only on e-stack */\
+  _REF_TYPE_USES_SIZE,		/* t_operator */\
   0,				/* t_real */\
   0,				/* t_save */\
-  _rtype_uses_access | _rtype_uses_size, /* t_string */\
-  _rtype_uses_access,		/* t_device */\
-  _rtype_uses_size,		/* t_oparray */\
+  _REF_TYPE_USES_ACCESS | _REF_TYPE_USES_SIZE, /* t_string */\
+  _REF_TYPE_USES_ACCESS,		/* t_device */\
+  _REF_TYPE_USES_SIZE,		/* t_oparray */\
     /*\
      * The remaining types are the extended pseudo-types used by the\
      * interpreter for operators.  We need to fill up the table.\
      */\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*24*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*28*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*32*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*36*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*40*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*44*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*48*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*52*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*56*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size, /*60*/\
-  _rtype_uses_size,_rtype_uses_size,_rtype_uses_size,_rtype_uses_size	/*64 */
-#define _rtype_has(rtype,props)\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*24*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*28*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*32*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*36*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*40*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*44*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*48*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*52*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*56*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE, /*60*/\
+  _REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE,_REF_TYPE_USES_SIZE	/*64 */
+#define _REF_TYPE_HAS(rtype,props)\
   ((ref_type_properties[rtype] & (props)) != 0)
 #define ref_type_uses_access(rtype)\
- _rtype_has(rtype, _rtype_uses_access)
+ _REF_TYPE_HAS(rtype, _REF_TYPE_USES_ACCESS)
 #define ref_type_uses_size(rtype)\
- _rtype_has(rtype, _rtype_uses_size)
+ _REF_TYPE_HAS(rtype, _REF_TYPE_USES_SIZE)
 #define ref_type_uses_size_or_null(rtype)\
- _rtype_has(rtype, _rtype_uses_size | _rtype_is_null)
+ _REF_TYPE_HAS(rtype, _REF_TYPE_USES_SIZE | _REF_TYPE_IS_NULL)
 /*
  * Define the type names for debugging printout.
  * All names must be the same length, so that columns will line up.
@@ -251,49 +262,78 @@ extern const byte ref_type_properties[1 << 6];	/* r_type_bits */
  * The layout given below results in the most efficient code overall.
  */
 
-/* Location attributes. */
-/* Note that these are associated with the *location*, not with the */
-/* ref that is *stored* in that location. */
+/*
+ * A few of the fields of a ref are associated with the *location*;
+ * most are associated with the ref that is *stored* in that location.
+ * When a ref is copied from one location to another, the former are
+ * preserved in the destination, the latter are copied.
+ */
+/*
+ * The following are the attributes associated with the location:
+ */
 #define l_mark 1		/* mark for garbage collector */
 #define l_new 2			/* stored into since last save */
-/* Attributes visible at the PostScript language level. */
-/* Reserve bits for VM space information (defined in ivmspace.h). */
+/*
+ * The following are attributes associated with the ref itself (the
+ * contents of the location).  These are visible to PostScript code.
+ */
+/*
+ * Reserve bits for VM space information (defined in ivmspace.h).  Note that
+ * These bits refer to the VM space of the pointer in the ref, if any, not
+ * the location in which the ref itself is stored.  For scalars, these bits
+ * are always zero.
+ */
 #define r_space_bits 2
 #define r_space_shift 2
+/*
+ * Define the protection attributes.  Only 4 combinations are legal:
+ * 0, execute, execute + read, execute + read + write.  Note that some
+ * refs (such as scalars) do not use these: in such refs, they are
+ * always zero.
+ */
 #define a_write 0x10
 #define a_read 0x20
 #define a_execute 0x40
+#define a_readonly (a_read + a_execute)
+#define a_all (a_write + a_read+a_execute)
+/*
+ * Define the executable attribute.  All refs use this.
+ */
 #define a_executable 0x80
-#define a_readonly (a_read+a_execute)
-#define a_all (a_write+a_read+a_execute)
+/*
+ * Define the bits used for the ref type.  See ipacked.h for more
+ * information about the possible values of the type byte.
+ */
 #define r_type_shift 8
 #define r_type_bits 6
 
-/* Define the attribute names for debugging printout. */
-/* Each entry has the form <mask, value, character>. */
-typedef struct attr_print_mask_s {
+/*
+ * Define the attribute names for debugging printout.
+ * Each entry has the form <mask, value, character>.
+ */
+typedef struct ref_attr_print_mask_s {
     ushort mask;
     ushort value;
     char print;
-} attr_print_mask;
+} ref_attr_print_mask_t;
 
-#define attr_print_flag(m,c)\
+#define _REF_ATTR_PRINT_FLAG(m,c)\
   {m,m,c},{m,0,'-'}
-#define attr_print_space(v,c)\
+#define _REF_ATTR_PRINT_SPACE(v,c)\
   {((1<<r_space_bits)-1)<<r_space_shift,v,c}
-#define attr_print_masks\
-  attr_print_flag(l_mark,'m'),\
-  attr_print_flag(l_new,'n'),\
-  attr_print_space(avm_foreign,'F'),\
-  attr_print_space(avm_system,'S'),\
-  attr_print_space(avm_global,'G'),\
-  attr_print_space(avm_local,'L'),\
-  attr_print_flag(a_write,'w'),\
-  attr_print_flag(a_read,'r'),\
-  attr_print_flag(a_execute,'x'),\
-  attr_print_flag(a_executable,'e'),\
-  attr_print_flag(0x4000,'?'),\
-  attr_print_flag(0x8000,'?')
+#define REF_ATTR_PRINT_MASKS\
+  _REF_ATTR_PRINT_FLAG(l_mark,'m'),\
+  _REF_ATTR_PRINT_FLAG(l_new,'n'),\
+  _REF_ATTR_PRINT_SPACE(avm_foreign,'F'),\
+  _REF_ATTR_PRINT_SPACE(avm_system,'S'),\
+  _REF_ATTR_PRINT_SPACE(avm_global,'G'),\
+  _REF_ATTR_PRINT_SPACE(avm_local,'L'),\
+  _REF_ATTR_PRINT_FLAG(a_write,'w'),\
+  _REF_ATTR_PRINT_FLAG(a_read,'r'),\
+  _REF_ATTR_PRINT_FLAG(a_execute,'x'),\
+  _REF_ATTR_PRINT_FLAG(a_executable,'e'),\
+  _REF_ATTR_PRINT_FLAG(0x4000,'?'),\
+  _REF_ATTR_PRINT_FLAG(0x8000,'?')
 
 /* Abstract types */
 typedef struct dict_s dict;
@@ -321,7 +361,7 @@ typedef struct obj_header_s obj_header_t;
 #  define i_ctx_t_DEFINED
 typedef struct gs_context_state_s i_ctx_t;
 #endif
-typedef int (*op_proc_t)(P1(i_ctx_t *i_ctx_p));
+typedef int (*op_proc_t)(i_ctx_t *i_ctx_p);
 /* real_opproc is a holdover.... */
 #define real_opproc(pref) ((pref)->value.opproc)
 
@@ -331,6 +371,7 @@ typedef int (*op_proc_t)(P1(i_ctx_t *i_ctx_p));
  * the type_attrs member must be the first one in the ref structure.
  */
 struct tas_s {
+/* type_attrs is a single element for fast dispatching in the interpreter */
     ushort type_attrs;
     ushort rsize;
 };
@@ -338,73 +379,6 @@ struct ref_s {
 
     struct tas_s tas;
 
-#define r_size(rp) ((rp)->tas.rsize)
-#define r_inc_size(rp,inc) ((rp)->tas.rsize += (inc))
-#define r_dec_size(rp,dec) ((rp)->tas.rsize -= (dec))
-#define r_set_size(rp,siz) ((rp)->tas.rsize = (siz))
-/* type_attrs is a single element for fast dispatching in the interpreter */
-#if r_type_shift == 8
-#  if arch_is_big_endian
-#    define r_type(rp) (((const byte *)&((rp)->tas.type_attrs))[sizeof(ushort)-2])
-#  else
-#    define r_type(rp) (((const byte *)&((rp)->tas.type_attrs))[1])
-#  endif
-#  define r_has_type(rp,typ) (r_type(rp) == (typ))
-#else
-#  define r_type(rp) ((rp)->tas.type_attrs >> r_type_shift)
-#  define r_has_type(rp,typ) r_has_type_attrs(rp,typ,0)		/* see below */
-#endif
-/* A special macro for testing arrayhood. */
-#define r_is_array(rp) _r_has_masked_type_attrs(rp,t_array,_t_array_span,0)
-#define r_set_type(rp,typ) ((rp)->tas.type_attrs = (typ) << r_type_shift)
-#define r_btype(rp)\
- ((rp)->tas.type_attrs >= (t_next_index << r_type_shift) ?\
-  t_operator : r_type(rp))
-#define r_type_xe_shift (r_type_shift - 2)
-#define type_xe_(tas) ((tas) >> r_type_xe_shift)	/* internal use only */
-/*
- * The r_type_xe macro is used in (and only in) the main interpreter loop,
- * where its rp operand may be a ref_packed, not necessarily aligned as
- * strictly as a full-size ref.  The DEC C compiler, and possibly others,
- * may compile code assuming that rp is ref-aligned.  Therefore, we
- * explicitly cast the pointer to a less-strictly-aligned type.
- * In order to convince the compiler, we have to do the cast before
- * indexing into the structure.
- */
-#define r_type_xe(rp)\
-  type_xe_(((const ushort *)(rp))[offset_of(ref, tas.type_attrs) / sizeof(ushort)])
-#define type_xe_value(t,xe) type_xe_(((t) << r_type_shift) + (xe))
-#define r_type_attrs(rp) ((rp)->tas.type_attrs)		/* reading only */
-#define r_has_attrs(rp,mask) !(~r_type_attrs(rp) & (mask))
-#define r_has_masked_attrs(rp,attrs,mask)\
-  ((r_type_attrs(rp) & (mask)) == (attrs))
-#define r_has_attr(rp,mask1)		/* optimize 1-bit case */\
-   (r_type_attrs(rp) & (mask1))
-/* The following macro is not for external use. */
-#define _r_has_masked_type_attrs(rp,typ,tspan,mask)\
- (((rp)->tas.type_attrs &\
-   ((((1 << r_type_bits) - (tspan)) << r_type_shift) + (mask))) ==\
-  (((typ) << r_type_shift) + (mask)))
-#define r_has_type_attrs(rp,typ,mask)\
-  _r_has_masked_type_attrs(rp,typ,1,mask)
-/* A special macro for testing procedurehood. */
-#define r_is_proc(rp)\
-  _r_has_masked_type_attrs(rp,t_array,_t_array_span,a_execute+a_executable)
-#define r_set_attrs(rp,mask) ((rp)->tas.type_attrs |= (mask))
-#define r_clear_attrs(rp,mask) ((rp)->tas.type_attrs &= ~(mask))
-#define r_store_attrs(rp,mask,attrs)\
-  ((rp)->tas.type_attrs = ((rp)->tas.type_attrs & ~(mask)) | (attrs))
-#define r_copy_attrs(rp,mask,sp)\
-  r_store_attrs(rp,mask,(sp)->tas.type_attrs & (mask))
-#define r_set_type_attrs(rp,typ,mask)\
-  ((rp)->tas.type_attrs = ((typ) << r_type_shift) + (mask))
-/* Macros for t_[a]struct objects. */
-#define r_is_struct(rp) _r_has_masked_type_attrs(rp,t_struct,_t_struct_span,0)
-#define r_has_stype(rp,mem,styp)\
-  (r_is_struct(rp) && gs_object_type(mem, (rp)->value.pstruct) == &styp)
-#define r_ptr(rp,typ) ((typ *)((rp)->value.pstruct))
-#define r_set_ptr(rp,ptr) ((rp)->value.pstruct = (obj_header_t *)(ptr))
-
     union v {			/* name the union to keep gdb happy */
 	long intval;
 	ushort boolval;
@@ -432,6 +406,143 @@ struct ref_s {
     } value;
 };
 
+/* ---------------- Private ref macros ---------------- */
+
+/*
+ * Test whether a ref has a type within a given span, and also has all of a
+ * given set of attributes.
+ */
+#define _REF_HAS_MASKED_TYPE_ATTRS(rp,typ,tspan,mask)\
+ (((rp)->tas.type_attrs &\
+   ((((1 << r_type_bits) - (tspan)) << r_type_shift) + (mask))) ==\
+  (((typ) << r_type_shift) + (mask)))
+
+/* ---------------- Public ref macros ---------------- */
+
+/*
+ * All of these macros take an argument "rp" which is a pointer to a ref.
+ * Unless otherwise specified, they only apply to full-size (not packed)
+ * refs.
+ */
+
+/*
+ * Read, set, increment, and decrement the size field of a ref.
+ */
+#define r_size(rp) ((rp)->tas.rsize)
+#define r_inc_size(rp,inc) ((rp)->tas.rsize += (inc))
+#define r_dec_size(rp,dec) ((rp)->tas.rsize -= (dec))
+#define r_set_size(rp,siz) ((rp)->tas.rsize = (siz))
+
+/*
+ * Get the type of a ref; test whether a ref has a given type.  The
+ * difference between r_type and r_btype is that for refs with types greater
+ * than or equal to t_next_index, r_type returns that type, but r_btype
+ * returns t_operator (since those types just encode specific operators for
+ * faster dispatch in the interpreter -- see interp.h for more information).
+ */
+#if r_type_shift == 8
+#  if arch_is_big_endian
+#    define r_type(rp) (((const byte *)&((rp)->tas.type_attrs))[sizeof(ushort)-2])
+#  else
+#    define r_type(rp) (((const byte *)&((rp)->tas.type_attrs))[1])
+#  endif
+#  define r_has_type(rp,typ) (r_type(rp) == (typ))
+#else
+#  define r_type(rp) ((rp)->tas.type_attrs >> r_type_shift)
+#  define r_has_type(rp,typ) r_has_type_attrs(rp,typ,0)		/* see below */
+#endif
+#define r_btype(rp)\
+ ((rp)->tas.type_attrs >= (t_next_index << r_type_shift) ?\
+  t_operator : r_type(rp))
+
+/*
+ * Test whether a ref is an array, or a procedure, or a(n) [a]struct.
+ */
+#define r_is_array(rp)\
+  _REF_HAS_MASKED_TYPE_ATTRS(rp,t_array,_REF_T_ARRAY_SPAN,0)
+#define r_is_proc(rp)\
+  _REF_HAS_MASKED_TYPE_ATTRS(rp,t_array,_REF_T_ARRAY_SPAN,a_execute+a_executable)
+#define r_is_struct(rp)\
+  _REF_HAS_MASKED_TYPE_ATTRS(rp,t_struct,_REF_T_STRUCT_SPAN,0)
+
+/*
+ * Test whether a ref is a struct or astruct with a specific structure type
+ * (GC descriptor).
+ */
+#define r_has_stype(rp,mem,styp)\
+  (r_is_struct(rp) && gs_object_type(mem, (rp)->value.pstruct) == &styp)
+
+/*
+ * Set the type of a ref.  This is only used in a few very special places.
+ * Normally the type of a ref is set when the ref is created (by one of
+ * the make_xxx macros in store.h) and never changed.
+ */
+#define r_set_type(rp,typ) ((rp)->tas.type_attrs = (typ) << r_type_shift)
+
+/*
+ * Get, test, or set the type and attributes of a ref together as a single
+ * value.  This too is only used in a few special places.
+ */
+#define r_type_attrs(rp) ((rp)->tas.type_attrs)		/* reading only */
+#define r_has_type_attrs(rp,typ,mask)\
+  _REF_HAS_MASKED_TYPE_ATTRS(rp,typ,1,mask)
+#define r_set_type_attrs(rp,typ,mask)\
+  ((rp)->tas.type_attrs = ((typ) << r_type_shift) + (mask))
+
+/*
+ * Get the combined type, a_executable, and a_execute bits of a ref,
+ * for fast dispatching in the interpreter.
+ */
+/*
+ * The r_type_xe macro is used in (and only in) the main interpreter loop,
+ * where its rp operand may be a ref_packed, not necessarily aligned as
+ * strictly as a full-size ref.  The DEC C compiler, and possibly others,
+ * may compile code assuming that rp is ref-aligned.  Therefore, we
+ * explicitly cast the pointer to a less-strictly-aligned type.
+ * In order to convince the compiler, we have to do the cast before
+ * indexing into the structure.
+ */
+#define _REF_TYPE_XE_SHIFT (r_type_shift - 2)
+#define _REF_TAS_TYPE_XE(tas) ((tas) >> _REF_TYPE_XE_SHIFT)
+#define r_type_xe(rp)\
+  _REF_TAS_TYPE_XE(((const ushort *)(rp))[offset_of(ref, tas.type_attrs) / sizeof(ushort)])
+#define type_xe_value(typ,xe) _REF_TAS_TYPE_XE(((typ) << r_type_shift) + (xe))
+
+/*
+ * Test whether a ref has a given attribute, or all the given attributes.
+ */
+#define r_has_attr(rp,mask1)		/* optimize 1-bit case */\
+   (r_type_attrs(rp) & (mask1))
+#define r_has_attrs(rp,mask) !(~r_type_attrs(rp) & (mask))
+
+/*
+ * Test whether those attributes of a ref selected by a mask have a
+ * given value.
+ */
+#define r_has_masked_attrs(rp,attrs,mask)\
+  ((r_type_attrs(rp) & (mask)) == (attrs))
+
+/*
+ * Set, clear, store, or copy the attributes of a ref.  These are rarely
+ * needed.  Note that, unfortunately, the attrs and mask parameters of
+ * r_store_attrs are reversed from r_has_masked_attrs.
+ */
+#define r_set_attrs(rp,mask) ((rp)->tas.type_attrs |= (mask))
+#define r_clear_attrs(rp,mask) ((rp)->tas.type_attrs &= ~(mask))
+#define r_store_attrs(rp,mask,attrs)\
+  ((rp)->tas.type_attrs = ((rp)->tas.type_attrs & ~(mask)) | (attrs))
+#define r_copy_attrs(rp,mask,sp)\
+  r_store_attrs(rp,mask,(sp)->tas.type_attrs & (mask))
+
+/*
+ * Get or set the pointer field of a struct or astruct ref.  The typ
+ * argument of r_ptr is the (C) type of the structure.
+ */
+#define r_ptr(rp,typ) ((typ *)((rp)->value.pstruct))
+#define r_set_ptr(rp,ptr) ((rp)->value.pstruct = (obj_header_t *)(ptr))
+
+/* ---------------- End of ref macros ---------------- */
+
 /* Define data for initializing an empty array or string. */
 #define empty_ref_data(type, attrs)\
   { /*tas*/ { /*type_attrs*/ ((type) << r_type_shift) | (attrs),\
@@ -441,9 +552,9 @@ struct ref_s {
 #define arch_sizeof_ref sizeof(ref)
 /* Define the required alignment for refs. */
 /* We assume all alignment values are powers of 2. */
-#define arch_align_ref_mod\
- (((arch_align_long_mod - 1) | (arch_align_float_mod - 1) |\
-   (arch_align_ptr_mod - 1)) + 1)
+#define ARCH_ALIGN_REF_MOD\
+ (((ARCH_ALIGN_LONG_MOD - 1) | (ARCH_ALIGN_FLOAT_MOD - 1) |\
+   (ARCH_ALIGN_PTR_MOD - 1)) + 1)
 
 /* Define the maximum size of an array or a string. */
 /* The maximum array size is determined by the fact that */
