@@ -1,23 +1,19 @@
-/* Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: siinterp.c,v 1.2.6.1 2002/02/22 19:46:00 ray Exp $ */
+/* $Id: siinterp.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Image interpolation filter */
 #include "memory_.h"
-#include <assert.h>
 #include "gxfixed.h"		/* for gxdda.h */
 #include "gxdda.h"
 #include "gxfrac.h"
@@ -73,7 +69,7 @@ gs_private_st_ptrs2(st_IIEncode_state, stream_IIEncode_state,
     prev, cur);
 
 /* Forward references */
-private void s_IIEncode_release(P1(stream_state * st));
+private void s_IIEncode_release(stream_state * st);
 
 /* Initialize the filter. */
 private int
