@@ -1,20 +1,17 @@
-/* Copyright (C) 2003 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: ttfmain.c,v 1.32 2004/11/23 12:18:29 igor Exp $ */
+/* $Id: ttfmain.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* A Free Type interface adapter. */
 /* Uses code fragments from the FreeType project. */
 
@@ -31,7 +28,7 @@
 private const bool skip_instructions = 0; /* Debug purpose only. */
 
 typedef struct { 
-    Fixed a, b, c, d, tx, ty;
+    TT_Fixed a, b, c, d, tx, ty;
 } FixMatrix;
 
 struct ttfSubGlyphUsage_s { 
@@ -43,7 +40,7 @@ struct ttfSubGlyphUsage_s {
 
 /*------------------------------------------------------------------- */
 
-private Fixed AVE(F26Dot6 a, F26Dot6 b)
+private TT_Fixed AVE(F26Dot6 a, F26Dot6 b)
 {   return (a + b) / 2;
 }
 
@@ -55,7 +52,7 @@ private F26Dot6 floatToF26Dot6(float a)
 {   return (F26Dot6)(a * (1 << 6) + 0.5);
 }
 
-private Fixed floatToF16Dot16(float a)
+private TT_Fixed floatToF16Dot16(float a)
 {   return (F26Dot6)(a * (1 << 16) + 0.5);
 }
 
@@ -184,7 +181,7 @@ void ttfInterpreter__release(ttfInterpreter **ptti)
 
 void ttfFont__init(ttfFont *this, ttfMemory *mem, 
 		    void (*DebugRepaint)(ttfFont *),
-		    void (*DebugPrint)(ttfFont *, const char *s, ...))
+		    int (*DebugPrint)(ttfFont *, const char *s, ...))
 {
     memset(this, 0, sizeof(*this));
     this->DebugRepaint = DebugRepaint;
@@ -323,6 +320,8 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
     code = Instance_Init(this->inst);
     if (code == TT_Err_Out_Of_Memory)
 	return fMemoryError;
+    if (code >= TT_Err_Invalid_Opcode && code <= TT_Err_Invalid_Displacement)
+	return fBadInstruction;
     if (code)
 	return fBadFontData;
     I.z = this->inst;
@@ -339,6 +338,8 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
 	return fPatented;
     if (code == TT_Err_Out_Of_Memory)
 	return fMemoryError;
+    if (code >= TT_Err_Invalid_Opcode && code <= TT_Err_Invalid_Displacement)
+	return fBadInstruction;
     if (code)
 	return fBadFontData;
     return code;
@@ -590,15 +591,15 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
             }
 	    m.b = m.c = m.tx = m.ty = 0;
 	    if (flags & WE_HAVE_A_SCALE)
-		m.a = m.d = (Fixed)ttfReader__Short(r) << 2;
+		m.a = m.d = (TT_Fixed)ttfReader__Short(r) << 2;
 	    else if (flags & WE_HAVE_AN_X_AND_Y_SCALE) {
-		m.a = (Fixed)ttfReader__Short(r) << 2;
-		m.d = (Fixed)ttfReader__Short(r) << 2;
+		m.a = (TT_Fixed)ttfReader__Short(r) << 2;
+		m.d = (TT_Fixed)ttfReader__Short(r) << 2;
 	    } else if (flags & WE_HAVE_A_TWO_BY_TWO) {
-		m.a = (Fixed)ttfReader__Short(r)<<2;
-		m.b = (Fixed)ttfReader__Short(r)<<2;
-		m.c = (Fixed)ttfReader__Short(r)<<2;
-		m.d = (Fixed)ttfReader__Short(r)<<2;
+		m.a = (TT_Fixed)ttfReader__Short(r)<<2;
+		m.b = (TT_Fixed)ttfReader__Short(r)<<2;
+		m.c = (TT_Fixed)ttfReader__Short(r)<<2;
+		m.d = (TT_Fixed)ttfReader__Short(r)<<2;
             } else 
 		m.a = m.d = 65536;
 	    e = &usage[nUsage];
@@ -622,7 +623,7 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
 	    ttfSubGlyphUsage *e = &usage[i];
 	    int j;
 	    TT_Error code;
-	    int nPointsStored = this->nPointsTotal, nContoursStored = this->nContoursTotal;
+	    int nPointsStored = gOutline->pointCount, nContoursStored = gOutline->contourCount;
 
 	    out.contourCount = 0;
 	    out.pointCount = 0;
