@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zimage.c,v 1.14 2004/08/04 19:36:13 stefan Exp $ */
+/* $Id: zimage.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Image operators */
 #include "math_.h"
 #include "memory_.h"
@@ -295,13 +292,16 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
 	}
 	*ep = *pp;
     }
-    if ((penum = gs_image_enum_alloc(imemory, "image_setup")) == 0)
+    /* Always place the image enumerator into local memory,
+       because pie may have local objects inherited from igs,
+       which may be local when the current allocation mode is global. 
+       Bug 688140. */
+    if ((penum = gs_image_enum_alloc(imemory_local, "image_setup")) == 0)
 	return_error(e_VMerror);
     code = gs_image_enum_init(penum, pie, (const gs_data_image_t *)pim, igs);
     if (code != 0) {		/* error, or empty image */
-	int code1 = gs_image_cleanup(penum);
+	int code1 = gs_image_cleanup_and_free_enum(penum);
 
-	ifree_object(penum, "image_setup");
 	if (code >= 0)		/* empty image */
 	    pop(npop);
 	if (code >= 0 && code1 < 0)
@@ -312,7 +312,7 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
     esp += inumpush - 1;
     make_int(ETOP_PLANE_INDEX(esp), 0);
     make_int(ETOP_NUM_SOURCES(esp), num_sources);
-    make_istruct(esp, 0, penum);
+    make_struct(esp, avm_local, penum);
     switch (r_type(sources)) {
 	case t_file:
 	    push_op_estack(image_file_continue);
@@ -551,10 +551,8 @@ image_cleanup(i_ctx_t *i_ctx_p)
 {
     es_ptr ep_top = esp + NUM_PUSH(EBOT_NUM_SOURCES(esp)->value.intval);
     gs_image_enum *penum = r_ptr(ep_top, gs_image_enum);
-    int code = gs_image_cleanup(penum);
-
-    ifree_object(penum, "image_cleanup");
-    return code;
+    
+    return gs_image_cleanup_and_free_enum(penum);
 }
 
 /* ------ Initialization procedure ------ */
