@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 1995, 1997, 1998, 1999, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: ziodev.c,v 1.6.2.1 2002/02/22 19:46:00 ray Exp $ */
+/* $Id: ziodev.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Standard IODevice implementation */
 #include "memory_.h"
 #include "stdio_.h"
@@ -33,6 +30,7 @@
 #include "files.h"
 #include "scanchar.h"		/* for char_EOL */
 #include "store.h"
+#include "ierrors.h"
 
 /* Import the dtype of the stdio IODevices. */
 extern const char iodev_dtype_stdio[];
@@ -56,12 +54,12 @@ extern const char iodev_dtype_stdio[];
  */
 
 #define LINEEDIT_BUF_SIZE 20	/* initial size, not fixed size */
-private iodev_proc_open_device(lineedit_open);
+/*private iodev_proc_open_device(lineedit_open);*/ /* no longer used */
 const gx_io_device gs_iodev_lineedit =
     iodev_special("%lineedit%", iodev_no_init, iodev_no_open_device);
 
 #define STATEMENTEDIT_BUF_SIZE 50	/* initial size, not fixed size */
-private iodev_proc_open_device(statementedit_open);
+/*private iodev_proc_open_device(statementedit_open);*/ /* no longer used */
 const gx_io_device gs_iodev_statementedit =
     iodev_special("%statementedit%", iodev_no_init, iodev_no_open_device);
 
@@ -244,11 +242,14 @@ rd:
 	    }
 	}
 	buf->data[count++] = char_EOL;
+	s_init(ts, NULL);
 	sread_string(ts, buf->data, count);
 sc:
 	scanner_state_init_check(&state, false, true);
 	code = scan_token(i_ctx_p, ts, &ignore_value, &state);
 	ref_stack_pop_to(&o_stack, depth);
+	if (code < 0)
+	    code = scan_EOF;	/* stop on scanner error */
 	switch (code) {
 	    case 0:		/* read a token */
 	    case scan_BOS:
