@@ -1,20 +1,17 @@
-/* Copyright (C) 2000-2002 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: ztrans.c,v 1.23 2005/03/14 18:08:37 dan Exp $ */
+/* $Id: ztrans.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Transparency operators */
 #include "string_.h"
 #include "memory_.h"
@@ -25,6 +22,7 @@
 #include "gsipar3x.h"
 #include "gstrans.h"
 #include "gxiparam.h"		/* for image enumerator */
+#include "gxcspace.h"
 #include "idict.h"
 #include "idparam.h"
 #include "ifunc.h"
@@ -243,10 +241,10 @@ zendtransparencygroup(i_ctx_t *i_ctx_p)
     return gs_end_transparency_group(igs);
 }
 
-/* <paramdict> <llx> <lly> <urx> <ury> .begintransparencymask - */
+/* <paramdict> <llx> <lly> <urx> <ury> .begintransparencymaskgroup - */
 private int tf_using_function(floatp, float *, void *);
 private int
-zbegintransparencymask(i_ctx_t *i_ctx_p)
+zbegintransparencymaskgroup(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     os_ptr dop = op - 4;
@@ -265,12 +263,17 @@ zbegintransparencymask(i_ctx_t *i_ctx_p)
     if ((code = enum_param(imemory, pparam, subtype_names)) < 0)
 	return code;
     gs_trans_mask_params_init(&params, code);
-    if ((code = dict_floats_param(imemory, dop, "Background", 1,
+    if ((code = dict_floats_param(imemory, dop, "Background", 
+		    cs_num_components(gs_currentcolorspace(i_ctx_p->pgs)),
 				  params.Background, NULL)) < 0
 	)
 	return code;
     else if (code > 0)
-	params.has_Background = true;
+	params.Background_components = code;
+    if ((code = dict_floats_param(imemory, dop, "GrayBackground", 
+		    1, &params.GrayBackground, NULL)) < 0
+	)
+	return code;
     if (dict_find_string(dop, "TransferFunction", &pparam) >0) {
 	gs_function_t *pfn = ref_function(pparam);
 
@@ -282,13 +285,28 @@ zbegintransparencymask(i_ctx_t *i_ctx_p)
     code = rect_param(&bbox, op);
     if (code < 0)
 	return code;
-    code = gs_begin_transparency_mask(igs, &params, &bbox);
+    code = gs_begin_transparency_mask(igs, &params, &bbox, false);
     if (code < 0)
 	return code;
     pop(5);
     return code;
 }
 
+/* - .begintransparencymaskimage - */
+private int
+zbegintransparencymaskimage(i_ctx_t *i_ctx_p)
+{
+    gs_transparency_mask_params_t params;
+    gs_rect bbox = { { 0, 0} , { 1, 1} };
+    int code;
+
+    gs_trans_mask_params_init(&params, TRANSPARENCY_MASK_Luminosity);
+    code = gs_begin_transparency_mask(igs, &params, &bbox, true);
+    if (code < 0)
+	return code;
+    return code;
+}
+
 /* Implement the TransferFunction using a Function. */
 private int
 tf_using_function(floatp in_val, float *out, void *proc_data)
@@ -347,7 +365,7 @@ zimage3x(i_ctx_t *i_ctx_p)
 	return_error(e_rangecheck);
     if ((code = pixel_image_params(i_ctx_p, pDataDict,
 				   (gs_pixel_image_t *)&image, &ip_data,
-				   12, false)) < 0 ||
+				   16, false)) < 0 ||
 	(code = dict_int_param(pDataDict, "ImageType", 1, 1, 0, &ignored)) < 0
 	)
 	return code;
@@ -382,7 +400,7 @@ image_params *pip_data, const char *dict_name,
     if (dict_find_string(op, dict_name, &pMaskDict) <= 0)
 	return 1;
     if ((mcode = code = data_image_params(mem, pMaskDict, &pixm->MaskDict,
-					  &ip_mask, false, 1, 12, false)) < 0 ||
+					  &ip_mask, false, 1, 16, false)) < 0 ||
 	(code = dict_int_param(pMaskDict, "ImageType", 1, 1, 0, &ignored)) < 0 ||
 	(code = dict_int_param(pMaskDict, "InterleaveType", 1, 3, -1,
 			       &pixm->InterleaveType)) < 0 ||
@@ -452,7 +470,8 @@ const op_def ztrans2_op_defs[] = {
     {"5.begintransparencygroup", zbegintransparencygroup},
     {"0.discardtransparencygroup", zdiscardtransparencygroup},
     {"0.endtransparencygroup", zendtransparencygroup},
-    {"5.begintransparencymask", zbegintransparencymask},
+    {"5.begintransparencymaskgroup", zbegintransparencymaskgroup},
+    {"5.begintransparencymaskimage", zbegintransparencymaskimage},
     {"0.discardtransparencymask", zdiscardtransparencymask},
     {"1.endtransparencymask", zendtransparencymask},
     {"1.inittransparencymask", zinittransparencymask},
