@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 1993, 1994, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 artofcode LLC.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zvmem2.c,v 1.3.2.1 2002/02/22 19:46:01 ray Exp $ */
+/* $Id: zvmem2.c 6651 2006-03-13 16:18:19Z stefan $ */
 /* Level 2 "Virtual memory" operators */
 #include "ghost.h"
 #include "oper.h"
@@ -25,8 +22,8 @@
 #include "store.h"
 
 /* Garbage collector control parameters. */
-#define DEFAULT_VM_THRESHOLD_SMALL 20000
-#define DEFAULT_VM_THRESHOLD_LARGE 250000
+#define DEFAULT_VM_THRESHOLD_SMALL 100000
+#define DEFAULT_VM_THRESHOLD_LARGE 1000000
 #define MIN_VM_THRESHOLD 1
 #define MAX_VM_THRESHOLD max_long
 
@@ -78,8 +75,6 @@ zgcheck(i_ctx_t *i_ctx_p)
 int
 set_vm_threshold(i_ctx_t *i_ctx_p, long val)
 {
-    gs_memory_gc_status_t stat;
-
     if (val < -1)
 	return_error(e_rangecheck);
     else if (val == -1)
@@ -89,12 +84,8 @@ set_vm_threshold(i_ctx_t *i_ctx_p, long val)
 	val = MIN_VM_THRESHOLD;
     else if (val > MAX_VM_THRESHOLD)
 	val = MAX_VM_THRESHOLD;
-    gs_memory_gc_status(idmemory->space_global, &stat);
-    stat.vm_threshold = val;
-    gs_memory_set_gc_status(idmemory->space_global, &stat);
-    gs_memory_gc_status(idmemory->space_local, &stat);
-    stat.vm_threshold = val;
-    gs_memory_set_gc_status(idmemory->space_local, &stat);
+    gs_memory_set_vm_threshold(idmemory->space_global, val);
+    gs_memory_set_vm_threshold(idmemory->space_local, val);
     return 0;
 }
 
@@ -102,17 +93,9 @@ int
 set_vm_reclaim(i_ctx_t *i_ctx_p, long val)
 {
     if (val >= -2 && val <= 0) {
-	gs_memory_gc_status_t stat;
-
-	gs_memory_gc_status(idmemory->space_system, &stat);
-	stat.enabled = val >= -1;
-	gs_memory_set_gc_status(idmemory->space_system, &stat);
-	gs_memory_gc_status(idmemory->space_global, &stat);
-	stat.enabled = val >= -1;
-	gs_memory_set_gc_status(idmemory->space_global, &stat);
-	gs_memory_gc_status(idmemory->space_local, &stat);
-	stat.enabled = val == 0;
-	gs_memory_set_gc_status(idmemory->space_local, &stat);
+	gs_memory_set_vm_reclaim(idmemory->space_system, (val >= -1));
+	gs_memory_set_vm_reclaim(idmemory->space_global, (val >= -1));
+	gs_memory_set_vm_reclaim(idmemory->space_local, (val == 0));
 	return 0;
     } else
 	return_error(e_rangecheck);
